package musr.client.bestfit;

import musr.client.datacolumn.*;
import javax.swing.JOptionPane;

/** Used to fit to the form f(x) = a*exp(b*x)
 **/
public class ExponentialFit extends AbstractFit
{
    //Stores best-fit values, for form f(x) = a*exp(b*x)
    private double a=0;
    private double b=0;
    
    //Fit is perfrormed by finding right-hand vector (exp(a) b) such that alpha * (ln(a) b) = beta
    public static double[][] alpha;
    public static double[] beta;
    
    public static void main(String[] args)
    {
        ConcreteDataColumn xValues = new ConcreteDataColumn();
        ConcreteDataColumn yValues = new ConcreteDataColumn();
        ConcreteDataColumn yPlusErrors = new ConcreteDataColumn();
        ConcreteDataColumn yMinusErrors = new ConcreteDataColumn();
        for(int i=1; i<100; i++)
        {
            xValues.addElement(new Double(i));
            yValues.addElement((new Double(4*Math.exp(.2*i))));
            yPlusErrors.addElement(new Double(.001));
            yMinusErrors.addElement(new Double(.001));
        }
        MultiColumn yColumn = new MultiColumn(yValues, yPlusErrors, yMinusErrors);
        
        ExponentialFit testFit = new ExponentialFit(xValues, yColumn);
        testFit.fit();
        System.out.println("result:");
        System.out.println(testFit.toString());
    }
    
    //Constructor most likely to be used
    public ExponentialFit(DataColumn xColumn, MultiColumn yColumn)
    {
        double[] xData = dataToArray(xColumn);
        double[] yData = dataToArray(yColumn);
        double[] yErrorSquared = null;      //Initialize to keep the compiler happy
        try
        {
            yErrorSquared = errorSquaredToArray(yColumn.getErrorMinusColumn(), yColumn.getErrorPlusColumn());
        }
        //Exception thrown if there is a problem with error bars; if this happens just weight each value equally
        //(set each error bar size to 1.0)
        catch(ErrorBarException e)
        {
            JOptionPane.showMessageDialog(null, e.getMessage());
            yErrorSquared = new double[yColumn.getSize()];
            for(int i=0; i<yErrorSquared.length; i++)
                yErrorSquared[i] = 1.0;
        }
        finally
        {
            makeAlpha(xData, yData, yErrorSquared);
            makeBeta(xData, yData, yErrorSquared);
        }
    }
    
    //Constructor to be used if for some reason the values for alpha and beta are already known
    public ExponentialFit(double[][] newAlpha, double[] newBeta)
    {
        alpha = newAlpha;
        beta = newBeta;
    }
    
    public String toString()
    {
        return a + "*exp(" + b + "*x)";
    }
    
    //Solves the 2x2 linear system to determine the values of a and b
    //Since the coefficient vector is actually ( ln(a)  b ) it is necessary to take the base-e exponenet of the first term
    public void fit()
    {
        double det = getDeterminant();
        System.out.println("det alpha = " + det);
        a = Math.exp((alpha[1][1]*beta[0]-alpha[0][1]*beta[1])/det);
        b = (alpha[0][0]*beta[1]-alpha[1][0]*beta[0])/det;
    }
    
    private void makeAlpha(double[] xData, double[] yData, double[] yErrorSquared)
    {
        //Only two parameters (a and b) so this least-squares fit case is almost trivially easy
        alpha = new double[2][2];
        //Bad programming practice - but here sensible - to hard-code values for elements of alpha
        for(int i=0; i<yData.length; i++)
        {
            alpha[0][0] += yData[i]/yErrorSquared[i];
            alpha[1][0] += xData[i]*yData[i]/yErrorSquared[i];
            alpha[1][1] += xData[i]*xData[i]*yData[i]/yErrorSquared[i];
        }
        alpha[0][1] = alpha [1][0];
        System.out.println("alpha completed:");
        printMatrix(this.alpha);
    }
    
    private void makeBeta(double[] xData, double[] yData, double[] yErrorSquared)
    {
        //Once again, since there are only two parameters and their positions etc are known, it makes sense
        //to hard-code things in as necessary
        beta = new double[2];
        for(int i=0; i<beta.length; i++)
        {
            beta[0] += Math.log(yData[i])*yData[i]/yErrorSquared[i];
            beta[1] += xData[i]*Math.log(yData[i])*yData[i]/yErrorSquared[i];
        }
        System.out.println("beta completed:");
        printArray(this.beta);
    }
    
    //Returns the determinant of the matrix representing alpha
    private double getDeterminant()
    {
        return alpha[0][0]*alpha[1][1] - alpha[1][0]*alpha[0][1];
    }
}
