package musr.client.bestfit;

import musr.client.datacolumn.*;
import java.text.DecimalFormat;
import javax.swing.JOptionPane;

public class PolynomialFit extends AbstractFit
{
    //Following arrays store values to solve equation of form Ax=b for x,
    //where A is an NxN real matrix and x and b are real vectors of length N.
    private double[][] a;
    private double[] b;
    
    //holds the result of the calculations
    private double[] coefficient;
    
    //holds the chi-squared estimator of goodness of fit
    private double chiSquared;
    //holds the Q value, the probability that the real value for chi-squared exceeds the value of chiSquared
    private double qValue;
    //holds the variances in the fitted parameters
    private double[] variance;
    
    //Holds the degree of the polynomial to which the equation is being fitted
    private int degree;
    
    //Holds the data and the errors after processing
    double[] xData;
    double[] yData;
    double[] yErrorSquared;
    
    public static void main(String[] args)
    {
        ConcreteDataColumn xColumn = new ConcreteDataColumn();
        MultiColumn yColumn;
        ConcreteDataColumn yValues = new ConcreteDataColumn();
        ConcreteDataColumn yPlusErrors = new ConcreteDataColumn();
        ConcreteDataColumn yMinusErrors = new ConcreteDataColumn();
        
        for(int i=1; i<=10; i++)
        {
            xColumn.addElement(new Double(i));
            yValues.addElement(new Double(0.00005*Math.pow(i, 2) - .0001*i + .000098 + Math.random()/1000000));
            yPlusErrors.addElement(new Double(i+0.01));
            yMinusErrors.addElement(new Double(i+0.01));
        }
        /*for(int i=11; i<=20; i++)
        {
                        
            xColumn.addElement(new Double(-1.0*i));
            yValues.addElement(new Double(14*Math.pow(i, 2)) - 5*i + 8);
            yPlusErrors.addElement(new Double(i+0.01));
            yMinusErrors.addElement(new Double(i + 0.01));
        }*/

        yColumn = new MultiColumn(yValues, yPlusErrors, yMinusErrors);
        //System.out.println("Variables initialized.");
        PolynomialFit test = new PolynomialFit(2, xColumn, yColumn);
        try
        {
            test.fit();
        }
        catch(DataNotFittableException e)
        {
            System.out.println("CATASTROPHIC ERROR: DATA NOT FITTABLE! AAAAAAAARGH!");
            System.out.println("This computer will now explode. Oh noes :(.");
            System.exit(0);
        }
        double[] result = test.getCoefficients();
        System.out.println("FINAL RESULT:");
        System.out.println(test.toString());
        System.out.println("Contents of matrix a:");
        printMatrix(test.a);
        System.out.println("Contents of vector b:");
        printArray(test.b);
        System.out.println("Right-hand vector product Ax:");
        double[] output = new double[test.coefficient.length];
        double sum = 0;
        for(int i=0; i<output.length; i++)
        {
            sum = 0;
            for(int j=0; j<test.a.length; j++)
                sum += test.a[i][j]*test.coefficient[j];
            output[i] = sum;
        }
        printArray(output);
        System.out.println("Chi squared value = " + test.getChiSquared());
        System.out.println("Q-value = " + test.getQValue());
    }
    
     //Takes the data columns representing the x and y data and calls methods to make the matrix a and the vector b
    //Probably quite costly in terms of time to call this constructor so unecessary calls should be avoided
    public PolynomialFit(int newDegree, DataColumn xColumn, MultiColumn yColumn, int start, int end, boolean weighted)
    {
        if(degree>=0)
            degree = newDegree;
        else
        {
            System.out.println("An error has occured and a degree less than one was selected. Assuming linear fit.");
            degree = 1;
        }
        System.out.println("PolynomialFit:102");
        coefficient = null;    //initialize it - values will be put in once calculations have been done
        
        System.out.println("start = " + start);
        System.out.println("end = " + end);
        
        if(start>=end)
        {
            JOptionPane.showMessageDialog(null, "The selected range of x values has a starting value after its" +
                    " finishing value.\nThe default (full) range will be used for this fit.");
            start = 0;
            end = xColumn.getSize()-1;
        }
        System.out.println("PolynomialFit:115");
        double[] tempX = dataToArray(xColumn);
        double[] tempY = dataToArray(yColumn.getValueColumn());
        xData = new double[end-start+1];
        yData = new double[end-start+1];
        for(int i=start; i<=end; i++)
        {
            xData[i-start] = tempX[i];
            yData[i-start] = tempY[i];
        }
        System.out.println("PolynomialFit:118");
        chiSquared = 0;
        qValue = 0;
        variance = null;
              
        if(!weighted)
        {   System.out.println("PolynomialFit:124");
            yErrorSquared = new double[yColumn.getSize()];
            for(int i=0; i<yErrorSquared.length; i++)
                yErrorSquared[i] = 1.0;
            System.out.println("PolynomialFit:128");
            makeA();
            makeB();
        } 
        else
        {   System.out.println("PolynomialFit:133");
            try
            {   System.out.println("PolynomialFit:135 (try)");
                yErrorSquared = errorSquaredToArray(yColumn.getErrorPlusColumn(), yColumn.getErrorMinusColumn(), start, end);
                System.out.println("PolynomialFit:137");
            }
            catch(ErrorBarException e)
            {System.out.println("PolynomialFit:140 (catch)");
                //No error bars were found on the yData
                //Set all errors to 1.0 (that is, weight all the data points equally) and go with it
                JOptionPane.showMessageDialog(null, e.getMessage());
                yErrorSquared = new double[yColumn.getSize()];
                for(int i=0; i<yErrorSquared.length; i++)
                    yErrorSquared[i] = 1.0;
                System.out.println("PolynomialFit:147");
            }        
            finally
            {System.out.println("PolynomialFit:150 (finally)");
            /*System.out.println("Error squared values:");
            printArray(yErrorSquared);*/
            /*System.out.println("x data values:");
            printArray(xData);*/
            
            makeA();
            makeB();
            //System.out.println("Constructor call complete.");
            }
        }  
    }
    
    public PolynomialFit(int newDegree, DataColumn xColumn, MultiColumn yColumn, int start, int end)
    {
        this(newDegree, xColumn, yColumn, start, end, true);
    }
    
    public PolynomialFit(int newDegree, DataColumn xColumn, MultiColumn yColumn)
    {
        if(degree>=0)
            degree = newDegree;
        else
        {
            System.out.println("An error has occured and a degree less than one was selected. Assuming linear fit.");
            degree = 1;
        }
        
        coefficient = null;    //initialize it - values will be put in once calculations have been done
        
        xData = dataToArray(xColumn);
        yData = dataToArray(yColumn.getValueColumn());
        
        chiSquared = 0;
        qValue = 0;
        variance = null;
                
        try
        {
            yErrorSquared = errorSquaredToArray(yColumn.getErrorPlusColumn(), yColumn.getErrorMinusColumn());
            System.out.println("PolynomialFit:190");
        }
        catch(ErrorBarException e)
        {
            //No error bars were found on the yData
            //Set all errors to 1.0 (that is, weight all the data points equally) and go with it
            JOptionPane.showMessageDialog(null, e.getMessage());
            yErrorSquared = new double[yColumn.getSize()];
            for(int i=0; i<yErrorSquared.length; i++)
                yErrorSquared[i] = 1.0;
            System.out.println("PolynomialFit:200");
        }
        
        finally
        {
            /*System.out.println("Error squared values:");
            printArray(yErrorSquared);*/
            /*System.out.println("x data values:");
            printArray(xData);*/
            
            makeA();
            makeB();
            //System.out.println("Constructor call complete.");
        }
    }
    
    //In case (for some unknown reason) you already have A and b prepared - used in testing
    public PolynomialFit(int newDegree, double[][] newA, double[] newB)
    {
        if(degree>=0)
            degree = newDegree;
        else
        {
            System.out.println("An error has occured and a degree less than one was selected. Assuming linear fit.");
            degree = 1;
        }
        coefficient = null;    //initialize it - values will be put in once calculations have been done
        a = newA;
        b = newB;
    }
    
    //Prints out the polynomial best fit in a nice legible format
    public String toString()
    {
        //DecimalFormat formatter = new DecimalFormat("######.####");

        String result = "";
        
        System.out.println("Constant term in polynomial approximation: " + coefficient[0]);
        
        if(coefficient.length>0)
            result += format(coefficient[0]);
        
        if(coefficient.length>1)
        {
            if(coefficient[1]>0)
                result += " + " + format(coefficient[1]) + "*x";
            else if(coefficient[1]<0)
                result += " - " + format(Math.abs(coefficient[1])) + "*x";
        }
        
        for(int i=2; i<coefficient.length; i++)
        {
            if(coefficient[i]>0)
                result += " + " + format(coefficient[i]) + "*x^" + i;
            else if(coefficient[i]<0)
                result += " - " + format(Math.abs(coefficient[i])) + "*x^" + i;
        }
        //System.out.println("Putting data in string complete.");
        return result;
    }
    
    //returns the array of coefficients of the polynomial
    public double[] getCoefficients()
    {
        return this.coefficient;
    }
    
    //Performs the fitting operation and puts the results in the coefficients array
    public void fit() throws DataNotFittableException
    {
        try
        {
            this.coefficient = multiply(invert(a), b);
        }
        catch(DataNotFittableException e)
        {
            throw e;
        }
        //System.out.println("Fit performed.");
    }

    /**
     *Returns the value of the goodness-of-fit indicator chi-squared for the fit function
     **/
    public double getChiSquared()
    {
        if(chiSquared==0)   //chiSquared has not been calculated yet, so calculate it
            for(int i=0; i<xData.length; i++)
                chiSquared += Math.pow(yData[i]-evaluateFitAt(xData[i]), 2)/yErrorSquared[i];
        return chiSquared;
    }
    
    /**
     *Evalates the function returned by the best fit at the specified x
     **/
    public double evaluateFitAt(double x)
    {
        double result = coefficient[0];
        
        if(coefficient==null)   //fitting has not occured yet - so just return zero
        {
            System.out.println("Error: fit function not yet determined.");
            return 0.0;
        }
        
        for(int i=1; i<coefficient.length; i++)
            result += coefficient[i]*Math.pow(x, i);
        
        return result;
    }
    
    /**
     *Evaluates the fit at the x data point with the specified index
     **/
    public double evaluateFitAtIndex(int index)
    {
        return evaluateFitAt(xData[index]);
    }
    
    public double getQValue()
    {
        if(qValue==0)       //qValue has not been calculated yet, so calculate it
        {
           qValue = incompleteGamma((xData.length-coefficient.length)/2, 0.5*getChiSquared()) / 
                   incompleteGamma((xData.length-coefficient.length)/2, 0);
        }
        return qValue;
    }
    
    public double[] getVariance()
    {
        if(variance==null)    //covariance not yet initialized, so initialize it
        {
            variance = new double[coefficient.length];
            double[][] aInverse = new double[a.length][a.length];
            try
            {
                aInverse = invert(a);
            }
            catch(DataNotFittableException e)
            {
                JOptionPane.showMessageDialog(null, "The variance cannot be calculated as singular matrix was encountered.");
            }
            for(int i=0; i<variance.length; i++)
                variance[i] = aInverse[i][i];
        }
        return variance;
    }
    
    public double getVarianceAt(int index)
    {
        if(variance==null)
            return getVariance()[index];
        else
            return variance[index];
    }
    
    //Performs an LU decompostion the square matrix a using Crout's algorithm
    //WORKS PROPERLY
    private double[][] decomposeLU() throws DataNotFittableException
    {

        double[][] result = new double[a.length][a[0].length];      //Stores the upper triangular factor of a, and the components
                                                                    //of the lower triangular factor which lie below the diagonal
                                                                    //(the elements along the diagonal are all 1 and the elements
                                                                    //the diagonal are all zero).
        double sum;
        //System.out.println("Variables initialized in decomposeLU().");
        for(int i=0; i<result.length; i++)
            result[i][i] = 1.0;
        //System.out.println("a[0][0]=" + a[0][0]);
        for(int j=0; j<result.length; j++)
        {
            for(int i=0; i<=j; i++)
            {
                sum = 0;
                for(int k=0; k<i; k++)
                    sum += result[i][k]*result[k][j];
                /*if(sum==a[i][j])
                    System.out.println("sum==a[i][j] for i=" + i + ", j=" + j);*/
                result[i][j] = a[i][j]-sum;
                /*System.out.println("i=" + i + " j=" + j + " upper[i][j]=" + upper[i][j]);*/
            }
            for(int i=j+1; i<result.length; i++)
            {
                sum = 0;
                for(int k=0; k<j; k++)
                    sum += result[i][k]*result[k][j];
                result[i][j] = (a[i][j]-sum)/result[j][j];
                if(result[j][j]==0)
                {
                    System.out.println("Diagonal element at " + j + " in upper triangular matrix is zero.");
                    throw new DataNotFittableException();
                }
                /*System.out.println("i=" + i + " j=" + j + " lower[i][i]=" + lower[i][j]);*/
            }
            //System.out.println("Main loop completed for j=" + j + '.');
        }
        //System.out.println("Matrix decomposed:");
        //printMatrix(result);
        return result;
    }
    
    /*//Takes a packed matrix containing lower triangular unipotent matrix L and upper triangular matrix U such that A = LU
    //To solve Ax = LUx = b, first solves intermediate system Ly = b for y, then Ux = y for x, both by backsubstitution
    //Since Ly=b and Ux=y, LUx=Ax=b - therefore, x is the solution the the linear system
    //Returns x, the solution to the linear system given by Ax = b
    private double[] backSubstitute(double[][] packed)
    {
        double[] y = new double[packed.length]; //Holds solution of intermediate linear equation Ly=b
        double[] x = new double[packed.length]; //Holds solution to the final linear equation Ax=b
        double sum;                             //Holds summations in loops throughout method
        
        //Throughout the following backsubstitution algorithm, L is taken to have all ones along its main diagonal -
        //this precondition is able to be arbitrarily set (and implemented in decomposeLU()) as LU decomposition is not unique.
        printMatrix(packed);
        
        //First, solve for the terms of y:
        y[0] = b[0];
        sum = 0;
        for(int i=1; i<y.length; i++)
        {
            sum += packed[i][i-1]*y[i-1];
            /*sum = 0;
             for(int j=0; j<i; j++)
                sum += packed[i][j]*y[j];*/
            /*y[i] = b[i] - sum;
        }
        
        //Then, solve for the terms of x:
        x[x.length-1] = y[y.length-1]/packed[packed.length-1][packed.length-1];
        sum = 0;
        for(int i=x.length-2; i>=0; i--)
        {
            sum += packed[i+1][i]*x[i+1];
            /*sum = 0;
            for(int j=i+1; j<packed.length; j++)
                sum += packed[i][j]*x[j];*/
            /*x[i] = (y[i] - sum)/packed[i][i];
            if(packed[i][i]==0)
                System.out.println("Division by zero is occuring at line 239 for i=" + i + ".");
        }
        System.out.println("Backsubstitution performed.");
        /*System.out.println("Constant term: " + x[0]);*/
        /*return x;
    }*/
    
    /**Returns the inverse of the real symmetric matrix symm
     *Uses LU decomposition, and relies on the fact that the inverse of a symmetric matrix is itself symmetric
     *THIS METHOD WORKS
     *DO NOT MESS WITH IT
     *IT ACTUALLY RETURNS THE CORRECT INVERSE
     *AND METHODS THAT ARE CONFIRMED TO WORK CORRECTLY ARE IN BLOODY SHORT SUPPLY IN THIS PROGRAM RIGHT NOW
     *SO DO NOT MESS WITH IT
     */
    private double[][] invert(double[][] symm) throws DataNotFittableException
    {
        double[][] packed = null;
        
        try
        {
            //get the "packed" version of the LU decomposition of the symmetric matrix symm:
            packed = decomposeLU();
        }
        catch(DataNotFittableException e)
        {
            throw e;
        }
        //declare an array to hold the result (ie, the inverse):
        double[][] result = new double[degree+1][degree+1];
        //this temporary vector (1-D array) holds values at an intermediate stage of the calulations:
        double[] temp = new double[degree+1];
        //holds the intermediate summation values during calculations:
        double sum;
        
        for(int k=0; k<result.length; k++)      // here k represents the column of the inverted matrix
        {
            //To find each column of the inverse, solve Ax=e where e is the kth standard basis vector
            
            //First, forward-substitute in the values for the temp array:
            if(k==0)
                temp[0] = 1.0;
            else
                temp[0] = 0.0;
            for(int i=1; i<temp.length; i++)
            {
                sum = 0;
                for(int j=0; j<i; j++)
                    sum += packed[i][j]*temp[j];
                if(i==k)
                    temp[i] = 1.0-sum;
                else
                    temp[i] = -1.0*sum;
            }
            
            //System.out.println("k=" + k);
            ///printArray(temp);
            //Then, back-substitute the solution to the linear system directly into the result array:
            result[k][a.length-1] = temp[temp.length-1]/packed[packed.length-1][packed.length-1];
            for(int i = result[k].length-2; i>=0; i--)
            {
                sum = 0;
                for(int j=i+1; j<result.length; j++)
                    sum += packed[i][j]*result[k][j];
                result[k][i] = (temp[i]-sum)/packed[i][i];
            }
        }
        /*System.out.println("Inverse of A:");
        printMatrix(result);
        System.out.println("A times its inverse:");
        double[][] identity = new double[a.length][a.length];
        for(int i=0; i<identity.length; i++)
            for(int j=0; j<identity.length; j++)
            {
                sum = 0;
                for(int k=0; k<identity.length; k++)
                    sum += a[i][k]*result[k][j];
                identity[i][j] = sum;
            }
        printMatrix(identity);*/
        return result;
    }
    
    /**Returns the right-hand product of matrix and vector
     *This method works!
     ***/
    private static double[] multiply(double[][] matrix, double[] vector)
    {
        double[] result = new double[vector.length];    //Holds the result of the multiplication
        double sum;                                     //Holds the intermediate sums
        
        for(int i=0; i<result.length; i++)
        {
            sum = 0;
            for(int j=0; j<matrix[i].length; j++)
                sum += matrix[i][j]*vector[j];
            result[i] = sum;
        }
        
        return result;
    }
    
    //Uses the x and y data to make the matrix a - called by constructor
    //Since a is symmetric, it is only necessary to find the suummation for k=0..2N and then just fill in the other
    //elements with those values - proportional to 2N not N^2 which is really nice
    private void makeA()
    {
        a = new double[degree+1][degree+1];
        double sum;     //Used to hold intermediate values in sigmas for calculations throughout the method
        
        //Fill up the first and last columns of the array with the calculated values for those elements
        for(int i=0; i<a[0].length; i++)
        {
            sum = 0;
            if(xData==null)
                System.out.println("PolynomialFit:552 xData==null");
            if(yErrorSquared==null)
                System.out.println("PolynomialFit:554 yErrorSquared==null");
            for(int j=0; j<xData.length; j++)
                sum += Math.pow(xData[j], i)/yErrorSquared[j];
            a[i][0] = sum;
        }
        /*System.out.println("Matrix A after filling first column:");
        printMatrix(a);*/
        //Don't bother with the first element of the last column as it is equal to the last element of the first column:
        for(int i=1; i<a[a.length-1].length; i++)
        {
            sum = 0;
            for(int j=0; j<xData.length; j++)
                sum += Math.pow(xData[j], i+a.length-1)/yErrorSquared[j];
            a[a.length-1][i] = sum;
        }
        //System.out.println("Matrix A after filling last row:");
        //printMatrix(a);
        //Use the elements from the first column to fill the upper-left triangle (including the first element of the last column):
        for(int i=0; i<a.length; i++)
            for(int j=0; j<=i; j++)
               a[j][i-j] = a[i][0];
        //System.out.println("Matrix A after filling upper left components:");
        //printMatrix(a);
        //Use the elements from the last row to fill the lower-right triangle:
        for(int i=1; i<a.length; i++)       //i represents the column
            for(int j=0; j<i; j++)  //j represents the row
                a[a.length-1-j][i] = a[a.length-1][i-j];
        /*System.out.println("Matrix A after filling lower right components:");
        printMatrix(a);*/
    }
    
    //Uses the x and y data to make the vector b - called by constructor
    private void makeB()
    {
        double sum;
        b = new double[degree+1];
        for(int i=0; i<b.length; i++)
        {
            sum = 0;
            for(int j=0; j<yData.length; j++)
                sum += yData[j]*Math.pow(xData[j], i)/yErrorSquared[j];
            b[i] = sum;
        }
        /*System.out.println("Values in b:");
        printArray(b);*/
    }
    
    //returns n!
    private static int factorial(int n)
    {
        int product = 1;
        while(n>0)
        {
            product *= n;
            n--;
        }
        return product;
    }
    
    //returns the incomplete gamma function for the integer n and the real x
    private static double incompleteGamma(int n, double x)
    {
        int sum=0;
        for(int k=0; k<n; k++)
            sum += Math.pow(x, k)/factorial(k);
        return factorial(n-1)*sum/Math.exp(x);
    }
    
    //returns a nicely formatted version of number, ready for printing
    private String format(double number)
    {
        if(number>=1e4)
            return new DecimalFormat("#.####E0").format(number);
        
        if(number>=1)
            return new DecimalFormat("####.###").format(number);
        
        return new DecimalFormat("#.####E0").format(number);
    }
}
