package musr.client.csv;
import musr.client.datacolumn.*;
import musr.client.gui.*;
import musr.client.plot2d.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import java.net.*;
import java.util.*;
import java.security.*;
import java.util.Date;

/**
 *This class exports data tables from muView into CSV (comma-separated values) format, which can be read by spreadsheet programs.
 *
 *@author Jacob Cosman
 */

public class ExportCSV
{
    public Frame parent;
    public FileDialog dialog;
    public DataTable dataTable;
    public OptionsInputGUI options;
    private char separator = ','; //This variable is used to accommodate the case that the computer being used uses ',' as its decimal
    /**
     Creates a new instance of ExportCSV
     @param parentFrame The Frame (ie, GUI) which is contains the DataTable that this method will be filling.
     */
    public ExportCSV(Frame parentFrame)   
    {
        parent = parentFrame;
        dialog = new FileDialog(parentFrame, "Save selected DataTable into '.csv' file", FileDialog.SAVE);
        options = new OptionsInputGUI();
    }
    
    /**
     Reads the file out of the data table, prompts the user for a location to save it to, puts it into .csv format.
     */
    public void exportCSV()
    {
        String filename = "";
        PrintWriter outputStream = null;
        try
        {
            dialog.setFile("Data Table.csv");
            dialog.setLocation(300,300);
            dialog.show();
            options.setVisible(true);
            filename = dialog.getFile();
            
            if((filename!=null)&&(filename.length()>0))
            {
                int dataTableKey = ((GUI)parent).getDataTableIndex();
                dataTable = (DataTable) ((GUI)parent).dictionary.getElement(dataTableKey);
                
                outputStream = new PrintWriter(new FileOutputStream(dialog.getDirectory() + filename));
                
                String temp = dataTable.getTitle();
                if(temp!=null)
                    outputStream.println("Title: " + toCSVFormat(temp));
                temp = dataTable.getAbstract();
                if(temp!=null)
                    outputStream.println("Abstract: " + toCSVFormat(temp));
                temp = dataTable.getComment();
                if(temp!=null)    
                    outputStream.println("Comments: " + toCSVFormat(temp));
                //The " + separator + ' ' + " etc. leaves room for error columns.
                temp = "";
                for(int i=0; i<dataTable.columnCount(); i++)
                    temp = temp + toCSVFormat(dataTable.getColumn(i).getLabel()) + separator + ' ' + separator + ' ' + separator + ' ';
                outputStream.println(temp);
                
                temp = "";
                for(int i=0; i<dataTable.columnCount(); i++)
                    temp = temp + toCSVFormat(dataTable.getColumn(i).getName()) + separator + ' ' + separator + ' ' + separator + ' ' ;
                outputStream.println(temp);
                
                temp = "";
                for(int i=0; i<dataTable.columnCount(); i++)
                    temp = temp + toCSVFormat(dataTable.getColumn(i).getUnits()) + separator + ' ' + separator + ' ' + separator + ' ';
                outputStream.println(temp);
                
                for (int j=0; j<dataTable.columnCount(); j++)
                {    
                    temp = "";
                    for(int i=0; i<dataTable.getColumn(j).getSize(); i++)
                    {
                        temp = temp + toCSVFormat((String)((DataColumn)dataTable.getColumn(j)).getValue(i)) + separator + ' ';
                        
                        if(Double.parseDouble((String)((DataColumn)dataTable.getColumn(j)).getErrorPlus(i).toString())!=0)
                            temp = temp + toCSVFormat((String)(((DataColumn)dataTable.getColumn(j)).getErrorPlus(i))) + separator + ' ';
                        else
                            temp = temp + separator + ' ';
                        
                        if(Double.parseDouble((String)((DataColumn)dataTable.getColumn(j)).getErrorPlus(i).toString())!=0)
                            temp = temp + toCSVFormat((String)(((DataColumn)dataTable.getColumn(j)).getErrorPlus(i))) + separator + ' ';
                        else
                            temp = temp + separator + ' ';                    
                    }
                    outputStream.println(temp);
                }
            }
            else
                JOptionPane.showMessageDialog(null, "The file you have selected is not a valid file.");
        }
        catch(IOException e)
        {
            System.out.println("I/O Exception in file writing: "+ e.getMessage());
        }
        finally //outputSteam must be closed
        {
            if(outputStream!=null)
                outputStream.close();
        }
        
        if (filename != null)
            ((GUI)parent).statusTextField.setText("The CSV file has been created and saved:  " + filename);
        else 
            ((GUI)parent).statusTextField.setText("");
    }
    
    private String toCSVFormat(String entry) //modifies certain characters which are prone to misinterpretation by spreadsheet programs
    {
        for(int i=0; i<entry.length(); i++)
        {    
            if(entry.charAt(i)=='"')
                entry = entry.substring(0, i) + '"' + '"' + entry.substring(i+1);
            else if(entry.charAt(i)=='\n')
                entry = entry.substring(0,i) + "     " + entry.substring(i+1);
        }
        entry = '"' + entry + '"';
        return entry;            
    }
    
    private class OptionsInputGUI extends JFrame implements ActionListener //Finds out if computer uses '.' or ',' as its decimal point
    {
        Checkbox dot;
        Checkbox comma;
        
        public OptionsInputGUI()
        {
            this.setSize(300,200);;
            this.setTitle("CSV Options");
            
            Container contentPane = this.getContentPane();
            contentPane.setLayout(new BorderLayout());
            
            JPanel decimalPanel = new JPanel();
            decimalPanel.setLayout(new GridLayout(3,1));
            CheckboxGroup decimalType = new CheckboxGroup();
            JLabel decimalExplanation = new JLabel("Select the decimal format used by your computer:");
            dot = new Checkbox("Dot decimal (e.g. 123.45)", decimalType, true);
            comma = new Checkbox("Comma decimal (e.g. 123,45)", decimalType, false);
            decimalPanel.add(decimalExplanation);
            decimalPanel.add(dot);
            decimalPanel.add(comma);
            
            JPanel buttonPanel = new JPanel();
            buttonPanel.setLayout(new FlowLayout());
            JButton okButton = new JButton("OK");
            okButton.setSize(25,50);
            okButton.addActionListener(this);
            buttonPanel.add(okButton);
            
            contentPane.add(decimalPanel, BorderLayout.CENTER);
            contentPane.add(buttonPanel, BorderLayout.SOUTH);
        }
        
        public void actionPerformed(ActionEvent e)
        {
            if((e.getActionCommand()).equals("OK"))
            {
                if(comma.getState()) //user's computer uses ',' as decimal point, so csv expected to use ';' as separator
                    separator = ';';
                else //user's computer uses '.' as decimal point, so csv expected to use ',' as separator
                    separator = ',';
                this.setVisible(false);
            }
            else
                System.out.println("Error in graphical options interface.");
        }
    }
}