
package musr.client.datacolumn;

import java.util.Date;
/**
 * This class is a collection of value points that are to be plotted.
 * Classes LinearColumn, PoissonDataColumn and MultiColumn (that contains ConcreteDataColumn)
 * have different ways of implementing a few methods.
 * This interface defines those methods.
 *
 * So far:
 * Multicolumn is generated through loading of db files. And through GENERATING new columns
 * PoissonDataColumn and LinearColumn are created when importing MUD files
 *
 * @author Ivan Tucakov, Nikola Tucakov
 */
public interface DataColumn {

  // methods that are defined in Columns
  public int getSize();
  public Object getValue(int index);
  public Object getErrorPlus(int index);
  public Object getErrorMinus(int index);
  /**
   * Can be "virtual" or "real"
   */
  public String getType();
  /**
   * Returns a Column defined by the range given to it
   */
  public DataColumn getSubColumn(Range someRange);

  public String toString();

  // these methods are defined in the ColumnHeader class of each Column
  public String getName();
  public String getLabel();
  public String getUnits();
  public Date getDate();
  public void setName(String name);
  public void setLabel(String label);
  public void setUnits(String units);
  public void setDate(Date date);


  public Object clone() throws CloneNotSupportedException;

}

