package musr.client.datacolumn;

import java.util.Date;

/**
 * Also known as Virtual Column, this Class Creates a Column with linearly accending values in it.
 * These values start from the first 'begin' value, accend in 'step' amount and end with the 'end' value
 */

public class LinearColumn extends ColumnHeader implements DataColumn , java.io.Serializable {

  // instance variables
  public double begin;
  public double end;
  public int size;
  public double step;

  // constructors
  public LinearColumn(String name, String label, String units, double begin, double step, int size, Date date) {
    super(name, label, units, date);
    this.begin = begin;
    this.size = size;
    this.step = step;
     // trows a division by zero exeption
  }

  // Methods from the DataColumn interface
    public int getSize() { return size; }
    public Object getValue(int index) {
      return new Double(begin + index*step);
    }
    public String getType() { return "virtual"; }

    public final boolean isUncertain(){return false;};
    public Object getErrorPlus(int index){return new Double(0.0);}
    public Object getErrorMinus(int index) {return new Double(0.0);}


    public DataColumn getSubColumn(Range r) {
      double minValue = ((Double)getValue(r.min)).doubleValue();
      double maxValue = ((Double)getValue(r.max)).doubleValue();
      return new LinearColumn(name, label, units, minValue, step ,(int) ((maxValue-minValue)/step)+1, date);
    }


    public Object clone() throws CloneNotSupportedException {
      LinearColumn returnValue = (LinearColumn)super.clone();
      // Need to clone the instances: begsin, end, label, name, size, step and units!!
      return returnValue;
    }
		/*
			public static void main(String args[]) {
			LinearColumn a = new LinearColumn("Juhu",3.0,17.0,29);
			Range r = new Range(2, 12);
			a = (LinearColumn) a.getSubColumn(r);
			System.out.println("label: "+a.getLabel());
			System.out.println("size: "+a.size);
			System.out.println("step: "+a.step);
			for(int i=0; i<a.getSize(); i++) {
			System.out.println("vlue"+i+" : "+a.getValue(i));
			}
		}
		*/
}
