package musr.client.datacolumn;

import java.util.Date;

/**
 * Multi column is consistent of three columns.
 * First one is the value column, second one is posErr and third one is negErr. They are all ConcreteDataColumns
 * The MultiColumn is created when loading the db files and also when generating a new column
 *
 *  @author Ivan Tucakov, Nikola Tucakov
 */
public class MultiColumn implements Cloneable, DataColumn, java.io.Serializable {

  static final long serialVersionUID = 0L; // SUID

    private ConcreteDataColumn ValueColumn;
    private ConcreteDataColumn ErrorPlusColumn;
    private ConcreteDataColumn ErrorMinusColumn;

    // instance methods only in MultiColumn
    public final DataColumn getValueColumn() { return ValueColumn; }
    public final DataColumn getErrorPlusColumn() { return ErrorPlusColumn; }
    public final DataColumn getErrorMinusColumn() { return ErrorMinusColumn; }
    public final void setValueColumn(ConcreteDataColumn dc) { ValueColumn = dc; }
    public final void setErrorPlusColumn(ConcreteDataColumn dc) { ErrorPlusColumn = dc; }
    public final void setErrorPlusColumn(Object obj, int index) { ErrorPlusColumn.setElementAt(obj, index); }
    public final void setErrorMinusColumn(ConcreteDataColumn dc) { ErrorMinusColumn = dc; }
    public final void setErrorMinusColumn(Object obj, int index) { ErrorMinusColumn.setElementAt(obj, index); }

    // constructor
    public MultiColumn(ConcreteDataColumn valCol, ConcreteDataColumn errPlus, ConcreteDataColumn errMinus) {
				ValueColumn = valCol;
				ErrorPlusColumn = errPlus;
				ErrorMinusColumn = errMinus;
    }


    public boolean isEmpty() { if (((DataColumn)ValueColumn).getSize()>0) { return false; } else { return true; } }

    // Methods declared in the DataColumn interface
    // Operate on ValueColumn DataColumn !!!!!
    public int getSize() { return ValueColumn.getSize(); }
    public Object getValue(int index) {
        return ValueColumn.getValue(index);
    }
    /**
     * This method will never be used since it is not a column of data but a column of columns so it contains no virtual nor real data
     */
    public String getType() { return "real"; }
    public DataColumn getSubColumn(Range someRange) {
        return ValueColumn.getSubColumn(someRange); }

    public String getName() { return ValueColumn.getName(); }
    public String getLabel() { return ValueColumn.getLabel(); }
    public String getUnits() { return ValueColumn.getUnits(); }
    public Date getDate() { return ValueColumn.getDate(); }
    public void setName(String name) { ValueColumn.setName(name); }
    public void setLabel(String label) { ValueColumn.setLabel(label); }
    public void setUnits(String units) { ValueColumn.setUnits(units); }
    public void setDate(Date date) { ValueColumn.setDate(date); }

    // These methods that can be declared in a seperate ErrorColumn interface.
    // No need for that right now.
    // There are no methods to operate with their Names and Labels.
    // Right now these functions are not used because GUI
    // operates only with the Error Columns themselves (above methods)
    public int getErrorPlusSize() { return ErrorPlusColumn.getSize(); }
    public Object getErrorPlus(int index) {
      return ErrorPlusColumn.getValue(index); }
    public DataColumn getErrorPlusSubColumn(Range someRange) {
      return ErrorPlusColumn.getSubColumn(someRange); }

    public int getErrorMinusSize() { return ErrorMinusColumn.getSize(); }
    public Object getErrorMinus(int index) {
      return ErrorMinusColumn.getValue(index); }
    public DataColumn getErrorMinusSubColumn(Range someRange) {
      return ErrorMinusColumn.getSubColumn(someRange); }

    public Object clone() throws CloneNotSupportedException {
      MultiColumn returnValue = (MultiColumn)super.clone();
      returnValue.ErrorMinusColumn = (ConcreteDataColumn)ErrorMinusColumn.clone();
      returnValue.ErrorPlusColumn = (ConcreteDataColumn)ErrorPlusColumn.clone();
      returnValue.ValueColumn = (ConcreteDataColumn)ValueColumn.clone();
      return returnValue;
    }

    /**
    * with this overridden method the column can be printed in a nice format
    */
    public String toString() {
      return getValueColumn() +
         "Plus Error column: " + (getErrorPlusColumn()!=null) + "\n" +
         "Minus Error column: " + (getErrorMinusColumn()!=null) + "\n";
    }

}
