package musr.client.fourier;

public class FFT
{
    /**
     *Main method for testing purposes only.
     */
    public static void main(String[] args)
    {
        System.out.println(2 + "\t" + ((double)paddingZerosNeeded(2))/2);
        System.out.println(20 + "\t" + ((double)paddingZerosNeeded(20))/20);
        System.out.println(200 + "\t" + ((double)paddingZerosNeeded(200))/200);
        System.out.println(2000 + "\t" + ((double)paddingZerosNeeded(2000))/2000);
        System.out.println(20000 + "\t" + ((double)paddingZerosNeeded(20000))/20000);
        System.out.println(200000 + "\t" + ((double)paddingZerosNeeded(200000))/200000);
        /*Complex[] testData;
        long start;
        int length;

        for(int i=10;i<20;i++)
        {
            length = (int)(Math.pow(2,i));
            testData = generateData(length);
            start = System.currentTimeMillis();
            fft(testData);
            System.out.println(length + "\t" + (System.currentTimeMillis()-start));
        }
        
        System.out.println("========");
        for(int i=9;i<19;i++)
        {
            length = 3*(int)(Math.pow(2,i));
            testData = generateData(length);
            start = System.currentTimeMillis();
            fft(testData);
            System.out.println(length + "\t" + (System.currentTimeMillis()-start));
        }*/
    }
    /**
     *Testing only!
     */
    private static Complex[] generateData(int length)
    {
        Complex[] testData = new Complex[length];
        for(int i=0; i<length; i++)
        {
            testData[i] = new Complex(Math.cos(2*Math.PI*i/8), Math.sin(2*Math.PI*i/8));
            //System.out.println(testData[i].realPart() + "\t" + testData[i].imaginaryPart());
        }
        return testData;
    }
    
    /**
     *Takes as input a complex data set and returns the Fourier transform of this data set using the FFT algorithm.
     *The input data set is assumed in length to be a power of two. Trouble will result if this is not the case.
     */
    public static Complex[] fft(Complex[] data)
    {
        //base case
        if(data.length==1)
            return new Complex[] {data[0]};
            
        //additional base case - added to reduce necessity of padding
        //uses definition of discrete Fourier transform for N=3
        if(data.length==3)
        {
                Complex a = data[0];
                Complex b = (data[1].add(data[2])).multiply(-0.5);
                Complex c = (data[1].subtract(data[2])).multiply(new Complex(0, 0.866025404));
                return new Complex[]{a.add(b.multiply(-2)), a.add(b.subtract(c)), a.add(b.add(c))};
        }
        
        //otherwise, check to ensure that data is subdividable into two arrays of equal length
        if(data.length %2 != 0)
            throw new RuntimeException("Data set length is not a power of two.");
                
        //break array up into even and odd
        Complex[] oddData  = new Complex[data.length/2];
        Complex[] evenData = new Complex[data.length/2];
        for(int i=0; i<data.length/2; i++)
        {
            evenData[i] = data[2*i];
            oddData[i]  = data[2*i+1];
        }   
        
        Complex[] evenFFT = fft(evenData);
        Complex[] oddFFT  = fft(oddData);
        
        //recombine the Fourier transforms of the even and odd halves by the FFT algorithm
        Complex[] result = new Complex[data.length];
        double jTheta;
        Complex wToJ = new Complex();
        for (int j = 0; j < data.length/2; j++) 
        {
            jTheta = -2*j*Math.PI/data.length;
            wToJ.setValue(Math.cos(jTheta), Math.sin(jTheta));
            result[j]               = evenFFT[j].add(wToJ.multiply(oddFFT[j]));
            result[j+data.length/2] = evenFFT[j].subtract(wToJ.multiply(oddFFT[j]));
        }
        return result;
    }
    
    /**
     *Finds the number of zero values needed to pad a data set of length n, so that the data set can be used by the FFT algorithm above.
     */
    public static int paddingZerosNeeded(int n)
    {
        return totalLengthNeeded(n) - n;
    }
    
    /**
     *Given n, finds the next largest length that would give a data set to which the FFT algorithm avobe could be applied.
     */
    private static int totalLengthNeeded(int n)
    {
        if(n<=4)    //no padding needed
            return n;
        int alpha = (int)Math.floor(Math.log(n)/Math.log(2));
        if(3*Math.pow(2,alpha-1)>n)     //can use multiple of three to reduce padding somewhat
            return 3*(int)Math.pow(2,alpha-1);
        else                            //just use a power of two, which the algorith treats effectively
            return (int)Math.pow(2,alpha+1);
    }
}
