package musr.client.function;
import java.util.*;  // For Stack
import musr.client.dbreader.*;
import musr.client.gui.*;
import java.awt.*;       // for Frame()
import javax.swing.JFrame;
import javax.swing.JOptionPane;

/**
 * The function is defined as any expression that can be used as a function call.
 * This function contains only one parameter x. The function is evaluated with the
 * appropriate x value when the evaluate(x) method is called.
 *
 * @author Ari
 */

public class Function {
	// Instance Variables
	/**
         *  Expression to be evaluated
         */
        private Tokenizer Expression;

	// Constructors
	public Function(String input) {
            //System.out.println("input:\n" + input);
		Expression = new Tokenizer(input);
	}

	// Instance Methods
	/**
         *  This Method evaluates the postfix expression
         *  -the x value is the parameter value
         *  -the caseOff value checks function names case-(in)sensitively
         *  -the functionsTable holds 'custom' defined functions
         */
      public double evaluate(Frame parent, double x, boolean caseOn, Functions functionsTable, Constants constantsTable) {
        Stack fStack = new Stack();
        int tempKind = -1;
        Token tempToken;

                try {

                for(int i=0; i<Expression.getPostfixTokenCount(); i++) {
                      tempToken = Expression.getPostfixToken(i);
                      //System.out.println("Token String value:\n" + tempToken.getName());
                      tempKind = tempToken.getKind();

			Double D;
			double d;
			switch (tempKind) {
			case TokenKind.number:
				fStack.push(tempToken.getValue());
				break;
			case TokenKind.x:
				D = new Double(x);
				fStack.push(D);
				break;
			case TokenKind.negate:
				d = -((Double)fStack.pop()).doubleValue();
				D = new Double(d);
				fStack.push(D);
				break;
                        case TokenKind.pozitive:
				d = +((Double)fStack.pop()).doubleValue();
				D = new Double(d);
				fStack.push(D);
				break;
			case TokenKind.add:
				d = ((Double)fStack.pop()).doubleValue();
				d += ((Double)fStack.pop()).doubleValue();
				D = new Double(d);
				fStack.push(D);
				break;
			case TokenKind.subtract:
				d = ((Double)fStack.pop()).doubleValue();
				d = ((Double)fStack.pop()).doubleValue() - d;
				D = new Double(d);
				fStack.push(D);
				break;
			case TokenKind.multiply:
				d = ((Double)fStack.pop()).doubleValue();
				d *= ((Double)fStack.pop()).doubleValue();
				D = new Double(d);
				fStack.push(D);
				break;
			case TokenKind.divide:
				d = ((Double)fStack.pop()).doubleValue();
				d = ((Double)fStack.pop()).doubleValue() / d;
				D = new Double(d);
				fStack.push(D);
				break;
			case TokenKind.power:
				d = ((Double)fStack.pop()).doubleValue();
				d = Math.pow(((Double)fStack.pop()).doubleValue(), d);
				D = new Double(d);
				fStack.push(D);
				break;
			case TokenKind.funcname:
                                // check if the function is in the Table
                                if (constantsTable.constantExists(parent, tempToken.getName())) 
                                {
                                  //System.out.println("Considering the following token:\n" + tempToken.getName());
                                  // check for case sensitivity first
                                  String caseString = "";
                                  if (!caseOn) caseString = tempToken.getName().toUpperCase();
                                  else caseString = tempToken.getName();
                                  //Function fun = new Function( Double.toString( constantsTable.getConstantValue(parent, caseString.toString()) ) );
				  //d = ((Double)fStack.pop()).doubleValue();
                                  //D = new Double(fun.evaluate(parent, d, caseOn, functionsTable, constantsTable));
                                  //D = new Double(d);
                                  //System.out.println("Pushing the following token:\n" + D.toString());
                                  //fStack.push(D);
                                  //System.out.println("Pushing the following token:\n" + constantsTable.getConstantValue(parent, caseString));
                                  fStack.push(constantsTable.getConstantValue(parent, caseString));
                                } 
                                // check if the function is in the Table
                                else if (functionsTable.functionExists(parent, tempToken.getName())) 
                                {
                                  // check for case sensitivity first
                                  String caseString = "";
                                  if (!caseOn) caseString = tempToken.getName().toUpperCase();
                                  else caseString = tempToken.getName();
                                  Function fun = new Function(functionsTable.getFunctionValue(caseString).toString());
				  d = ((Double)fStack.pop()).doubleValue();
                                  D = new Double(fun.evaluate(parent, d, caseOn, functionsTable, constantsTable));
                                  fStack.push(D);
                                }
                                else if (tempToken.getName().toUpperCase().equals("EXP")) {
					d = ((Double)fStack.pop()).doubleValue();
					D = new Double(Math.exp(d));
					fStack.push(D);
				}
				else if (tempToken.getName().toUpperCase().equals("SIN")) {
					d = ((Double)fStack.pop()).doubleValue();
					D = new Double(Math.sin(d));
					fStack.push(D);
				}
				else if (tempToken.getName().toUpperCase().equals("COS")) {
					d = ((Double)fStack.pop()).doubleValue();
					D = new Double(Math.cos(d));
					fStack.push(D);
				}
				else if (tempToken.getName().toUpperCase().equals("TAN")) {
					d = ((Double)fStack.pop()).doubleValue();
					D = new Double(Math.tan(d));
					fStack.push(D);
				}
				else if (tempToken.getName().toUpperCase().equals("SQRT")) {
					d = ((Double)fStack.pop()).doubleValue();
					D = new Double(Math.sqrt(d));
					fStack.push(D);
				}
				else if (tempToken.getName().toUpperCase().equals("SQR")) {
					d = ((Double)fStack.pop()).doubleValue();
					D = new Double(Math.pow(d,2));
					fStack.push(D);
				}
                                else if ((tempToken.getName().toUpperCase().equals("LOG"))||(tempToken.getName().toUpperCase().equals("LN"))) 
                                {
                                        d = ((Double)fStack.pop()).doubleValue();
                                        D = new Double(Math.log(d));
                                        fStack.push(D);
                                }
                                else {
                                  System.out.println("Expression: "+tempToken.getName()+" is undefined.");
                                  // in case this function was called through the DBConsole, display error message
                                  ((GUI)parent).console.jta.append(((GUI)parent).console.jtf.getText()+"\n");
                                  ((GUI)parent).console.jta.append("Expression: "+tempToken.getName()+" is undefined.\n");
                                  ((GUI)parent).console.jtf.setText("DB:");
                                  ((GUI)parent).console.newLineRecognizer = ""; // restart that value
                                  //((GUI)parent).popAlertDialog("Error","Expression: "+tempToken.getName()+" is undefined.\n", "");
                                  JOptionPane.showMessageDialog(parent, "Expression: "+tempToken.getName()+" is undefined.\n", "Undefined expression",
                                          JOptionPane.WARNING_MESSAGE);
                                }
                                // else throw Exception - function undefined
			}
		  }
                } // end try

                // Display messages if something goes wrong
                catch (EmptyStackException e) {
                  System.err.println("Error in Function parsing, method evaluate(): " + e.getMessage());
                }
                return ((Double)fStack.pop()).doubleValue();
	}
}
