package musr.client.function;

import java.awt.*;
import java.util.*;

/**
 * Creates an array of tokens from the given String input
 *
 * @author Ari
 */

public class Tokenizer {
	// Instance Variables
	private Token infixExpression[];
	private int infixTokenCount = 0;
	private Token postfixExpression[];
	private int postfixTokenCount = 0;

	private int parameterCount = 0;


	// Constructors
	/**
         *  Breaks the input into Tokens and then puts it into the infix Expression array
         */
	public Tokenizer(String input) {
		input += " "; // Add " " at the end to avoid problems

		infixTokenCount = 0;

		Stack infixStack = new Stack();
		int cur=0;  // positions where to read out a Token from the input
		int leng=0;
		TokenKind kind = new TokenKind();

		// Put each token from the input onto the stack
		while (cur < input.length()) {
			if (input.charAt(cur)!=' ') {

				char c = input.charAt(cur);
				if (c == '(') { kind.setKind(TokenKind.openpar); leng=1; }
				else if (c == ')') { kind.setKind(TokenKind.closepar); leng=1; }
				else if (c == '+') { kind.setKind(TokenKind.add); leng=1; }
				else if (c == '-') { kind.setKind(TokenKind.subtract); leng=1; }
				else if (c == '*') { kind.setKind(TokenKind.multiply); leng=1; }
				else if (c == '/') { kind.setKind(TokenKind.divide); leng=1; }
				else if (c == '^') { kind.setKind(TokenKind.power); leng=1; }
				else if (c == 'x' && !Character.isLetterOrDigit(input.charAt(cur+1))) {
					kind.setKind(TokenKind.x); leng=1;
				}
				else if (c == 'p' && Character.isDigit(input.charAt(cur+1))) {
					parameterCount++;
					kind.setKind(TokenKind.parameter);
					for (leng = 1; Character.isDigit(input.charAt(cur+leng));
						 leng++); // Digits
				}
				else if (Character.isLetter(c) || c == '_') {
					kind.setKind(TokenKind.funcname);
					for (leng = 1; Character.isLetterOrDigit(input.charAt(cur+leng))|| input.charAt(cur+leng) == '_';
						 leng++); // Letters of Digits
				}
				else {
					kind.setKind(TokenKind.number);
					for (leng = 1; Character.isDigit(input.charAt(cur+leng));
						 leng++); // Digits
					if (input.charAt(cur+leng) == '.') leng++;  // Decimal point
					for (; Character.isDigit(input.charAt(cur+leng)); leng++); // Digits
					if (input.charAt(cur+leng) == 'E'
						|| input.charAt(cur+leng) == 'e') { // Exponent
						leng++;
						if ((input.charAt(cur+leng) == '+')
							|| (input.charAt(cur+leng) == '-')) leng++;
						for (; Character.isDigit(input.charAt(cur+leng));
							 leng++); // Digits
					}
				}

				// Put the next Token onto the infixStack
				Token result = new Token(input.substring(cur,cur+leng), kind);
				infixStack.push(result);
				infixTokenCount++;

				cur+=leng;
			}
			else cur++;
		}

		// Fill the infix array of tokens with the tokens from infix stack
		infixExpression = new Token[infixTokenCount];
		for(int i=infixTokenCount-1; i>=0; i--) {
			infixExpression[i] = (Token) infixStack.pop();
		}

		// Convert to postfix expression
		postfix();
	}


	/**
         *  Convert from infix expression to a postfix expression
         */
	public void postfix() {
		Stack opStack = new Stack();  // Stack to keep operations
		Stack postfixStack = new Stack();
		int tempKind = -1;
		int prevKind = -1;
		postfixTokenCount = 0;

		for(int i=0; i<infixTokenCount; i++) {
			prevKind = tempKind;
			tempKind = infixExpression[i].getKind();

			switch (tempKind) {
				// process constants or vars
			case TokenKind.number:
			case TokenKind.x:
			case TokenKind.parameter:
				postfixStack.push(infixExpression[i]);
				postfixTokenCount++;
				break;
				// put openpar and functions onto opStack
			case TokenKind.openpar:
			case TokenKind.funcname:
				opStack.push(infixExpression[i]);
				break;
				// when closepar process all the ops on opStack until openpar
			case TokenKind.closepar:
				// load the ops
				while ( ((Token)opStack.peek()).getKind()!=TokenKind.openpar ) {
					postfixStack.push(opStack.pop());
					postfixTokenCount++;
					}
				// pop the openpar
				opStack.pop();
				// if there is a funcname after, pop it off too
				if ( !opStack.empty() &&
					 ((Token)opStack.peek()).getKind()==TokenKind.funcname ) {
					postfixStack.push(opStack.pop());
					postfixTokenCount++;
				}
				break;
				// unary minus
			case TokenKind.subtract:
				if (prevKind==-1 || prevKind==TokenKind.openpar) {
					opStack.push(new Token("neg",new TokenKind(TokenKind.negate)));
					break;
				}
			case TokenKind.add:
				if (prevKind==-1 || prevKind==TokenKind.openpar) {
					opStack.push(new Token("poz",new TokenKind(TokenKind.pozitive)));
					break;
				}
				// else continue to binary sutract
				// Operators
			case TokenKind.multiply:
			case TokenKind.divide:
			case TokenKind.power:
				// pop opStack while operators have higher precedence
				while (!opStack.empty() &&
					   (infixExpression[i].getPrecedence() <
						((Token)opStack.peek()).getPrecedence())) {
					postfixStack.push(opStack.pop());
					postfixTokenCount++;
				}
				// then push current operator
				opStack.push(infixExpression[i]);
			}
		}
		// empty the opStack et the end of the input
		while (!opStack.empty()) {
			postfixStack.push(opStack.pop());
			postfixTokenCount++;
		}

		// Fill the postfix array of tokens with the tokens from postfix stack
		postfixExpression = new Token[postfixTokenCount];
		for(int i=postfixTokenCount-1; i>=0; i--) {
			postfixExpression[i] = (Token) postfixStack.pop();
		}
	}



	public int getInfixTokenCount() { return infixTokenCount; }
	public Token getInfixToken(int i) { return infixExpression[i]; }
	public int getPostfixTokenCount() { return postfixTokenCount; }
	public Token getPostfixToken(int i) { return postfixExpression[i]; }

}
