package musr.client.gui;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.lang.*;
import javax.swing.*;
import java.util.*;
import musr.client.function.*;
import musr.client.gui.*;
import musr.client.plot2d.*;
import musr.client.datacolumn.*;

/**
 *
 * @author Ivan Tucakov
 */
public class DBConsole extends JDialog implements KeyListener, java.io.Serializable {

  static final long serialVersionUID = 0L; // SUID
  
  public static final int CONSOLE_WIDTH = 500;
  public static final int CONSOLE_HEIGHT = 809;

  Button buttonClose;            // The CLOSE button of the dialog
  public JTextArea jta;     // The message displayed by the dialog and also anything that was written in jtf comes here
  public JTextField jtf;   // THis is where the user enters the command
  JScrollPane jsp;

  Vector jtfStack = new Vector();
  int jtfStackIndex = -1;

  public JFrame parent;  // This is the GUI where the console is called from

  String delimiters = "=,\\ ";
  public String newLineRecognizer = ""; // used to recognize the second line after some command input
  boolean doplot = false; // This is used for the two modes the DB and the DOPLOT mode.
  String fileName = ""; // this is the file name of the loaded DataTable
  private Panel dataPanel;

  // These are helper variables that are used for the GUI DataTable loading
  String abstr = "";
  String comm = "";
  String modName = "";
  boolean modErr = false; // used when the user wants to modify the error column
  String genName = "";
  int labIndex = -1;


  /** This variable is used for GENERATE command purposes */
  public long rowCount = 0;
  /** This variable is used when DATA is loaded, to get a line of data for columns */
  public String fullLine = "";

  // Add a "Close" button at the bottom of the Panel
  private class ActionListenerS implements java.awt.event.ActionListener, java.io.Serializable {
    public void actionPerformed(ActionEvent e) {}
  };
  ActionListenerS buttonListener;

  public DBConsole(JFrame paren, String title) {
 		super(paren, title, false);  // set the Dialog to be non-modal (false)

    this.parent = paren;
    setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);
    this.getContentPane().setLayout(new BorderLayout());

    jta = new JTextArea();
    jta.setEditable(false); // this is where the past commands are displayed
    jtf = new JTextField("DB:");
    jtf.setCaretPosition(jtf.getText().length());
    jtf.addKeyListener(this);
    jsp = new JScrollPane(jta); // use the scroll pane if the textarea can't fit the Component
    
    JPanel consolePanel = new JPanel();
    consolePanel.setLayout(new BorderLayout());

    consolePanel.add(jsp, BorderLayout.CENTER);
    consolePanel.add(jtf, BorderLayout.SOUTH);
    
    this.getContentPane().add(consolePanel, BorderLayout.CENTER);
    
    setSize(CONSOLE_WIDTH, CONSOLE_HEIGHT);
    setLocationRelativeTo(parent);

    /*buttonListener = new ActionListenerS() {
      public void actionPerformed(ActionEvent e) {
        String command = ((Button)e.getSource()).getLabel();
        if (command.equals("Close"))
          DBConsole.this.hide();   // pop down dialog
      }
    };
    buttonClose = new Button("Close");
    buttonClose.addActionListener(buttonListener);
    Panel p = new Panel();
    p.setLayout(new FlowLayout(FlowLayout.CENTER, 10, 10));
    p.add(buttonClose);
    this.getContentPane().add(p, BorderLayout.SOUTH);*/
    // Set the dialog size to the preferred size of its components
    //this.pack();
  }
  // Handle Key Events in the TextArea
  protected void processConsole(KeyEvent e) {
    String keyCommand = e.getKeyText(e.getKeyCode());
    // this will give out the text that has been put in before the text that is cirrently in the jtf textfield
    if (keyCommand.equals("Up")) {
      if (!jtfStack.isEmpty()) {// this means that the jtfStack is not empty
        // load from stack only if the 'DB:' prompt is on.
        if (jtf.getText().length()>=3 && jtf.getText().substring(0,3).equals("DB:")) {
          if (jtfStackIndex>0) {
            jtfStackIndex--;
            setText((String)jtfStack.get(jtfStackIndex));
          }
        }
      }
    // this will give out the text that has been put in after the text that is currently in the jtf textfield
    } else if (keyCommand.equals("Down")) {
      if (!jtfStack.isEmpty()) {// this means that the jtfStack is not empty
        // load from stack only if the 'DB:' prompt is on.
        if (jtf.getText().length()>=3 && jtf.getText().substring(0,3).equals("DB:")) {
          if (jtfStackIndex<jtfStack.size()-1) {
            jtfStackIndex++;
            setText((String)jtfStack.get(jtfStackIndex));
          }
        }
      }
    } else if (keyCommand.equals("Enter")) {
      try {
        // Read the last line from the TextArea to get the command
        // calculate the offset first
          // Note: the Prompts (DB: and DOPLOT:) differ in lenght so cut the String off depending on the MODE
        String s = "";
        if (jtf.getText().length()>=3) // in case there is no prompt (for non 'direct command' input)
          if (doplot) s = jtf.getText().substring(7); // cut off the 'DOPLOT:' prompt
          else if (jtf.getText().substring(0,3).equals("DB:")) {
            s = jtf.getText().substring(3); // cut off the 'DB:' prompt
            // Add this command to the jtfStack for future reference
            // note: Only the commands that have the 'DB:' promtp are loaded
            jtfStack.add((String)jtf.getText());
            // 'restart' the Stack Index value to the new vector size - 1
            jtfStackIndex = jtfStack.size();
          }

        StringTokenizer st = new StringTokenizer(s);
        int numTokens = st.countTokens();
        String command = "";
        if(numTokens!=0) {command = st.nextToken().trim();}
        // Check for MODE. It can be in DB or DOPLOT. Right below is the DB mode
        if (!doplot) {
          // First check if user wants to read from a file
          if (command!="" && command.substring(0,1).equals("@")) { // first one is to make sure I can take substring()
            fileName = command.substring(1); // get rid of "@"
            // Check to see if the path is Relative (doesn't start with "/") or Absolute (contains full directory path)
            // if Absolute: leave fileName as is but save the directory path into dirPath (in GUI)
            if (fileName.startsWith("/")) {
              ((GUI)parent).dirPath = fileName.substring(0, fileName.lastIndexOf("/")+1);
              fileName = fileName.substring(fileName.lastIndexOf("/")+1, fileName.length());
            }
//System.out.println("dirPath: "+((GUI)parent).dirPath+"fileName: "+fileName);
            ((GUI)parent).loadDBFile(((GUI)parent).dirPath, fileName);
            append(jtf.getText()+"\n");
            // Since DATA has been most probaly loaded, the variable tempLabels has to be updated
            // note: the data has been loaded in the first column of the DataTableList
            ((GUI)parent).tempLabels.clear();
            int selectedDataTable = ((GUI)parent).dataTableList.getItemCount()-1; // it was loaded in the last row of DataTableList in the GUI
            if ( ((DataTable)((GUI)parent).dictionary.getElement(selectedDataTable)).columnCount()!=0) //if DATA has been loaded
              for (int i=0;i<((DataTable)((GUI)parent).dictionary.getElement(selectedDataTable)).columnCount();i++) {
                ((GUI)parent).tempLabels.add(i,
                  ((MultiColumn)((DataTable)((GUI)parent).dictionary.getElement(selectedDataTable)).getColumn(i)).getLabel());
              }
            setText("DB:");
            jtf.setCaretPosition(jtf.getText().length());
          }
          else if ((command.toUpperCase().equals("A")) || (command.toUpperCase().equals("AB")) ||
                  (command.toUpperCase().equals("ABS")) || (command.toUpperCase().equals("ABST")) ||
                  (command.toUpperCase().equals("ABSTR")) || (command.toUpperCase().equals("ABSTRA")) ||
                  (command.toUpperCase().equals("ABSTRAC")) || (command.toUpperCase().equals("ABSTRACT"))) {
            append(jtf.getText()+"\n");
            setText(":");
            abstr = "";
            newLineRecognizer = "abstract";
          }
          else if ((command.toUpperCase().equals("COMMENTS")) || (command.toUpperCase().equals("COMMENT")) ||
                  (command.toUpperCase().equals("COMMEN")) || (command.toUpperCase().equals("COMME")) ||
                  (command.toUpperCase().equals("COMM")) || (command.toUpperCase().equals("COM")) ||
                  (command.toUpperCase().equals("CO")) || (command.toUpperCase().equals("C"))) {
            append(jtf.getText()+"\n");
            setText(":");
            comm = "";
            newLineRecognizer = "comments";
          }
          // This sets up teh dirPath in the GUI
          else if ((command.toUpperCase().equals("C")) || (command.toUpperCase().equals("CD"))) {
            append(jtf.getText()+"\n");
            // Check to see it the format of teh command is 'cd <dirpath>' (i.e. two tokens)
            if(numTokens == 2) {
              String newDirPath = st.nextToken().trim();
              // if the user forgot to put "/" at the end, append it
              if (!newDirPath.endsWith("/")) newDirPath = newDirPath+"/";
              ((GUI)parent).dirPath =  newDirPath;
              append("The directory path has been set to: "+newDirPath+"\n");
              setText("DB:");
              newLineRecognizer = "";
            } else {
              append("Make sure that your command is of 'cd <dirpath>' format \n");
              setText("DB:");
              newLineRecognizer = "";
            }
          }
          //
          // Note: the code below closely resembles the "DATA" code in the dbFileReader Class
          else if ((command.toUpperCase().equals("DATA")) || (command.toUpperCase().equals("DAT")) ||
                  (command.toUpperCase().equals("DA")) || (command.toUpperCase().equals("D"))) {

            Date date = new Date();

            // Read in the names of variable arrays
            StringTokenizer tokenizer;
            tokenizer = new StringTokenizer(s, "=,\\ ", false);
            tokenizer.nextToken();  // just pop "DATA"
            int i=0; // this is to count the number of columns


            // Check whether columns already exist in the DataTable
            // if it has columns, then just add new rows regardless to whether user had column names after DATA

                int columnCount = 0;
                if (((GUI)parent).dictionary.size()!=0 && ((GUI)parent).dataTableList.getItemCount()!=0)
                  columnCount = ((DataTable)((GUI)parent).dictionary.getElement(((GUI)parent).dataTableList.getSelectedIndex())).columnCount();
            if ( columnCount>0 ) { // the DataTable has columns
              // this means that Vectors: tempNames,tempDataColumns
              // have all been loaded already. Now give the 'status' of the DataTable to the user and
              // just have the user add rows to these columns.
              append(jtf.getText()+"\n");
              append(" DATA exists. There are "+columnCount+" Columns:\n");
              for (int k=0; k<columnCount; k++) append("  "+(String)((GUI)parent).tempNames.get(k)+"  ");
              append("\n So far we have "+((DataColumn)((GUI)parent).tempDataColumns.getColumn(0)).getSize()+" Rows.\n" );
              append("Enter data as ordered Ntuples: (X1,+,-, X2,+,-,..., Remark) \n");
              setText(":");
              newLineRecognizer = "dataFullLineCollect";
            }
            // the DataTable is empty
            else {
              // check whether the user entered column names after DATA
              // if yes, then load new columns, as many as there are names
              if (tokenizer.hasMoreTokens()) { // There are column names..
                ((GUI)parent).tempNames.removeAllElements();
                while (tokenizer.hasMoreTokens())  { // Read name
                 ((GUI)parent).tempNames.addElement((tokenizer.nextToken()).trim());
                 i++;
                }
                columnCount=i; // define the number of coulums
                for(int j=0; j<columnCount; j++) {
                  // Create all the columns and add set their column names and labels
                  if(((GUI)parent).tempLabels.isEmpty()) // If LABELS command hasn't been called jet
                    // Note! I am setting the label as " ", not as "". THis is important since the Plot2D
                    // has problems when trying to draw the labels that are Strings of the form ""
                    ((GUI)parent).tempDataColumns.addColumn(new MultiColumn(
                                                                  new ConcreteDataColumn((String) ((GUI)parent).tempNames.elementAt(j)," "," ", date),
                                                                  new ConcreteDataColumn(),
                                                                  new ConcreteDataColumn() ));
                  else // LABELS command has been called so load the labels in as well
                    ((GUI)parent).tempDataColumns.addColumn(new MultiColumn(
                                                                  new ConcreteDataColumn((String) ((GUI)parent).tempNames.elementAt(j),(String) ((GUI)parent).tempLabels.elementAt(j)," ", date),
                                                                  new ConcreteDataColumn(),
                                                                  new ConcreteDataColumn() ));
                }
                append(jtf.getText()+"\n");
                append("Enter data as ordered Ntuples: (X1,+,-, X2,+,-,..., Remark) \n");
                setText(":");
                newLineRecognizer = "dataFullLineCollect";
              } else { // the user didn't enter column names after DATA and the DataTable is empty, give error
                append(jtf.getText()+"\n");
                append("Error: the DataTable contains no columns, you need to give column names after the DATA command\n");
                setText("DB:");
                newLineRecognizer = "";
              }
            }
            // restart the fullLine so that it can be filled in
            fullLine = "";
          }
          else if ((command.toUpperCase().equals("DELETE")) || (command.toUpperCase().equals("DELET")) ||
                  (command.toUpperCase().equals("DELE")) || (command.toUpperCase().equals("DEL")) ||
                  (command.toUpperCase().equals("DE")) || (command.toUpperCase().equals("D"))) {
            if(numTokens == 1) { // Only DELETE has been entered, rest will be taken care of at the next 'enter'
              append(jtf.getText()+"\n");
              setText("Enter NAME of the column you wish to delete:");
              newLineRecognizer = "deleteColumn"; // this is so that it is recognized once ENTER is typed again (see code below)
            }
            else { // command DELETE WITH the name of the column to delete has been entered
            }
            deleteColumn(st.nextToken().trim());
          }
          else if ((command.toUpperCase().equals("EXIT")) || (command.toUpperCase().equals("EXI")) ||
                  (command.toUpperCase().equals("EX")) || (command.toUpperCase().equals("E"))) {
            this.setVisible(false);
            append(jtf.getText()+"\n");
            setText("DB:");
          }
          else if ((command.toUpperCase().equals("QUIT")) || (command.toUpperCase().equals("QUI")) ||
                  (command.toUpperCase().equals("QU")) || (command.toUpperCase().equals("Q"))) {
            this.setVisible(false);
            append(jtf.getText()+"\n");
            setText("DB:");
          }          
          else if ( ((command.toUpperCase().equals("GENERATE")) || (command.toUpperCase().equals("GENERAT")) ||
                   (command.toUpperCase().equals("GENERA")) || (command.toUpperCase().equals("GENER")) ||
                   (command.toUpperCase().equals("GENE")) || (command.toUpperCase().equals("GEN")) ||
                   (command.toUpperCase().equals("GE")) || (command.toUpperCase().equals("G")))
                   // since it can be "generateName", "generateNumber" & "generateColumn"
                  && (newLineRecognizer.indexOf("generate")==-1) ) {
            if(numTokens == 1) { // Only GENERATE has been entered, rest will be taken care of at the next 'enter'
              append(jtf.getText()+"\n");
              setText("Enter NAME for new variable: ");
              newLineRecognizer = "generateName"; // this is so that it is recognized once ENTER is typed again (see code below)
            }
            else { // command GENERATE WITH the name of the new column has been entered
              genName = st.nextToken().trim();
              if (!genName.equals("")) { // if not an empty string, use the name
                // if no data has been previosly loaded
                int columnCount = 0;
                if (((GUI)parent).dictionary.size()!=0 && ((GUI)parent).dataTableList.getItemCount()!=0)
                  columnCount = ((DataTable)((GUI)parent).dictionary.getElement(((GUI)parent).dataTableList.getSelectedIndex())).columnCount();
                if (columnCount>0) {
                  append(jtf.getText()+"\n");
                  if (((GUI)parent).dictionary.size()==0)
                    append("(Index name is \"I\".)\n");
                  setText("Enter defining statement: ");
                  newLineRecognizer = "generateColumn"; // this is so that it is recognized once ENTER is typed again (see code below)
                } else {
                  // get the number of desired rows for the new column
                  append(jtf.getText()+"\n");
                  setText("How many new data points? ");
                  newLineRecognizer = "generateNumber"; // this is so that it is recognized once ENTER is typed again (see code below)
                }
              } else {
                append(jtf.getText()+"\n");
                setText("DB:");
                newLineRecognizer = "";
              }
            }
          }
          else if(command.toUpperCase().equals("HELP"))
          {
                append(jtf.getText()+"\n");
                
                String helpTopic = "";
                if(numTokens == 1)
                {
                    helpTopic = "prompt";
                }
                else if(numTokens == 2)
                {
                    helpTopic = st.nextToken().trim();
                }
                helpTopic = helpTopic.toLowerCase();
                
                append(getHelpText(helpTopic)+"\n");
                
                setText("DB:");
                newLineRecognizer = "";
          }
          else if ((command.toUpperCase().equals("LABELS")) || (command.toUpperCase().equals("LABEL")) ||
                  (command.toUpperCase().equals("LABE")) || (command.toUpperCase().equals("LAB")) ||
                  (command.toUpperCase().equals("LA")) || (command.toUpperCase().equals("L"))) {
            if (numTokens>1) { // there is only one label to be changed
                    // find which label is supposed to be changed, check for case sensitivity first
                    labIndex = -1;
                    String stName = st.nextToken().trim();
                    // traverse through the names of the columns in the GUI DataTable first to see if it exists
                    int dataTableKey = ((GUI)parent).getDataTableIndex();
                    for (int i=0; i<((DataTable)((GUI)parent).dictionary.getElement(dataTableKey)).columnCount(); i++) {
//System.out.println("DBConsole, Label - DataTable columnCount: "+((DataTable)((GUI)parent).dictionary.getElement(dataTableKey)).columnCount());
                      String name = ((DataTable)((GUI)parent).dictionary.getElement(dataTableKey)).getColumn(i).getName();
//System.out.println("DBConsole, Label - DataTable getName("+i+"): "+((DataTable)((GUI)parent).dictionary.getElement(dataTableKey)).getColumn(i).getName());
                      if (!((GUI)parent).caseMenuItem.getState()) {
                        if ( name.toUpperCase().equals(stName.toUpperCase()) ) {labIndex=i;}
                      } else {
                        if ( name.equals(stName) ) {labIndex=i;}
                      }
                    }
                    // If the column name has been found
                    if (labIndex!=-1) {
                      append(jtf.getText()+"\n");
                      setText(":");
                      newLineRecognizer = "labelChange";
                    } else {
                      append(jtf.getText()+"\n");
                      append("Unrecognized column name, please enter the command with the name again.\n");
                      setText("DB:");
                      newLineRecognizer = "";
                    }
//System.out.println("lab next token index: "+labIndex);
                  } else { // there is an array of new labels
                    append(jtf.getText()+"\n");
                    setText(":");
                    // Just zero the tempLabels Vector (in case it is not empty) and start loading it from the start
                    ((GUI)parent).tempLabels.clear();
                    newLineRecognizer = "labelNew";
                  }
          }
          else if (((command.toUpperCase().equals("MODIFY")) || (command.toUpperCase().equals("MODIF")) ||
                  (command.toUpperCase().equals("MODI")) || (command.toUpperCase().equals("MOD")) ||
                  (command.toUpperCase().equals("MO")) || (command.toUpperCase().equals("M")))
                  && (newLineRecognizer.indexOf("modify")==-1) ) { // since it can be "modifyWhat", "modifyColumn"
                DataTable table = (DataTable)(((GUI)parent).dictionary.getElement(((GUI)parent).getDataTableIndex()));
              if(numTokens == 1) { // Only MODIFY has been entered, rest will be taken care of at the next 'enter'
              modErr = false; // just in case it is true..
              append(jtf.getText()+"\n");
              setText("Modify WHAT? ");
              newLineRecognizer = "modifyWhat"; // this is so that it is recognized once ENTER is typed again (see code below)
            } else if(numTokens == 2) { // command MODIFY WITH a 'what to modify' has been entered
              modErr = false; // just in case it is true..
              modName = st.nextToken().trim();
              // find which column is supposed to be modified, check for case sensitivity first
              int index = ((GUI)parent).indexOfDataColumnNames(parent, modName);
              if (index!=-1) { // the column name exists
                append(jtf.getText()+"\n");
                append(" OK, ready to modify values of array "+modName+"\n");
                setText("Enter desired replacement statement: ");
                newLineRecognizer = "modifyColumn";
              } else { // ask for the column name again
                append(jtf.getText()+"\n");
                setText("Modify WHAT? ");
                newLineRecognizer = "modifyWhat"; // this is so that it is recognized once ENTER is typed again (see code below)
              }
            // This is the modify error command. It modifies the error into the given expression
            } else if(numTokens == 3) {
              modName = st.nextToken().trim();
              String err = st.nextToken().trim();
              if (err.toUpperCase().equals("ERR") ) {
                modErr = true;
                // find which column is supposed to be modified, check for case sensitivity first
                int index = ((GUI)parent).indexOfDataColumnNames(parent, modName);
                if (index!=-1) { // the column name exists
                  // Only modify if it is "real"
                  if ( (table.getColumn(index)).getType().equals(new String("real"))) {
                    append(jtf.getText()+"\n");
                    append(" OK, ready to modify the error of array "+modName+"\n");
                    setText("Enter desired replacement statement: ");
                    newLineRecognizer = "modifyColumn";
                  // if it is virual(Linear) or poisson, don't change anyting and display message
                  } else {
                    append(jtf.getText()+"\n Error: You can't modify Errors of Linear or Poisson Columns. \n");
                    setText("DB:");
                    newLineRecognizer=""; // 'restart' this value
                  }
                } else { // ask for the column name again
                  append(jtf.getText()+"\n");
                  setText("Modify WHICH error Column? ");
                  newLineRecognizer = "modifyWhat"; // this is so that it is recognized once ENTER is typed again (see code below)
                }
              // the user didn't input ERR
              } else {
                append(jtf.getText()+"\n Error: The format of the command should be MOD NAME ERR. \n");
                setText("DB:");
                newLineRecognizer=""; // 'restart' this value
              }
            }
          }
          // This returns a new DataTable that is repacked so that it calculates averages for every packBIns number of bins
          else if ((command.toUpperCase().equals("REPACK")) || (command.toUpperCase().equals("REPAC")) ||
                  (command.toUpperCase().equals("REPA")) || (command.toUpperCase().equals("REP")) ||
                  (command.toUpperCase().equals("RE")) || (command.toUpperCase().equals("R"))) {
            if(numTokens == 2) { // command REPACK WITH a packBIns number has been entered
              // since there is a new datatable being created, create a new reference date to feed into it's columns
              Date date = new Date();
              int packBins = Integer.parseInt( st.nextToken().trim() );
              int dataTableKey = ((GUI)parent).getDataTableIndex();
              DataTable dt = (DataTable)((GUI)parent).dictionary.getElement(dataTableKey);
              DataTable returnDT = new DataTable(dt.filepath);
              returnDT.setTitle(dt.getTitle()+" repacked("+packBins+")");
              int dtSize = dt.columnCount();
              for (int i =0; i<dtSize; i++) {
                //ConcreteDataColumn
                returnDT.addColumn( ((GUI)parent).repack(packBins, dt.getColumn(i), date ));
              }
              ((GUI)parent).addDataTable(returnDT, false, "" );
              append(jtf.getText()+"\n");
              append("Successfully repacked the selected Data Table. \n");
              setText("DB:");
            } else { // give the error and describe the correct format
              append(jtf.getText()+"\n");
              append("Error. The proper format is 'repack (int)n'. \n");
              setText("DB:");
            }
          }
          // This returns a new DataTable that is repacked so that it calculates averages for every packBIns number of bins
          else if ((command.toUpperCase().equals("DEFINE")) || (command.toUpperCase().equals("DEFIN")) ||
                  (command.toUpperCase().equals("DEFI")) || (command.toUpperCase().equals("DEF"))) {
            if(numTokens == 4) { // command DEFINE WITH the constant name, equal sign and the expression has been entered
              String constName = st.nextToken().trim();
              String equalSign = st.nextToken().trim(); // this is not used for anyting
              String constExpr = st.nextToken().trim();
              ((GUI)parent).dataTableVariables.setConstant(constName,constExpr,new String(""));
              append(jtf.getText()+"\n");
              append("Constant "+constName+" has been succesfully added to muView Constants table. \n");
              setText("DB:");
            } else { // give the error and describe the correct format
              append(jtf.getText()+"\n");
              append("Error. The proper format is 'define name = expr'. \n");
              setText("DB:");
            }
          }
          //This adds a new function in terms of the variable 'x'
            else if ((command.toUpperCase().equals("FUNCTION")) || (command.toUpperCase().equals("FUNCT")) ||
                  (command.toUpperCase().equals("FUN")) || (command.toUpperCase().equals("F"))) {
            if(numTokens >= 4) { // command DEFINE WITH the constant name, equal sign and the expression has been entered
              String functName = st.nextToken().trim();
              String equalSign = st.nextToken().trim(); // this is not used for anyting
              //remainder of tokens go in functExpr, which holds the expression for the function
              //in terms of the variable x
              String functExpr = st.nextToken().trim();
              while(st.hasMoreTokens())
                  functExpr = functExpr + " " + st.nextToken().trim();
              ((GUI)parent).functionsTable.setFunction(functName,functExpr);
              append(jtf.getText()+"\n");
              append("Function "+functName+" has been succesfully added to muView Functions table. \n");
              setText("DB:");
            } else { // give the error and describe the correct format
              append(jtf.getText()+"\n");
              append("Error. The proper format is 'function name = expr'. \n");
              append("The function should be defined in terms of the variable x. \n");
              setText("DB:");
            }
          }
          // the plot command is the one that 'opens' the DOPLOT prompt, so check that command first.
          // Note: this same code (more or less) is implemented in the DBFileReader Class so that files
          // containing this command would be able to be loaded in with @filename
          else if ((command.toUpperCase().equals("P")) || (command.toUpperCase().equals("PL")) ||
                  (command.toUpperCase().equals("PLO")) || (command.toUpperCase().equals("PLOT")) ) {
            if(numTokens < 3) { // PLOT has been entered without the right number of arguments
              append(jtf.getText()+"\n");
              append("Error, please enter command with two arguments: PLOT <x> <y>.\n");
              setText("DB:");
            } else { // 'PLOT x y' has been entered OR 'PLOT x y IF ..' has been entered
              // check if <x> and <y> exist, and which columns they are
              boolean checkX = false; int columnX = -1;
              boolean checkY = false; int columnY = -1;
              int dataTableKey = ((GUI)parent).getDataTableIndex();
              // get X and Y arguments
              String X = st.nextToken().trim();
              String Y = st.nextToken().trim();              
              for (int i=0; i<((DataTable)((GUI)parent).dictionary.getElement(dataTableKey)).columnCount(); i++) {
                String tempX = ((DataTable)((GUI)parent).dictionary.getElement(dataTableKey)).getColumn(i).getName();
                String tempY = ((DataTable)((GUI)parent).dictionary.getElement(dataTableKey)).getColumn(i).getName();
                // Set it to upper case if required
                if (!((GUI)parent).caseMenuItem.getState()) {
                  X = X.toUpperCase();
                  Y = Y.toUpperCase();
                  tempX = tempX.toUpperCase();
                  tempY = tempY.toUpperCase();
                }
                if( tempX.equals(X) ) {
                  checkX = true; columnX = i; }
                if( tempY.equals(Y) ) {
                  checkY = true; columnY = i; }
              }
              // If OK, put the two columns into the GUI Columns List
              if (checkX && checkY) { // if they are both true, that means that columnX and columnY values have also changed;
                //** This is the REGULAR PLOT command *//
                if(numTokens == 3) {
                  // Load the X Column
                  ((GUI)parent).addX(columnX, ((GUI)parent).layoutChoice.getSelectedIndex());
                  // Load the Y Column
                  ((GUI)parent).addY(columnY, ((GUI)parent).xAxisList.getSelectedIndex(),
                                              ((GUI)parent).layoutChoice.getSelectedIndex());
                  append(jtf.getText()+"\n");
                  setText("Point Type? "); // This will be read when the next enter key is pressed
                  newLineRecognizer="plotPointType"; // This variable will help it recognize
                //** This is the CONDITIONAL PLOT command (..IF..)*//
                } else if(numTokens > 3) {
                  if (st.nextToken().trim().toUpperCase().equals("IF")){ // check if 'IF' was spelled correctly
                    if ( ((GUI)parent).findCondColumn(s, columnX, columnY) ) {
                      append(jtf.getText()+"\n");
                      setText("Point Type? "); // This will be read when the next enter key is pressed
                      newLineRecognizer="plotPointType"; // This variable will help it recognize
                    } else {
                      append(jtf.getText()+"\n");
                      append("Warning: no columns were added since the conditions capture no data points.\n");
                      setText("DB:");
                      newLineRecognizer="";
                    }
                  } else { // 'IF' not spelled correstly
                    append(jtf.getText()+"\n");
                    append("Spelling Error (if), please re-enter command \n");
                    setText("DB:");
                    newLineRecognizer="";
                  }
                } // no else here since it has already been checked whether there is enough arguments
              } // there is no else here for unrecognized column names. The two 'if' commands below take care of that.
              if (!checkX) {
                append(jtf.getText()+"\n");
                append("Unrecognized X argument, please re-enter the command. (Check for case sensitivity)\n");
                setText("DB:");
                newLineRecognizer="";
              }
              if (!checkY) {
                append(jtf.getText()+"\n");
                append("Unrecognized Y argument, please re-enter the command. (Check for case sensitivity)\n");
                setText("DB:");
                newLineRecognizer="";
              }
            }
          }
          // The SAVE command will take the DataTable and save it in the DataTable List with
          // a separate name that the user has to enter
         /* else if ((command.toUpperCase().equals("SAVE")) || (command.toUpperCase().equals("SAV")) ||
                   (command.toUpperCase().equals("SA")) || (command.toUpperCase().equals("S")) ) {
                if(numTokens == 1) { // Only SAVE has been entered, filename will be taken care of at the next 'enter'
                  append(jtf.getText()+"\n");
                  setText("Enter the DataTable name: ");
                  newLineRecognizer = "save"; // this is so that it is recognized once ENTER is typed again (see code below)
                }
                else { // command SAVE WITH a DataTableName has been entered
                  saveDataTable(st.nextToken().trim());
                  append(jtf.getText()+"\n");
                  setText("DB:");
                }
          }*/
          else if (((command.toUpperCase().equals("S")) || (command.toUpperCase().equals("SH")) ||
                  (command.toUpperCase().equals("SHO")) || (command.toUpperCase().equals("SHOW")) )
                  && (newLineRecognizer.indexOf("show")==-1) ) { // since it can be "showWhat", "showDataAsk" & "showData"
            if(numTokens == 1) { // Only SHOW has been entered, rest will be taken care of at the next 'enter'
              append(jtf.getText()+"\n");
              setText("Show WHAT? ");
              newLineRecognizer = "showWhat"; // this is so that it is recognized once ENTER is typed again (see code below)
            }
            else if (numTokens == 2) { // command SHOW WITH a 'what to show' has been entered
              String show = st.nextToken().trim().toUpperCase();
              if (show.equals("ABSTRACT") || show.equals("ABSTR") || show.equals("A")) showAbstract();
              else if (show.equals("COMMENTS") || show.equals("COMMENT") || show.equals("C")) showComments();
              else if (show.equals("DATA") || show.equals("DAT") || show.equals("D")) showData();
              else if (show.equals("LABELS") || show.equals("LAB") || show.equals("L")) showLabels();
              else if (show.equals("TITLE") || show.equals("TITL") || show.equals("T")) showTitle();
              else {
               append(jtf.getText()+"\n");
               setText("Show WHAT? ");
               newLineRecognizer = "showWhat"; // this is so that it is recognized once ENTER is typed again (see code below)
              }
            }
            // Only DATA command has a possibility of three tokens: show DATA columnName
            else if (numTokens == 3) {
              String show = st.nextToken().trim().toUpperCase();
              // if the command is DATA do the code above, otherwise ignore all and go back to 'DB:' prompt
              if (show.equals("DATA") || show.equals("DAT") || show.equals("DA")|| show.equals("D")) {
                String colName = st.nextToken().trim();
                append(jtf.getText()+"\n");
                int dataTableKey = ((GUI)parent).getDataTableIndex();
                DataTable table = (DataTable)((GUI)parent).dictionary.getElement(dataTableKey);
                if (table.columnCount()!=0) { // the DataTable contains columns
                  //find if the column exists
                  int dataNumber = -1;
                  for (int i=0;i<table.columnCount();i++)
                    if ( ((DataColumn)table.getColumn(i)).getName()
                                                .equals(colName)) dataNumber = i;
                  if (dataNumber!=-1) { // it exists
                    showData(dataNumber);
                    newLineRecognizer=""; // 'restart' this value
                  } else { // the columnName is invalid so ask again
                    append("\n There are "+table.columnCount()+" Columns of data, named:\n");
                    for (int i=0;i<table.columnCount();i++)
                    append(((DataColumn)table.getColumn(i)).getName()+"   ");
                    append("\n");
                    setText("Do you want to see numbers? ");
                    newLineRecognizer = "showDataAsk";
                  }
                } else {
                  setText("DB:");
                  newLineRecognizer = "";
                }
              } else {
                append(jtf.getText()+"\n");
                setText("DB:");
                newLineRecognizer = "";
              }
            }
          }
          else if (((command.toUpperCase().equals("S")) || (command.toUpperCase().equals("SO")) ||
                  (command.toUpperCase().equals("SOR")) || (command.toUpperCase().equals("SORT")) )
                  && (newLineRecognizer.indexOf("show")==-1) ) { // since it can be "showWhat", "showDataAsk" & "showData"
            if(numTokens != 3) { // The 'format' of SORT command is not adequate
              append(jtf.getText()+"\n");
              append("Error: SORT command needs two variables: sort <int DataTableIndex> <ColumnName> \n");
              setText("DB:");
              newLineRecognizer=""; // 'restart' this value
            }
            // Only DATA command has a possibility of three tokens: show DATA columnName
            else if (numTokens == 3) {
              append(jtf.getText()+"\n"); // display the message below the console input line
              String numDataTable = st.nextToken().trim();
              int numberDataTable = -1;
              try {
                numberDataTable = Integer.parseInt(numDataTable);
              }  catch (NumberFormatException nfe) {
                // display the message below the console input line
                append(jtf.getText()+"\n");
                append("Error: second argument should be positive integer"+"\n");
                setText("DB:");
                newLineRecognizer=""; // 'restart' this value
                System.err.println("Error: second argument should be positive integer referning to a DataTable in the list"+ nfe.getMessage());
              }
              if ((numberDataTable>0) && (numberDataTable<=((GUI)parent).dictionary.size())) {
                String refColumn = st.nextToken().trim();
                DataTable table = (DataTable)((GUI)parent).dictionary.getElement(numberDataTable-1); // subtract one because we need the index
                int dataNumber = -1; // this will be the index of the selected column
                for (int i=0;i<(table.columnCount());i++)
                  if ( ((DataColumn)table.getColumn(i)).getName().equals(refColumn)) dataNumber = i;
                if (dataNumber!=-1) { // this means that the column name was not found in the datatable
                  setText("DB:");
                  newLineRecognizer=""; // 'restart' this value
                  ((GUI)parent).sortDataTable(table, dataNumber); // send it the datatable and the reference column number
                  newLineRecognizer=""; // 'restart' this value
                } else {
                  append(jtf.getText()+"\n");
                  append("  Column "+refColumn+" doesn't exist in the DataTable"+"\n");
                  setText("DB:");
                  newLineRecognizer=""; // 'restart' this value
                }
              } else {
                append(jtf.getText()+"\n");
                append("  Make sure the integer number refers to a DataTable in the list"+"\n");
                setText("DB:");
                newLineRecognizer=""; // 'restart' this value
              }
            }
          }
          else if ((command.toUpperCase().equals("TITLE")) || (command.toUpperCase().equals("TITL")) ||
                  (command.toUpperCase().equals("TIT")) || (command.toUpperCase().equals("TI")) ||
                  (command.toUpperCase().equals("T")) ) {
            append(jtf.getText()+"\n");
            setText(":");
            newLineRecognizer = "title";
          }
        /*  else if ((command.toUpperCase().equals("Z")) || (command.toUpperCase().equals("ZE")) ||
                  (command.toUpperCase().equals("ZER")) || (command.toUpperCase().equals("ZERO")) ) {
                append(jtf.getText()+"\n");
                setText("Clear all buffers? ");
                newLineRecognizer = "zero"; // this is so that it is recognized once ENTER is typed again (see code below)
          }*/
          //** Here are ALL OTHER lines that are typed in, after some command line was typed in 'individually'
          //    (lines that don't involve command calls)
          else {
            // When ABSTRACT command is entered, the new lines will be read into the DataTable
            if (newLineRecognizer.equals("abstract")) {
              if (jtf.getText().substring(1).equals("")) {  //note: substring(1) because of ':'
                int dataTableKey = ((GUI)parent).getDataTableIndex();
                ((DataTable)((GUI)parent).dictionary.getElement(dataTableKey)).setAbstract(abstr);
                append(jtf.getText()+"\n");
                setText("DB:");
                newLineRecognizer=""; // 'restart' this value
              }
              else {
                abstr += new String(jtf.getText().substring(1)+"\n");  //note: substring(1) because of ':'
              append(jtf.getText()+"\n");
              setText(":");
              }
            }
            // When COMMENTS command is entered, the new lines will be read into the DataTable
            else if (newLineRecognizer.equals("comments")) {
              if (jtf.getText().substring(1).equals("")) {  //note: substring(1) because of ':'
                int dataTableKey = ((GUI)parent).getDataTableIndex();
                ((DataTable)((GUI)parent).dictionary.getElement(dataTableKey)).setComment(comm);
                append(jtf.getText()+"\n");
                setText("DB:");
                newLineRecognizer=""; // 'restart' this value
              }
              else {
                comm += new String(jtf.getText().substring(1)+"\n");  //note: substring(1) because of ':'
              append(jtf.getText()+"\n");
              setText(":");
              }
            }
            // Once DATA has been called, this is where one line of data is 'collected'.
            // Note that the user can put "\" at the end of the line and continue the stream of data in the next line
            else if (newLineRecognizer.equals("dataFullLineCollect")) {
              String newLine = jtf.getText().substring(1);
              // if nothing was typed in, work is done, pop back into DB prompt.
              if (newLine.equals("")) {
                int dataTableKey = ((GUI)parent).getDataTableIndex();
                DataTable dataTable = (DataTable)((GUI)parent).dictionary.getElement(dataTableKey);
                // Load the columns in the DataTable if they don't exist
                if ( dataTable.columnCount()<=0 ) {
                  for(int i=0; i<dataTable.columnCount(); i++)
                    dataTable.addColumn((DataColumn)((GUI)parent).tempDataColumns.getColumn(i));
                }
                // Last step: show DataTable's column contents in the ColumnList
                ((GUI)parent).reloadColumnList();

                append(jtf.getText()+"\n");
                setText("DB:");
                newLineRecognizer=""; // 'restart' this value
              }
              // if the last character is "\", that means that the line with data for this row is not complete jet
              else if (String.valueOf(newLine.charAt(newLine.length()-1)).equals("\\")) {
                fullLine = fullLine.concat(newLine.substring(0, newLine.length()-1));
                append(jtf.getText()+"\n");
                setText(":");
                newLineRecognizer="dataFullLineCollect";
              // this is the last 'part' of this line, so add it to the fullLine and then load it in
              } else {
                fullLine = fullLine.concat(newLine);
                // set all the column values for this row
                ((GUI)parent).setDataVariables(fullLine);
                fullLine = ""; // Restart this walue for further loading of new lines
                append(jtf.getText()+"\n");
                setText(":");
                // keep going until an empty line occurs
                newLineRecognizer="dataFullLineCollect";
              }
            }
            // When DELETE command is called and the column name has been given
            else if (newLineRecognizer.equals("deleteColumn")) {
              deleteColumn(jtf.getText().substring(44).trim());
            }
            // When GENERATE command is called on it's own, without a name right after
            else if (newLineRecognizer.equals("generateName")) {
              // the command is a new column name! (note: first 29 chars are the prompt: "Enter NAME for new variable: ")
              genName = jtf.getText().substring(29).trim();
              if (!genName.equals("")) { // if not an empty string, use the name// if no data has been previosly loaded
                int columnCount = 0;
                if (((GUI)parent).dictionary.size()!=0 && ((GUI)parent).dataTableList.getItemCount()!=0)
                  columnCount = ((DataTable)((GUI)parent).dictionary.getElement(((GUI)parent).dataTableList.getSelectedIndex())).columnCount();
                if (columnCount>0) {
                  append(jtf.getText()+"\n");
                  if (((GUI)parent).dictionary.size()==0)
                    append("(Index name is \"I\".)\n");
                  setText("Enter defining statement: ");
                  newLineRecognizer = "generateColumn"; // this is so that it is recognized once ENTER is typed again (see code below)
                } else {
                  // get the number of desired rows for the new column
                  append(jtf.getText()+"\n");
                  setText("How many new data points? ");
                  newLineRecognizer = "generateNumber"; // this is so that it is recognized once ENTER is typed again (see code below)
                }
              } else {
                append(jtf.getText()+"\n");
                setText("DB:");
                newLineRecognizer = "";
              }
            }
            // When GENERATE is called but there was no DATA loaded before that, ask for number of rows for the new column
            else if (newLineRecognizer.equals("generateNumber")) {
              // the command is a column row number! (note: first 26 chars are the prompt: "How many new data points? ")
              String genNumber = jtf.getText().substring(26).trim();
              if (!genNumber.equals("") && Long.parseLong(genNumber)!=-1) { // if not an empty string, if it is a number
                rowCount = Long.parseLong(genNumber);
                append(jtf.getText()+"\n");
                if (((GUI)parent).dictionary.size()==0)
                    append("(Index name is \"I\".)\n");
                setText("Enter defining statement: ");
                newLineRecognizer = "generateColumn"; // this is so that it is recognized once ENTER is typed again (see code below)
              } else {
                append(jtf.getText()+"\n");
                setText("DB:");
                newLineRecognizer = "";
              }
            }
            // When GENERATE command is called and the new column name has been given
            else if (newLineRecognizer.equals("generateColumn")) {
              // the command is an expression that can contain column names
              // (note: first 26 chars are the prompt: "Enter defining statement: ")
              String genExpr = jtf.getText().substring(26).trim();
              if (!genExpr.equals("")) { // if not an empty string, use the name
                // If there has been no data table loaded at all
                // ---------------
                if (((GUI)parent).tempDataColumns==null) ((GUI)parent).tempDataColumns = new DataTable(new String(""));
                if (((GUI)parent).tempDataColumns.columnCount()==0) {
                  generateColumn(genExpr, genName, new Date() );
                } else {
                  generateColumn(genExpr, genName, ((DataColumn) ((GUI)parent).tempDataColumns.getColumn(0)).getDate() );
                }
                // --------------
                // Update this dataTable's history (for the XML output later..)
                ((GUI)parent).tempDataColumns.tableHistory.append(new String(" GEN "+genName+" "+genExpr+"\n"));
                setText("DB:");
                newLineRecognizer=""; // 'restart' this value
              } else {
                append(jtf.getText()+"\n");
                setText("DB:");
                newLineRecognizer = "";
              }
            }
            // When LABEL command is entered, with a column name, a new label for that column is requested
            else if (newLineRecognizer.equals("labelChange")) {
              // get the label, note: substring(1) because of ':'
              int dataTableKey = ((GUI)parent).getDataTableIndex();
              ((DataTable)((GUI)parent).dictionary.getElement(dataTableKey)).getColumn(labIndex)
                          .setLabel(jtf.getText().substring(1).trim());
              ((GUI)parent).reloadColumnList();
              ((GUI)parent).tempLabels.setElementAt(jtf.getText().substring(1).trim(),labIndex);
              append(jtf.getText()+"\n");
              setText("DB:");
              newLineRecognizer=""; // 'restart' this value
            }
            // When LABEL command is entered, with nothing after it, a list of column labels are taken in
            else if (newLineRecognizer.equals("labelNew")) {
              // if it is empty string, stop loading the labels
              if (jtf.getText().substring(1).equals("")) {  //note: substring(1) because of ':'
                append(jtf.getText()+"\n");
                setText("DB:");
                newLineRecognizer=""; // 'restart' this value
                // if the label array hasn't been fully filled (in case DataTable exists), fill it up with DataTable labels
                int dataTableKey = ((GUI)parent).getDataTableIndex();
                if ( ((DataTable)((GUI)parent).dictionary.getElement(dataTableKey)).columnCount()!=0 &&
                     ((DataTable)((GUI)parent).dictionary.getElement(dataTableKey)).columnCount()>((GUI)parent).tempLabels.size() ) {
                      for (int i=((GUI)parent).tempLabels.size(); i<((DataTable)((GUI)parent).dictionary.getElement(dataTableKey)).columnCount();i++)
                        ((GUI)parent).tempLabels.add(
                          i,((MultiColumn)((DataTable)((GUI)parent).dictionary.getElement(dataTableKey)).getColumn(i)).getLabel());
                }
              }
              else {
                String label = new String(jtf.getText().substring(1)+"\n");  //note: substring(1) because of ':'
                // DATA hasn't been called jet, just load the tempLabel array
                int dataTableKey = ((GUI)parent).getDataTableIndex();
                if( ((DataTable)((GUI)parent).dictionary.getElement(dataTableKey)).columnCount()==0 ) {
                        ((GUI)parent).tempLabels.addElement(label);             // Read in the lables
                // DATA has been called so put the labels into the DataColumns
                } else {
                  // if there are too many labels being called, don't load them
                  if(((GUI)parent).tempLabels.size()<((DataTable)((GUI)parent).dictionary.getElement(dataTableKey)).columnCount()) {
                    ((GUI)parent).tempLabels.addElement(label);
                    ((MultiColumn)((DataTable)((GUI)parent).dictionary.getElement(dataTableKey))
                             .getColumn(((GUI)parent).tempLabels.size()-1)).setLabel(label.trim());
                    ((GUI)parent).reloadColumnList();
                  }
                }

                append(jtf.getText()+"\n");
                setText(":");
              }
                    //addElement(new ConcreteDataColumn((String) tempNames.elementAt(i)," "))
            }
            // When MODIFY command was entered without anything after or if the next word was unrecognized
            else if (newLineRecognizer.equals("modifyWhat")) {
              // the command is a column name! (note: first 13 chars are the prompt: "Modify WHAT? ")
              modName = jtf.getText().substring(13).trim();
              // find which column is supposed to be modified, check for case sensitivity first
              int index = ((GUI)parent).indexOfDataColumnNames(parent, modName);
              if (index!=-1) { // the column name exists
                append(jtf.getText()+"\n");
                append(" OK, ready to modify values of array "+modName+"\n");
                setText("Enter desired replacement statement: ");
                newLineRecognizer = "modifyColumn";
              } else if (!modName.equals("")) { // ask for the column name again
                append(jtf.getText()+"\n");
                setText("Modify WHAT? ");
                newLineRecognizer = "modifyWhat"; // this is so that it is recognized once ENTER is typed again (see code below)
              } else { // it is an empty strin so just return to the 'DB:' prompt
                append(jtf.getText()+"\n");
                setText("DB:");
                newLineRecognizer=""; // 'restart' this value
              }
            }
            // After MODIFY has been called and the column name has been recognized, change the column
            else if (newLineRecognizer.equals("modifyColumn")) {
              // the command is the expression to use to modify the column.
              // (note: first 37 chars are the prompt: "Enter desired replacement statement: ")
              String modExpr = jtf.getText().substring(37).trim();
              if (modExpr.equals("")) { // if the line is empty, just return to the 'DB:' prompt
                append(jtf.getText()+"\n");
                setText("DB:");
                newLineRecognizer=""; // 'restart' this value
              } else {
                // modify the error columns
                if (modErr) {
                  ((GUI)parent).modifyError(modExpr, modName);
                }
                // modify the column
                else ((GUI)parent).modifyColumn(modExpr, modName);
                // Last step: show DataTable's column contents in the ColumnList
                ((GUI)parent).reloadColumnList();
                // Update this dataTable's history (for the XML output later..)
                ((GUI)parent).tempDataColumns.tableHistory.append(new String(" MOD "+modName+" "+modExpr+"\n"));

                append(jtf.getText()+"\n Modification Complete. Remember to modify LABEL! \n");
                setText("DB:");
                newLineRecognizer=""; // 'restart' this value
              }
            }
            // When PLOT command is entered, a point type will be requested
            else if(newLineRecognizer.equals("plotPointType")) {
              String type = jtf.getText().substring(12).trim(); // s: "Point Type? " (12 chars)
              // The command is a number which represents one of the point types
              if (Character.isDigit(type.charAt(0))) {
                plotPointType(type);
                // Now, it can collect more PLOT commands or 'get ready' to plot
                newLineRecognizer = "prepareDoPlot";
              } else { //it is not a number it is some mistake
                append(jtf.getText()+"\n");
                setText("Point Type? "); // This will be read when the next enter key is pressed
                newLineRecognizer="plotPointType"; // This variable will help it recognize
              }
            }
            // Just before DOPLOT
            // Note: after the 'Point type:' key stroke, the newLineRecognizer becomes "prepareDoPlot" but
            // if the new line contains the PLOT command again, it will 'enter' the 'else if' for the PLOT command
            // It only enters here when the new line is blank
            else if (command.trim()=="" && newLineRecognizer.equals("prepareDoPlot")){
              // Switch the MODE to DOPLOT
              doplot = true;
              // Start the DOPLOT prompt
              append(jtf.getText()+"\n");
              append("NOTE: The DOPLOT mode has not been implemented.\n");
              append("      To make graph settings, use the GUI's 'Plot Options' feature.\n");
              append("      To display the graph, just press ENTER.\n");
              setText("DOPLOT:");
              newLineRecognizer=""; // 'restart' this value
            }
            // When SAVE command was entered without the DataTableName, it gets it's name now
            else if (newLineRecognizer.equals("save")) {
              // the command is the DataTableName! (note: first 26 chars are the prompt: "Enter the DataTable name: ")
              saveDataTable(jtf.getText().substring(26));
              newLineRecognizer=""; // 'restart' this value
            }
            // When SHOW command was entered without anything after or if the next word was unrcognized
            else if (newLineRecognizer.equals("showWhat")) {
              // the command is the DataTableName! (note: first 11 chars are the prompt: "Show WHAT? ")
              String show = jtf.getText().substring(11).toUpperCase();
                  if (show.equals("ABSTRACT") || show.equals("ABSTR") || show.equals("A"))
                    { showAbstract(); newLineRecognizer=""; } // 'restart' this value
                  else if (show.equals("COMMENTS") || show.equals("COMMENT") || show.equals("C"))
                    { showComments(); newLineRecognizer=""; } // 'restart' this value
                  else if (show.equals("DATA") || show.equals("DAT") || show.equals("D"))
                    { showData();
                      int dataTableKey = ((GUI)parent).dictionary.getKey(((GUI)parent).dataTableList.getSelectedIndex());
          // Note, this will not happen from user input, sometimes when a new datatable is created, it doesn't get selected even when it is requested in the code
          if (dataTableKey == -1 && ((GUI)parent).dataTableList.getItemCount()>0) ((GUI)parent).dictionary.getKey(((GUI)parent).dataTableList.getItemCount()-1);
                      DataTable table = (DataTable)((GUI)parent).dictionary.getElement(dataTableKey);
                      // Note: this conditional clause is also called in showData() method, this 'if' is to 'accompany' the one in showData()
                      if (table.columnCount()!=0) { // the DataTable contains columns
                       newLineRecognizer = "showDataAsk"; // 'restart' this value
                      }
                    }
                  else if (show.equals("LABELS") || show.equals("LAB") || show.equals("L"))
                    { showLabels(); newLineRecognizer=""; } // 'restart' this value
                  else if (show.equals("TITLE") || show.equals("TITL") || show.equals("T"))
                    { showTitle(); newLineRecognizer=""; } // 'restart' this value
                  else if (show.equals("")) // just do nothing
                    { append(jtf.getText()+"\n"); setText("DB:"); newLineRecognizer="";}
                  else {
                    append(jtf.getText()+"\n");
                    setText("Show WHAT? ");
                    newLineRecognizer = "showWhat"; // this is so that it is recognized once ENTER is typed again (see code below)
                  }
            }
            // When SHOW command was entered with DATA it will ASK if it wants to 'see numbers'
            else if (newLineRecognizer.equals("showDataAsk")) {
              // s: "Do you want to see numbers? " (28 chars)
              String sStr = jtf.getText().substring(28).trim().toUpperCase() ;
              if (sStr.equals("Y") || sStr.equals("YE") || sStr.equals("YES")) {
                append(jtf.getText()+"\n");
                setText("Which data? ");
                newLineRecognizer="showData";
              } else if (sStr.equals("N") || sStr.equals("NO")) {
                append(jtf.getText()+"\n");
                setText("DB:");
                newLineRecognizer=""; // 'restart' this value
              } else {
                append(jtf.getText()+"\n");
                append("$Please answer Y or N\n");
                setText("Do you want to see numbers? ");
                newLineRecognizer = "showDataAsk";
                //setText("Clear all buffers? ");
              }
            }
            // After SHOW command, when it ASKed whether to 'see numbers' it comes here to get the data to display
            else if (newLineRecognizer.equals("showData")) {
              //  (note: first 12 chars are the prompt: "Which data? ")
              String data = jtf.getText().substring(12).trim();
              int dataNumber = -1;
              int dataTableKey = ((GUI)parent).getDataTableIndex();
              for (int i=0;i<((DataTable)((GUI)parent).dictionary.getElement(dataTableKey)).columnCount();i++) {
                if (!((GUI)parent).caseMenuItem.getState()) { // if case insensitive
                  if ( ((DataColumn)((DataTable)((GUI)parent).dictionary.getElement(dataTableKey)).getColumn(i)).getName().toUpperCase()
                                            .equals(data.toUpperCase())) dataNumber = i;
                } else {
                  if ( ((DataColumn)((DataTable)((GUI)parent).dictionary.getElement(dataTableKey)).getColumn(i)).getName()
                                            .equals(data)) dataNumber = i;
                }
              }
              if (dataNumber!=-1) {
                showData(dataNumber);
                newLineRecognizer=""; // 'restart' this value
              } else {
                append(jtf.getText()+"\n");
                append("Unrecognized data column\n");
                setText("Which data? ");
                newLineRecognizer="showData";
              }
            }
            // When TITLE command is entered, the new lines will be read into the DataTable
            else if (newLineRecognizer.equals("title")) {
              //note: substring(1) because of ':'
              int dataTableKey = ((GUI)parent).getDataTableIndex();
              ((DataTable)((GUI)parent).dictionary.getElement(dataTableKey)).setTitle(jtf.getText().substring(1));
              append(jtf.getText()+"\n");
              setText("DB:");
              newLineRecognizer=""; // 'restart' this value
            }
            // this part of code is never used
            else if (newLineRecognizer.equals("zero")) {
              String zStr = jtf.getText().substring(19).trim().toUpperCase() ; // s: "Clear all buffers? " (19 chars)
              if (zStr.equals("Y") || zStr.equals("YE") || zStr.equals("YES")) {
                zeroDataTable();
                append(jtf.getText()+"\n");
                setText("DB:");
                newLineRecognizer=""; // 'restart' this value
                ((GUI)parent).statusTextField.setText("The DataTable has been 'cleaned up' (zero-ed)");
              } else if (zStr.equals("N") || zStr.equals("NO")) {
                append(jtf.getText()+"\n");
                setText("DB:");
                newLineRecognizer=""; // 'restart' this value
              } else {
                append(jtf.getText()+"\n");
                append("$Please answer Y or N");
                setText("Clear all buffers? ");
              }
            }
            // If nothing is typed in
            else if (command.trim()=="") {
              setText("DB:");
            }
            // If something unrecognizable is typed in
            else if (command.trim()!="") {
              append("Unrecognized command: "+ jtf.getText().substring(3)+"\n");
              setText("DB:");
            }
          }
        //** It is in the DOPLOT mode (if (doplot))
        } else {
          // in case no command is entered, leave the DOPLOT mode and diplay the plot
          if (command.trim()=="") {
            // display the plot in the GUI
            append(jtf.getText()+"\n Displaying the graph... \n");
            // Switch the mode back to DB
            setText("DB:");
            doplot = false;
            ((GUI)parent).display(); // I put this at the end just because it takes a little time for the plot to load
          // The KEEP has no use (so far) in the GUI since the columns are already loaded so won't be lost
          } else if ((command.toUpperCase().equals("KEEP")) || (command.toUpperCase().equals("KEE")) ||
                     (command.toUpperCase().equals("KE")) || (command.toUpperCase().equals("K")) ) {
            // Switch the mode back to DB
            doplot = false;
            append(jtf.getText()+"\n");
            setText("DB:");
          }
        }

      jsp.setCorner("LOWER_LEFT_CORNER", jsp.getCorner("LOWER_LEFT_CORNER" ));
      } catch (Exception exc) {
                append(jtf.getText()+"\n");
                append("Error"+"\n");
                setText("DB:");
                newLineRecognizer=""; // 'restart' this value
        System.out.println(exc.getClass());
        System.out.println(exc.getMessage());
        exc.printStackTrace();
      }
    }
  }

  /**
   * This method sets the Point Type for the newly added pair of columns.
   * The point type is set in the PlotOptionsFrame of the GUI
   */
  public void plotPointType(String pointType) {

    int selectedChoiceIndex = ((GUI)parent).layoutChoice.getSelectedIndex();
//System.out.println("selectedChoiceIndex: "+ selectedChoiceIndex);
    PlotOptionsFrame optionsFrame = (PlotOptionsFrame)((Vector)((GUI)parent).plotsOptions).elementAt(selectedChoiceIndex);
    // Set the GROUP number Choice to look at the newly added
    // Note: the GRAPH Choice will always have only one item since we are adding only one y variable into X group
    //       therefore, there is no need to change this Choice
    optionsFrame.groupNumberChoice.select(optionsFrame.groupNumberChoice.getItemCount()-1);

    // this means that LINE is requested to connect the points (PointType: 2X) X-some number
    if (pointType.length()==2 && pointType.substring(0,1).equals("2")) {
      // Set the gTypeLine CheckBox in the PlotOptionsFrame to true
      optionsFrame.gTypeLine.setState(true);
      // Set the variable graphTypeLine in the PlotOptionsFrame to true
      ((Vector)optionsFrame.graphTypeLine.lastElement()).
        setElementAt( new Boolean(true), optionsFrame.graphNumberChoice.getSelectedIndex());
    }
    // If there are three elements, then the form is: (eg.) 0.2 (int)type.(int)color
    else if (pointType.length()==3 && pointType.substring(1,2).equals(".")) {
      int tempColorIndex = (int)Integer.parseInt ( pointType.substring(2,3) )%12; // there are 12 colors
      // Set the 'Color' Choice in the GUI's plotOptionsFrame Class
      optionsFrame.graphColorChoice.select(tempColorIndex);
      // Assign the color selected into the variable GUI's PlotOptionsFrame Class
      PlotOptionConstants tempConstants = new PlotOptionConstants();
      ((Vector)optionsFrame.graphColor.elementAt(optionsFrame.groupNumberChoice.getSelectedIndex())).
        setElementAt( tempConstants.COLOR[tempColorIndex], optionsFrame.graphNumberChoice.getSelectedIndex() );
    }
    // If there are five elements, then the form is: (eg.) 0.2.1 (int)type.(int)color.(int)filled
    else if (pointType.length()==5 && pointType.substring(1,2).equals(".") && pointType.substring(3,4).equals(".")) {
      int tempColorIndex = (int)Integer.parseInt  ( pointType.substring(2,3) )%12; // there are 12 colors
      // Set the 'Color' Choice in the GUI's plotOptionsFrame Class
      optionsFrame.graphColorChoice.select(tempColorIndex);
      // Assign the color selected into the variable GUI's PlotOptionsFrame Class
      PlotOptionConstants tempConstants = new PlotOptionConstants();
      ((Vector)optionsFrame.graphColor.elementAt(optionsFrame.groupNumberChoice.getSelectedIndex())).
        setElementAt( tempConstants.COLOR[tempColorIndex], optionsFrame.graphNumberChoice.getSelectedIndex() );
      // set the Symbol to be filled
      if ( pointType.substring(4,5).equals("1") ) {
        optionsFrame.filledSymbol.setState(true);
        ((Vector)optionsFrame.graphSymbolFilled.elementAt(optionsFrame.groupNumberChoice.getSelectedIndex())).
          setElementAt( new Boolean(true), optionsFrame.graphNumberChoice.getSelectedIndex() );
      } else if  ( pointType.substring(4,5).equals("0") ) {
        optionsFrame.filledSymbol.setState(false);
        ((Vector)optionsFrame.graphSymbolFilled.elementAt(optionsFrame.groupNumberChoice.getSelectedIndex())).
          setElementAt( new Boolean(false), optionsFrame.graphNumberChoice.getSelectedIndex() );
      }
    }
    // this means that the points WON'T be connected with a LINE
    // Note: the code is the same as the one right above, with false instead of true
    else if (pointType.length()==1 || pointType.length()==3|| pointType.length()==5) {
//System.out.println("optionsFrame.graphTypeLine.size(): "+optionsFrame.graphTypeLine.size());
      // Set the gTypeLine CheckBox in the PlotOptionsFrame to false
      optionsFrame.gTypeLine.setState(false);
      // Set the variable graphTypeLine in the PlotOptionsFrame to false
      ((Vector)optionsFrame.graphTypeLine.lastElement()).
        setElementAt( new Boolean(false), optionsFrame.graphNumberChoice.getSelectedIndex());
    }
/*public static final String POINT_TYPE[] =
    { "square", "circle", "triangle", "nabla", "diamond", "star", "cross", "heart" };*/

    if (pointType.equals("-1")) { // a dot (note: it is a two character string, compared to other numbers)
      setTypeOther("dot", optionsFrame);
      // NOTE:   THREE CONDITIONS: "just number"(symbol) || "two numbers"(line and symbol) || "two numbers with a dot in between" (symbol and color)
     // a SQUARE
    } else if (pointType.equals("0") || (pointType.length()==2 && pointType.substring(1).equals("0"))
                                                 || (pointType.length()==3 && pointType.substring(0,1).equals("0"))
                                                 || (pointType.length()==5 && pointType.substring(0,1).equals("0"))  ) {
      setTypeOther("square", optionsFrame);
    // a CIRCLE
    } else if (pointType.equals("1") || (pointType.length()==2 && pointType.substring(1).equals("1"))
                                                 || (pointType.length()==3 && pointType.substring(0,1).equals("1"))
                                                 || (pointType.length()==5 && pointType.substring(0,1).equals("1"))  ) {
      setTypeOther("circle", optionsFrame);
    // a TRIANGLE
    } else if (pointType.equals("2") || (pointType.length()==2 && pointType.substring(1).equals("2"))
                                                 || (pointType.length()==3 && pointType.substring(0,1).equals("2"))
                                                 || (pointType.length()==5 && pointType.substring(0,1).equals("2"))  ) {
      setTypeOther("triangle", optionsFrame);
    // a NABLA (upside-down triangle)
    } else if (pointType.equals("3") || (pointType.length()==2 && pointType.substring(1).equals("3"))
                                                 || (pointType.length()==3 && pointType.substring(0,1).equals("3"))
                                                 || (pointType.length()==5 && pointType.substring(0,1).equals("3"))  ) {
      setTypeOther("nabla", optionsFrame);
    // a DIAMOND
    } else if (pointType.equals("4") || (pointType.length()==2 && pointType.substring(1).equals("4"))
                                                 || (pointType.length()==3 && pointType.substring(0,1).equals("4"))
                                                 || (pointType.length()==5 && pointType.substring(0,1).equals("4"))  ) {
      setTypeOther("diamond", optionsFrame);
    // a STAR
    } else if (pointType.equals("5") || (pointType.length()==2 && pointType.substring(1).equals("5"))
                                                 || (pointType.length()==3 && pointType.substring(0,1).equals("5"))
                                                 || (pointType.length()==5 && pointType.substring(0,1).equals("5"))  ) {
      setTypeOther("star", optionsFrame);
    // a CROSS
    } else if (pointType.equals("6") || (pointType.length()==2 && pointType.substring(1).equals("6"))
                                                 || (pointType.length()==3 && pointType.substring(0,1).equals("6"))
                                                 || (pointType.length()==5 && pointType.substring(0,1).equals("6"))  ) {
      setTypeOther("cross", optionsFrame);
    // a HEART
    } else if (pointType.equals("7") || (pointType.length()==2 && pointType.substring(1).equals("7"))
                                                 || (pointType.length()==3 && pointType.substring(0,1).equals("7"))
                                                 || (pointType.length()==5 && pointType.substring(0,1).equals("7"))  ) {
      setTypeOther("heart", optionsFrame);
    }
    append(jtf.getText()+"\n");
    setText("DB:");
  }
  /**
   * This method is used by the plotPointType() method to change the required settings in the GUI'sPlotOptionsFrame
   */
  public void setTypeOther(String type, PlotOptionsFrame optionsFrame) {
    // Set the 'Type Other' Choice in the GUI's plotOptionsFrame Class
    optionsFrame.gTypeOtherChoice.select(type);
    // Assign the type of graph selected into the variable GUI's PlotOptionsFrame Class
    ((Vector)optionsFrame.graphTypeOther.elementAt(optionsFrame.groupNumberChoice.getSelectedIndex())).
      setElementAt(type, optionsFrame.graphNumberChoice.getSelectedIndex() );
  }

  /**
   * This code is never used
   * This method takes the DataTable in the GUI DataTable List and saves it as a separate DataTable
   * with the specified name
   */
  public void saveDataTable(String name) {
    try {
    int dataTableKey = ((GUI)parent).getDataTableIndex();
    // Put the DataTable into the dictionary and the dataTable List
    ((GUI)parent).dictionary.addElement(
      (DataTable)(ObjectCloner.deepCopy( ((GUI)parent).dictionary.getElement(dataTableKey) )) );
    ((GUI)parent).dataTableList.add(String.valueOf("File: "+name+"; Title: "+
          ((DataTable)((GUI)parent).dictionary.getElement(dataTableKey)).getTitle()));  // Add it to the List
    ((GUI)parent).dataTableList.select(((GUI)parent).dataTableList.getItemCount()-1);                // Select it
    } catch (Exception e) { System.err.println(e.getMessage()); }
    append(jtf.getText()+"\n");
    setText("DB:");
    newLineRecognizer=""; // 'restart' this value
  }

  /**
   * This Method displays the Abstract in the DBConsole
   */
  public void showAbstract() {
    append(jtf.getText()+"\n");
    int dataTableKey = ((GUI)parent).getDataTableIndex();
    append("\n"+((DataTable)((GUI)parent).dictionary.getElement(dataTableKey)).getAbstract()+"\n");
    setText("DB:");
  }
  /**
   * This Method displays the Comments in the DBConsole
   */
  public void showComments() {
    append(jtf.getText()+"\n");
    int dataTableKey = ((GUI)parent).getDataTableIndex();
    append("\n"+((DataTable)((GUI)parent).dictionary.getElement(dataTableKey)).getComment()+"\n");
    setText("DB:");
  }
  /**
   * This Method displays the Data in the DBConsole
   */
  public void showData() {
    append(jtf.getText()+"\n");
    int dataTableKey = ((GUI)parent).getDataTableIndex();
    DataTable table = (DataTable)((GUI)parent).dictionary.getElement(dataTableKey);
    if (table.columnCount()!=0) { // the DataTable contains columns
      append("\n There are "+table.columnCount()+" Columns of data, named:\n");
      for (int i=0;i<table.columnCount();i++)
        append(((DataColumn)table.getColumn(i)).getName()+"   ");
      append("\n");
      setText("Do you want to see numbers? ");
      newLineRecognizer = "showDataAsk";
    } else {
      setText("DB:");
      newLineRecognizer = "";
    }
  }
  /**
   * This Method displays the Data in the DBConsole
   */
  public void showData(int i) {
    int dataTableKey = ((GUI)parent).getDataTableIndex();
    DataTable table = (DataTable)((GUI)parent).dictionary.getElement(dataTableKey);
    for (int j=0;j<((DataColumn)table.getColumn(i)).getSize();j++)
      append( "["+table.getColumn(i).getValue(j)+", "+
                      table.getColumn(i).getErrorPlus(j)+", "+
                      table.getColumn(i).getErrorMinus(j)+"]\n");
    setText("DB:");
  }
  /**
   * This Method displays the Labels in the DBConsole
   */
  public void showLabels() {
    append(jtf.getText()+"\n\n");
    int dataTableKey = ((GUI)parent).getDataTableIndex();
    for (int i=0;i<((DataTable)((GUI)parent).dictionary.getElement(dataTableKey)).columnCount();i++)
      append(((DataTable)((GUI)parent).dictionary.getElement(dataTableKey)).getColumn(i).getLabel()+"\n");
    setText("DB:");
  }
  /**
   * This Method displays the Title in the DBConsole
   */
  public void showTitle() {
    append(jtf.getText()+"\n");
    int dataTableKey = ((GUI)parent).getDataTableIndex();
    append("\n"+((DataTable)((GUI)parent).dictionary.getElement(dataTableKey)).getTitle()+"\n");
    setText("DB:");
  }

  /**
   *  this code is never used
   * This method "cleans up" the DataTable in the GUI. It is used when the ZERO command is called.
   */
  public void zeroDataTable() {
                // ****** this part of code erases all the associated x-axes from the xAxisList
                        // this filename is for the purposes of erasing every X and Y axis in the Axis list associated with the DataTable to be erased
                        String fileName = "default";
                        // Erase all the associated X and Y colums from the xAxisDataTable and yAxisDataTable
                        //  browse through the xAxis List and while there is a column that contains the title of this DataTable, erase it.
                        if (((GUI)parent).xAxisList.getItemCount() > 0) {
                          boolean noMore = false; // "noMore" means that there is no more x Axis Columns associated with this DataTable
                          int eraseIndex = -1;  // this is the index that needs to be erased
                          while (!noMore) { //means that there still might be more columns ( in the newly created (reduced) xAxisList)
                            eraseIndex = -1; //reset the eraseIndex Value
                            for (int i = 0; i<((GUI)parent).xAxisList.getItemCount(); i++) {
                                ((GUI)parent).xAxisList.select(i);
                              if (((GUI)parent).xAxisList.getSelectedItem().indexOf(fileName)!=-1) { // check if the filename is contained in the ListColumn
                                eraseIndex = ((GUI)parent).xAxisList.getSelectedIndex();
                                break;
                              }
                            }
//System.out.println(((GUI)parent).xAxisList.getSelectedItem()+" "+fileName+" "+eraseIndex);
                            if (eraseIndex!=-1) {
                              ((GUI)parent).removexAxisColumn(eraseIndex);
                              ((GUI)parent).reloadxAxisList();
                              ((GUI)parent).reloadyAxisList();
                            } else
                              noMore=true;
                            if (((GUI)parent).xAxisList.getItemCount() == 0) noMore=true; // this is in case the xAxisLIst empties out
                          }
                          if (((GUI)parent).xAxisList.getItemCount() > 0) // if there are still some elements, select the last one
                            ((GUI)parent).xAxisList.select(((GUI)parent).xAxisList.getItemCount());
                        }
            // ********* //

    int dataTableKey = ((GUI)parent).getDataTableIndex();
    ((DataTable)((GUI)parent).dictionary.getElement(dataTableKey)).setAbstract("");
    ((DataTable)((GUI)parent).dictionary.getElement(dataTableKey)).setComment("");
    ((DataTable)((GUI)parent).dictionary.getElement(dataTableKey)).setTitle("");
    ((DataTable)((GUI)parent).dictionary.getElement(dataTableKey)).removeAllColumns();
    ((GUI)parent).reloadColumnList();
    // clear all the local variables too
    ((GUI)parent).tempDataColumns = new DataTable(" ");
    ((GUI)parent).tempLabels.clear(); // removes any labels for this default DataTable
    ((GUI)parent).tempNames.clear();
    rowCount=0;
    fullLine = ""; // just in case..
  }
  /**
   * Invoked when a key has been typed.
   * This event occurs when a key press is followed by a key release.
   */
   public void keyTyped(KeyEvent e){}

   /**
    * Invoked when a key has been pressed.
    */
   public void keyPressed(KeyEvent e){processConsole(e);}
   /**
   * Invoked when a key has been released.
   */
   public void keyReleased(KeyEvent e){}


   /**
    * This method deletes a column in the GUI
    */
   public void deleteColumn(String delCol) {
              if (!delCol.equals("")) { // if not an empty string, use the name
                  // find it's index, check for case sensitivity
                  int index = ((GUI)parent).indexOfDataColumnNames(parent, delCol);
                  if (index==-1) {
                    jta.setText("Unrecognized Column name.");
                  } else {
                    // remove the name from the array (note: this will be done again when reloadColumnList in the GUI is called)
                    ((GUI)parent).tempNames.removeElementAt(index);
                    // remove the label (note: this will be done again when reloadColumnList in the GUI is called)
                    ((GUI)parent).tempLabels.removeElementAt(index);
                    // delete the column0
                    int dataTableKey = ((GUI)parent).getDataTableIndex();
                    // --------------
                    ((DataTable)((GUI)parent).dictionary.getElement(dataTableKey)).removeColumnAt(index);
                    // --------------
                    // reload the Column list in the GUI
                    ((GUI)parent).reloadColumnList();
                  }
                setText("DB:");
                newLineRecognizer=""; // 'restart' this value
              } else {
                append(jtf.getText()+"\n");
                setText("DB:");
                newLineRecognizer = "";
              }
      }

   /**
    * This method generates a completely new column in the GUI
    */
   public void generateColumn(String genExpr, String genName, Date date) {
     // If this name already exists as a column name, don't allow the user to generate it
     if (((GUI)parent).tempNames.indexOf(genName)!=-1 ) {// this means that there is a same name that was already loaded
       append(jtf.getText()+"\n Error: Generating a column with a name that already exists: "+genName+ ". Please use a different name. \n");
       setText("DB:");
       newLineRecognizer=""; // 'restart' this value
     } else {
           // if case insensitive, convert to upper case;
           // if (!((GUI)parent).caseMenuItem.getState()) genName = genName.toUpperCase();
           // add the new name into the array
           ((GUI)parent).tempNames.addElement(genName);
           // add an new dummy label which should be changed in the db file right after
           ((GUI)parent).tempLabels.addElement(genName);
           // if there are existing columns in the DataTable then adjust the rowCount value
           // otherwise, this means that a column is being GENerated from scratch, so
           // this value was updated when the user was asked to enter the number of data points after calling the GEN command.
           // So DON'T go in. two reasons: no need to change it & an exception will be thrown since there ARE no columns
          if (((GUI)parent).tempDataColumns.columnCount()!=0)
            rowCount = ((DataColumn) ((GUI)parent).tempDataColumns.getColumn(0)).getSize();
          // create the new column
          ((GUI)parent).tempDataColumns.addColumn( new MultiColumn(
                                                                 new ConcreteDataColumn(genName, genName, " ", date),
                                                                 new ConcreteDataColumn(),
                                                                 new ConcreteDataColumn() ) );
                  // create dummy rows in the columns and then 'modify' them using the modifyColumn() method
//System.out.println("  rowCount: "+rowCount);
        for (int i=0; i<rowCount; i++) {
                        ((ConcreteDataColumn) ((MultiColumn)((GUI)parent).tempDataColumns.getColumn(((GUI)parent).tempDataColumns.columnCount()-1)).getValueColumn()).addElement("0");// just add a dummy zero string
                        ((ConcreteDataColumn) ((MultiColumn)((GUI)parent).tempDataColumns.getColumn(((GUI)parent).tempDataColumns.columnCount()-1)).getErrorMinusColumn()).addElement("0");
                        ((ConcreteDataColumn) ((MultiColumn)((GUI)parent).tempDataColumns.getColumn(((GUI)parent).tempDataColumns.columnCount()-1)).getErrorPlusColumn()).addElement("0");
        }
        // in case the columns is being generated for the first time, the genExpr could contain references to 'i'. The value 'i'=i+1

        ((GUI)parent).modifyColumn(genExpr, genName);
        // Last step: show dataTable's column contents in the ColumnList

        ((GUI)parent).reloadColumnList();

        append(jtf.getText()+"\n Data Generation Complete. Remember to Add a LABEL! \n");
        newLineRecognizer=""; // 'restart' this value
      }
    }


  public static void main(String[] args) {
  }
  
  /**
   *Updates the text of jtf (the text field) with the supplied text, and moves the position of the input text caret accordingly.
   */
  public void setText(String text)
  {
        jtf.setText(text);
        jtf.setCaretPosition(jtf.getText().length());
  }
  
  /**
   *Appends the given text to the end of jta (the text area) and shifts the existing text down accordingly. 
   */
  public void append(String text)
  {
      jta.append(text);
  }
  
  /**
   *Returns an explanation of the commmand-line function associated with the given topic.
   *
   *The following are the complete set of valid input (that is, 'topic' must be equal() to one of the following strings:
   *    data
   *    cd
   *    delete
   *    exit
   *    generate
   *    label
   *    modify
   *    repack
   *    define
   *    function
   *    plot
   *    show
   *    sort
   */
  public String getHelpText(String topic)
  {
      String helpText = "";
      if(topic.equals("data"))
      {
          helpText = "This command adds data points to the specified columns \n" +
                  "of the data table.\n" +
                  "The format for this command is as follows:\n" +
                  "\tdata <a> <b> <c> (...)\n" +
                  "Here, <a>, <b>, and <c> are the names of the columns to which to add data points.\n" +
                  "There may be any number of columns specified here.";
      }
      else if(topic.equals("cd"))
      {
          helpText = "This command changes the current working directory.\n" +
                  "The format for this command is as follows:\n" +
                  "\tcd <x>\n" +
                  "Here, <x> is the path for the new working directory.";
      }
      else if(topic.equals("delete"))
      {
          helpText = "This command deletes a column from the data table.\n" +
                  "The format for this command is as follows:\n" +
                  "\tdelete <x>\n" +
                  "Here, <x> is the name of the column to be deleted.";
      }
      else if(topic.equals("exit"))
      {
          helpText = "This command closes the command line console.";
      }
      else if(topic.equals("generate"))
      {
          helpText = "This command generates a new column from existing columns.\n" +
                  "The format for this command is as follows:\n" +
                  "\tgenerate <x> <y>\n" +
                  "Here, <x> is the name of the column to be generated, and <y> is the\n" +
                  "expression corresponding to the new column.";
      }
      else if(topic.equals("label"))
      {
          helpText = "This command modifies the label of the specified data column.\n" +
                  "The format for this command is as follows:\n" +
                  "\tlabel <x>\n" +
                  "Here, <x> is the name of the column to be re-labelled.";
      }
      else if(topic.equals("modify"))
      {
          helpText = "This command modifies the contents of the specified data column.\n" +
                  "The format for this command is as follows:\n" +
                  "\tmodify <x>\n" +
                  "\tor\n" +
                  "\tmodify <x> err\n" +
                  "Here, <x> is the name of the column to be modified, and including 'err'\n" +
                  "specifies that the corresponding error column is to be modified.";
      }
      else if(topic.equals("repack"))
      {
          helpText = "This command repacks the bins of the data table.\n" +
                  "The format for this command is as follows:\n" +
                  "\trepack <n>\n" +
                  "Here, <n> is the number of bins to be repacked into each new bin.\n";
      }
      else if(topic.equals("define"))
      {
          helpText = "This command defines a new constant.\n" +
                  "The format for this command is as follows:\n" +
                  "\tdefine <c> = <y>\n" +
                  "Here, <c> is the name of the new constant, and <y> is an expression\n" +
                  "for that constant.";
      }
      else if(topic.equals("plot"))
      {
          helpText = "This command plots data columns.\n" +
                  "The format for this command is as follows:\n" +
                  "\tplot <x> <y>\n" +
                  "Here, <x> is the column to be used as the x-axis of the plot, and\n" +
                  "<y> is the column to be used as the y-axis.";
      }
      else if(topic.equals("function"))
      {
          helpText = "This command defines a new function.\n" +
                  "The format for this command is as follows:\n" +
                  "\tfunction <f> = <y>\n" +
                  "Here, <f> is the name of the new function, and <y> is an expression\n" +
                  "for that function in terms of the algebraic variable x.";
      }
      else if(topic.equals("show"))
      {
          helpText = "This command displays the attributes of the current table.\n" +
                  "The format for this command is as follows:\n" +
                  "\tshow <x>\n" +
                  "Here, <x> is one of 'abstract', 'comments', 'data', 'label', or 'title'.";
      }
      else if(topic.equals("sort"))
      {
          helpText = "This command sorts a data table in ascending order for a specified column.\n" +
                  "The format for this command is as follows:\n" +
                  "\tsort <x> <y>\n" +
                  "Here, <x> is the index of the data table to be sorted, and <y> is the name of the\n" +
                  "column within the specified data table.";
      }
      else if(topic.equals("prompt"))   //This occurs if the user has not entered any topic at all -
      {                                 //it prompts the user to enter a specific help topic.
          helpText = "To learn more about a particular command, use the following format:\n" +
                  "\thelp <x>\n" +
                  "Here, <x> is the command. The following commands are recognized:\n" +
                  "data, cd, delete, exit, generate, label, modify, repack, define,\n" +
                  "function, plot, show, sort.";
      }
      else
      {
          helpText = "Please select a valid topic.";
      }
      return helpText;
  }
}
