package musr.client.gui;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.lang.*;
import java.net.URL;
import java.util.*;
import javax.swing.*;
import musr.client.datacolumn.*;

/**
 * This Dialog so far has few uses:
 *  - to display a message or bunch of text lines (if nrows==-1, and with method addLine()
 *  - to display the MANUAL.html page from the Help menu (String entry is in this case URL path to thr file)
 *  - to dislay a whole DataTable like a spreadsheet (if nrows and ncols !=-1, with method add DataTable()
 *
 * @author Ivan Tucakov, Nikola Tucakov
 */
public class InfoDialog extends JDialog {
  protected Button button;            // The OK button of the dialog
  protected TextArea messageArea;     // The message displayed by the dialog

  public ScrollPane dataScrollPane;
  private Panel dataPanel;
  int nrows, ncols;


/**
 *  This subclass was created to allow the protected 'scrollToReference() method to be used
 *  I won't use it because I will load the html file in a different way
 */
 private class JEditorPaneGUI extends JEditorPane implements java.io.Serializable {
  static final long serialVersionUID = 0L; // SUID
      JEditorPaneGUI() {
          super();
      }

     // This is the reason why the whole subclass was made
     // This method scrolls the URL page to a desired text
     public void scrollToReference(String reference) {
       super.scrollToReference(reference);
     }
     // This is the method that loads the URL, usually, the JEditorPane loads it in the constructor, but there is 'exception' issues..
     public InputStream getStream(URL url) {
       InputStream is = null;
       try {
         is = super.getStream(url);
       } catch (Exception me) {
        System.err.println(me.getMessage());
       }
       return is;
     }
}
public JEditorPaneGUI manPaneG = new JEditorPaneGUI();

  public InfoDialog(Frame parent, String title, int nrows, int ncols) {
 		super(parent, title, false);  // set the Dialog not to be modal (so that other frames CAN be accessed)

    this.getContentPane().setLayout(new BorderLayout());


    // if there are no rows, then just add a regular TextArea
    if (nrows==-1) {
      messageArea = new TextArea();
      messageArea.setEditable(false);
      this.getContentPane().add(messageArea);
    // NOTE: this part of the code is not used anymore because of AppletContext code u sed in the GUI for the same purpose
    // This is for MANUAL.html display
    } else if (nrows==-2){
      try {
        JEditorPane manPane = new JEditorPane(new java.net.URL("http://cmms.triumf.ca/~itucakov/html/MANUAL.html"));
                                                                      //this.getClass().getResource("http://cmms.triumf.ca/~itucakov/html/MANUAL.html"));
        manPane.setVisible(true);
        dataScrollPane = new ScrollPane(ScrollPane.SCROLLBARS_ALWAYS);
        dataScrollPane.setSize(900,700);
        dataScrollPane.add(manPane);
        dataScrollPane.enableInputMethods(false);
        this.getContentPane().add(dataScrollPane, "North");
        //manPane.scrollToReference("Command Line Console");
      } catch (Exception me) {
        System.err.println("Error reading MANUAL.html file: "+ me.getMessage());
      }

    // NOTE: this part of the code is not used anymore because of AppletContext code u sed in the GUI for the same purpose
    // This is for "Console Instructions" display (it is the same as the MANUAL.html display except that it scrolls to the Console instructions.
    } else if (nrows==-3){
      try {
        URL manual = new java.net.URL("http://cmms.triumf.ca/~itucakov/html/MANUAL.html");
        manPaneG.getStream(manual);
        manPaneG.setPage("http://cmms.triumf.ca/~itucakov/html/MANUAL.html#4.1_Command_Line_Console");
        manPaneG.setVisible(true);
        dataScrollPane = new ScrollPane(ScrollPane.SCROLLBARS_ALWAYS);
        dataScrollPane.setSize(900,700);
        dataScrollPane.add(manPaneG);
        dataScrollPane.setScrollPosition(50, 150);
        manPaneG.scrollToReference("#4.1_Command_Line_Console");
        manPaneG.enableInputMethods(false);
        this.getContentPane().add(dataScrollPane, "North");
      } catch (Exception me) {
        System.err.println("Error reading Console Instructions: "+ me.getMessage());
      }



    // This is for Data spreadsheet display
    } else {
      dataScrollPane = new ScrollPane(ScrollPane.SCROLLBARS_ALWAYS);
      // '+1' is because there is column title at the top
      GridLayout grid = new GridLayout(nrows+1, ncols);
      grid.setVgap(1);
      dataPanel = new Panel(grid);
      dataScrollPane.add(dataPanel);
      dataScrollPane.setSize(500,400);
      this.getContentPane().add(dataScrollPane, "North");
      this.nrows = nrows;
      this.ncols =ncols;
    }


    button = new Button("OK");
    ActionListener buttonListener = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        InfoDialog.this.dispose();   // pop down dialog
      }
    };
    button.addActionListener(buttonListener);
    JPanel p = new JPanel();
    p.setLayout(new FlowLayout(FlowLayout.CENTER, 10, 10));
    p.add(button);
    this.getContentPane().add(p, "South");
    // Set the dialog size to the preferred size of its components
    this.pack();
  }

  /** this method just adds the line into the TextArea */
  public void addLine(String line) { messageArea.append(line+"\n"); }

  /** This method loads the dataPanel Panel with all the DataTable names and values
   *  Note: it loads from left to right */
  public void addDataTable(DataTable dataTable) {
    // Add column names
    for(int j=0; j<ncols; j++) dataPanel.add(new TextField(dataTable.getColumn(j).getName(),
                                                           dataTable.getColumn(j).getName().length() ));
    // Add all values with the error bars if there are any
    for(int i=0; i<nrows; i++)
      for(int j=0; j<ncols; j++) {
        String value = ((DataColumn) dataTable.getColumn(j)).getValue(i).toString();
        String plusErr = "", minusErr = "";
        //If errors are zero, don't display it, just leave it blank in between the two commas
        if ( Double.parseDouble(dataTable.getColumn(j).getErrorPlus(i).toString())!=0.0 )
          plusErr = dataTable.getColumn(j).getErrorPlus(i).toString();
        if ( Double.parseDouble(dataTable.getColumn(j).getErrorMinus(i).toString())!=0.0 )
          minusErr = dataTable.getColumn(j).getErrorMinus(i).toString();
        //Measure the lenght of these three values for setting the size of the TextField
        int length = value.length()+plusErr.length()+minusErr.length()+4; // 4 is for the two commas and blank spaces
        TextField text = new TextField(value+ ", " +plusErr+ ", " +minusErr, length);
        text.setEditable(false);
        dataPanel.add(text);
      }
  }

}
