package musr.client.gui;

import java.awt.event.*;
import musr.client.function.*;
import musr.spect.*;
import java.io.*;
import java.awt.*;
import java.awt.List;
import java.util.*;
import javax.swing.*;   // for Jframe, JPanel
import java.net.URL;

import musr.client.datacolumn.*;
import musr.spect.*;
import musr.client.plot2d.*;

/**
 * Frame where the use can specify the details about MUD Spectrum to load
 *
 * @author Ivan Tucakov
 */
public class MUDSpectrumFrame extends JFrame implements Serializable {


    public static final int DIFFERENTIAL_MODE = 1;
    public static final int INTEGRAL_MODE = 2;
  // Variables

  // This is the class through which everything else is loaded
  MuSR_Server srv = new MuSR_Server();

  // GUI
  Frame parent;

  // SwingWorker which is used as a new Thread when loading the Spectrum
  SwingWorker worker;

    static String filebase="/musr/data";

    // This Dialog will contain one OK button and will be used for Displaying any kind of information,
    // error warnings or anything that would only have one OK button.
    Dialog infoDialog = new Dialog(this,"", true);
    JButton OKButton = new JButton("OK");

  // Choices
  public Choice spectrumChoice = new Choice();

  // Buttons
  JButton addRunButton = new JButton("Add Run");
  JButton addRunButton2 = new JButton("Add Run from File");
  JButton deleteRunButton = new JButton("Delete Run");
  JButton deleteAllRunsButton = new JButton("Delete All Runs");
  JButton runHeaderInfoButton = new JButton("Run Header Info");

  JButton overridesButton = new JButton("Overrides");

  JButton RRFCButton = new JButton("Complex Spectra");

  JButton loadSpectrumButton = new JButton("Load");

  JFileChooser dImportLocal = new JFileChooser();

  // Lists
  JScrollPane runsListScrollbar = new JScrollPane();
  List runsList;

  // Panel
  JPanel runsPanel;
  JPanel infoPanel;
  JPanel loadSpectrumPanel;
  JPanel CloseButtonPanel;
  
  Panel mudPanel;

  // User Options
  TextField histNumberTextField = new TextField(8);
  TextField firstBinTextField = new TextField(4);
  TextField lastBinTextField = new TextField(4);
  TextField packingTextField = new TextField(5);
  Choice firstLastChoice = new Choice();  // this will determine if the data is retreived in bins, TIme [ns] or Time [us]
  Choice packingChoice = new Choice();  // this will determine if the packing will be in  bins or TIme [ns]
  public Choice displayChoice = new Choice();  // this will determine if the display is in bins or TIme [ns] or Time [us]


  // these are used in 'Add Run'
  JPanel addRunPanel = new JPanel();
  JFrame addRunFrame = new JFrame("Add Run");
  Choice beamlineChoice;
  Choice yearChoice;
  TextField runNumberTextField;

  /** This is the RRF Complex Spectra Vector of TextFields, it dynamically changes as new histograms are requested (see complexSpectrumFrame() method)*/
  Vector phaseRRF = new Vector();
  TextField RRFFreq = new TextField();
  TextField RRFPackBin = new TextField();
  Choice RRFMenu = new Choice();

  /** Frame for Overrides Variables */
  final JFrame overridesFrame = new JFrame("Overrides");
  /** Frame for Overrides Variables */
  ScrollPane overridesFrameScrollbar = new ScrollPane();
  /** This is the bkgd1Text Vector of TextFields used for overrides, it dynamically changes as new histograms are requested (see overridesFrame() method)*/
  Vector bkgd1Text = new Vector()
  /** This is the bkgd2Text Vector of TextFields used for overrides, it dynamically changes as new histograms are requested (see overridesFrame() method)*/;
  Vector bkgd2Text = new Vector();
  /** This is the NT0 Vector of TextFields used for overrides, it dynamically changes as new histograms are requested (see overridesFrame() method)*/
  Vector overrideNT0 = new Vector();
  /** This is the NT1 Vector of TextFields used for overrides, it dynamically changes as new histograms are requested (see overridesFrame() method)*/
  Vector overrideNT1 = new Vector();
  /** This is the NT2 Vector of TextFields used for overrides, it dynamically changes as new histograms are requested (see overridesFrame() method)*/
  Vector overrideNT2 = new Vector();

  public Choice useSpectrum = new Choice();

  // GridBag
  GridBagLayout gridbag = new GridBagLayout();
  GridBagConstraints c = new GridBagConstraints();

  /** Frame for Complex Variables */
  final JFrame aFrame = new JFrame("Phases and RRF for Complex Spectra");
  /** Frame for Complex Variables */
  ScrollPane aFrameScrollbar = new ScrollPane();

  // Event Listeners used
  private class ActionListenerS implements java.awt.event.ActionListener, java.io.Serializable {
    public void actionPerformed(ActionEvent e) {}
  };
  ActionListenerS jbuttonListener;
  private class ItemListenerS implements java.awt.event.ItemListener, java.io.Serializable {
    public void itemStateChanged(ItemEvent e) {}
  };
  ItemListenerS listListener;


    private class KeyListenerS implements java.awt.event.KeyListener, java.io.Serializable {
    public void keyTyped(KeyEvent e) {}
    public void keyPressed(KeyEvent e) {}
    public void keyReleased(KeyEvent e) {}
  };
  KeyListenerS keyListener;

    /**
     *  This Dialog just shows that the Frame is busy
     */
    public Dialog busyDialog = new Dialog(this);
    Button abortButton = new Button("Abort");
    boolean abort = false;
    JProgressBar progressBar = new JProgressBar();

  /**
   * Creates the Frame
   *
   * Passes the GUI to refer to (it is being called from the GUI btw)
   */
  public MUDSpectrumFrame(Frame paren) {
      super("Load MUD Spectrum");  // set frame's title

    parent = paren;
    // ***************************
    // Create all the components:

    setBackground(Color.lightGray);
    setFont(new Font("Helvetica", Font.PLAIN, 14));

    // Create Scrollbar pane (for the purpose if the window is not big enough for all the components in it
     ScrollPane mudScrollbar = new ScrollPane();
     mudPanel = new Panel();
     mudPanel.setLayout(new BorderLayout());

    mudPanel.setBackground(Color.lightGray);
    mudScrollbar.add(mudPanel);
    getContentPane().add(mudScrollbar);

    // Some default values for the constranints object
    c.fill = GridBagConstraints.BOTH;
    c.insets = new Insets(5,5,5,5);
    c.anchor = GridBagConstraints.CENTER;

    busyDialog.setLayout(new GridLayout(4,1));
    busyDialog.add(new Label("Loading MUD Spectrum"), 0);
    busyDialog.add(new Label("not loading yet"), 1);

    //progressBar.setStringPainted(true); //get space for the string
    //progressBar.setString("");          //but don't paint it
    //progressBar.setIndeterminate(true);

    busyDialog.add(abortButton);
    abort=false;


//    busyDialog.add(new Label("Time Left: 0"), 2);
    busyDialog.setSize(400,150);
    busyDialog.setLocation( (int)this.getLocation().getX()+(int)this.getSize().getWidth()/2, (int)this.getLocation().getY()+(int)this.getSize().getHeight()-50);

    // Create the runsPanel
    runsPanel = new JPanel();
    runsPanel.setBackground(Color.lightGray);
    // Fill the runsList
    runsList = new List(7, false);
    runsList.setSize(new Dimension(150, 200));
    runsList.setBackground(Color.lightGray);
    runsPanel.setLayout(gridbag);
    // Fill runsPanel
       c.gridx = 0; c.gridy = 0;
    runsPanel.add(new Label("Runs:                             "), c);
       c.gridx = 0; c.gridy = 1;
    //runsListScrollbar.add(runsList);
    runsPanel.add(runsList, c);
       c.gridx = 0; c.gridy = 2;
    runsPanel.add(addRunButton, c);
       c.gridx = 0; c.gridy = 3;
    runsPanel.add(addRunButton2, c);
       c.gridx = 0; c.gridy = 4;
    runsPanel.add(deleteRunButton, c);
       c.gridx = 0; c.gridy = 5;
    runsPanel.add(deleteAllRunsButton, c);
       c.gridx = 0; c.gridy = 6;
    runsPanel.add(runHeaderInfoButton, c);
    // Add runsPanel to the Frame
    c.gridx = 0;    c.gridy = 0;
      mudPanel.add(runsPanel, "West");

    // Create the infoPanel
    infoPanel = new JPanel();
    infoPanel.setBackground(Color.lightGray);
    infoPanel.setLayout(gridbag);
    // Fill infoPanel
        c.gridx = 0; c.gridy = 0;
    infoPanel.add(overridesButton, c);
        c.gridx = 0; c.gridy = 1;
    infoPanel.add(new Label(" Histograms: "), c);
        c.gridx = 1; c.gridy = 1;
    histNumberTextField.setText("");
    infoPanel.add(histNumberTextField, c);
        c.gridx = 2; c.gridy = 1;
    infoPanel.add(new Label("              (integer "), c);
        c.gridx = 3; c.gridy = 1;
    infoPanel.add(new Label(" [1,2,3,4...]"), c);
        c.gridx = 4; c.gridy = 1;
    infoPanel.add(new Label("  or blank) "), c);

        c.gridx = 0; c.gridy = 2;
    infoPanel.add(new Label(" From: "), c);
        c.gridx = 1; c.gridy = 2;
    firstBinTextField.setText("");
    infoPanel.add(firstBinTextField, c);
        c.gridx = 2; c.gridy = 2;
    infoPanel.add(new Label("              To: "), c);
        c.gridx = 3; c.gridy = 2;
    lastBinTextField.setText("");
    infoPanel.add(lastBinTextField, c);
        c.gridx = 4; c.gridy = 2;
    firstLastChoice.add("Time [s]");
    firstLastChoice.add("Time [ns]");
    firstLastChoice.add("bin #");
    firstLastChoice.select(0);
    infoPanel.add(firstLastChoice, c);

        c.gridx = 0; c.gridy = 3;
    infoPanel.add(new Label(" Packing: "), c);
        c.gridx = 1; c.gridy = 3;
    packingTextField.setText("");
    infoPanel.add(packingTextField, c);
        c.gridx = 2; c.gridy = 3;
    packingChoice.add("Time [ns]");
    packingChoice.add("# bins");
    infoPanel.add(packingChoice, c);
        c.gridx = 3; c.gridy = 3;
    infoPanel.add(new Label(" Display in: "), c);
        c.gridx = 4; c.gridy = 3;
    displayChoice.add("(default)");
    displayChoice.add("Time [s]");
    displayChoice.add("Time [ns]");
    displayChoice.add("bin #");
    infoPanel.add(displayChoice, c);

        c.gridx = 0; c.gridy = 4;
    infoPanel.add(RRFCButton, c);


    // Add infoPanel to the Frame
      mudPanel.add(infoPanel, "Center");

    // Create loadSpectrumPanel
    loadSpectrumPanel = new JPanel();
    loadSpectrumPanel.setBackground(Color.lightGray);
    loadSpectrumPanel.setLayout(gridbag);
    // Fill loadSpectrumPanel
        c.gridx = 0; c.gridy = 0;
    loadSpectrumPanel.add(new Label("Load Spectrum Type: "), c);
        c.gridx = 1; c.gridy = 0;
    loadSpectrumPanel.add(new Label("           "), c);
        c.gridx = 0; c.gridy = 1;
    spectrumChoice.add("Raw Histogram(s)");
    spectrumChoice.add("Early Bins");
    spectrumChoice.add("Late Bins");
    spectrumChoice.add("Background-Corrected Histogram(s)");
    spectrumChoice.add("Asymmetry from Each Histogram");
    spectrumChoice.add("Asym from Each Bkg-Corr Histogram");
    spectrumChoice.add("2-Histogram Asymmetry (i-j)(i+j)");
    spectrumChoice.add("4-Histogram Complex Asymmetry in RRF");
    spectrumChoice.add("n-Hist Complex Asym w/Phases in RRF");
/*
    spectrumChoice.add("Asy=(Fwd/Bwd)");
    spectrumChoice.add("Complex Asy in RRF");*/
    loadSpectrumPanel.add(spectrumChoice, c);
        c.gridx = 1; c.gridy = 1;
    loadSpectrumPanel.add(loadSpectrumButton, c);
    // Add loadSpectrumPanel to the Frame
      mudPanel.add(loadSpectrumPanel, "South");


    setSize((int)((GUI)parent).screenDimen.getWidth()*5/7, (int)((GUI)parent).screenDimen.getHeight()/2);  // setSize(1000,550);
    // pack();

    // ** Create Add Run Frame... **  //
           addRunPanel.setLayout(new GridBagLayout());
          // Open a new panel for the user to enter run info
          addRunPanel.setBackground(Color.lightGray);
          addRunPanel.setSize(250, 250);
          addRunFrame.setSize(250,250);
          addRunFrame.getContentPane().add(addRunPanel);
                                            c.gridx = 0; c.gridy = 0;
                addRunPanel.add(new Label("     "), c);
		c.gridx = 0; c.gridy = 1;
                addRunPanel.add(new Label("Beamline:"), c);
		c.gridx = 0; c.gridy = 2;
                addRunPanel.add(new Label("Year:"), c);
		c.gridx = 0; c.gridy = 3;
                addRunPanel.add(new Label("Run #:"), c);
		c.gridx = 0; c.gridy = 4;
                Button okButton = new Button("OK");
                addRunPanel.add(okButton, c);
                                            c.gridx = 1; c.gridy = 0;
                addRunPanel.add(new Label("Add Run into the Runs List:"),c);
		c.gridx = 1; c.gridy = 1;
                beamlineChoice = new Choice();
		beamlineChoice.add("M20");
		beamlineChoice.add("M15");
		beamlineChoice.add("M13");
		beamlineChoice.add("M9");
		beamlineChoice.add("BNMR");
		beamlineChoice.add("BNQR");
		beamlineChoice.add("ISAC");
                addRunPanel.add(beamlineChoice, c);
		c.gridx = 1; c.gridy = 2;
                yearChoice = new Choice();
	for(int i=Calendar.getInstance().get(Calendar.YEAR); i>=1974; i--) yearChoice.add(Integer.toString(i));
                addRunPanel.add(yearChoice, c);
		c.gridx = 1; c.gridy = 3;
                runNumberTextField = new TextField(5);
                addRunPanel.add(runNumberTextField, c);

		c.gridx = 1; c.gridy = 4;
                Button cancelButton = new Button("Cancel");
                addRunPanel.add(cancelButton, c);

                addRunFrame.pack();

                // RRF fields
                useSpectrum.add("ASY1BC");
                useSpectrum.add("ASY1");
                RRFMenu.add("RRF Periods");
                RRFMenu.add("ns");
                RRFMenu.add("s");
                RRFMenu.add("# Bins");

                        ActionListenerS buttonlistenerTypes = new ActionListenerS() {
                         public void actionPerformed(ActionEvent e) {
                          if (e.getActionCommand().equals("OK")) {
                            // Load the run info into the runsList
                            runsList.add(String.valueOf(beamlineChoice.getSelectedItem())+" "+String.valueOf(yearChoice.getSelectedItem())+
                                                               " "+String.valueOf(runNumberTextField.getText()));
                            runsList.select(runsList.getItemCount()-1);
                            addRunFrame.hide(); // and close the Frame
                           } else if (e.getActionCommand().equals("Cancel"))
                             addRunFrame.hide(); // just close the Frame
                          }
                        };
                        okButton.addActionListener(buttonlistenerTypes);
                        cancelButton.addActionListener(buttonlistenerTypes);


                        KeyListenerS keyListenerAddRun = new KeyListenerS() {
                         public void KeyPressed(KeyEvent e) {
                  //System.out.println(e.getKeyCode());
                          if (e.getKeyCode()==KeyEvent.VK_ENTER) {
                            // Load the run info into the runsList
                            runsList.add(String.valueOf(beamlineChoice.getSelectedItem())+" "+String.valueOf(yearChoice.getSelectedItem())+
                                                               " "+String.valueOf(runNumberTextField.getText()));
                            runsList.select(runsList.getItemCount()-1);
                            addRunFrame.hide(); // and close the Frame
                           } else if (e.getKeyCode()==KeyEvent.VK_ESCAPE)
                             addRunFrame.hide(); // just close the Frame
                          }
                        };
                        addRunFrame.addKeyListener(keyListenerAddRun);
                        addRunPanel.addKeyListener(keyListenerAddRun);
                        beamlineChoice.addKeyListener(keyListenerAddRun);
                        yearChoice.addKeyListener(keyListenerAddRun);
                        runNumberTextField.addKeyListener(keyListenerAddRun);
                        this.addKeyListener(keyListenerAddRun);

    // End of creating components
    // ***************************


    // Items Listeners:

    // Handle events on the JButtons
    // Add/Removing Columns and their information in/from x-axis/y-axis Lists
    jbuttonListener = new ActionListenerS() {
      public void actionPerformed(ActionEvent e) {
        String command = ((JButton)e.getSource()).getText();
        // add an item to layoutChoice.
        // clear xAxisList
        if (command.equals("Add Run")) {
          // show the addRunPanel to the user
          addRunFrame.show();
        }
        if (command.equals("Add Run from File")) {
          musr.client.gui.ExtensionFileFilter dbFilter = new ExtensionFileFilter();
          dbFilter.addExtension("msr");
          
          JFileChooser fileChooser = new JFileChooser();
          fileChooser.setFileFilter(dbFilter);
          int returnVal = fileChooser.showOpenDialog(MUDSpectrumFrame.this);
          if(returnVal == JFileChooser.APPROVE_OPTION)
          {
                File selected = fileChooser.getSelectedFile();
                runsList.add(selected.getAbsolutePath());
                runsList.select(runsList.getItemCount()-1); 
          }
         
          //dImportLocal.setFileFilter(dbFilter);

          //dImportLocal.setLocation(300,300);
          
/*Deleted as this was causing two file-choosers to pop up.
          int returnVal = dImportLocal.showOpenDialog((GUI)parent); // display the dialog and block until answered
          if(returnVal == JFileChooser.APPROVE_OPTION)  { // Load the file if the Dialog is not cancelled
            //String filename = dImportLocal.getSelectedFile().getName();
            //String dirPath = dImportLocal.getCurrentDirectory().getAbsolutePath()+"/";
                            // Load the run info into the runsList
                            runsList.add(selected.getAbsolutePath()); 
                            runsList.select(runsList.getItemCount()-1);
          }
 **/

        }
        else if (command.equals("Delete Run")) {
          if (runsList.getItemCount()>0)
            runsList.remove(runsList.getSelectedItem());
        }
        else if (command.equals("Delete All Runs")) {
          while (runsList.getItemCount()>0) {
            runsList.select(0);
            runsList.remove(runsList.getSelectedItem());
          }
        }
        else if (command.equals("Run Header Info")) {
          LoadRunHeader();
        }
        else if (command.equals("Overrides")) {
          // show the overridesPanel to the user
          overridesFrame();
        }
        else if (command.equals("Complex Spectra")) {
          // This method will create a Frame and put the complex spectra fields in
          complexSpectrumFrame();
        }
        else if (command.equals("Load")) {
          StringTokenizer histToken = new StringTokenizer(histNumberTextField.getText().trim(), ",", false);
          int numberOfTokens = histToken.countTokens();

          if (spectrumChoice.getSelectedItem()=="Raw Histogram(s)") {
            loadSpectrum("raw");
          } else if (spectrumChoice.getSelectedItem()=="Early Bins") {
            loadSpectrum("early");
          } else if (spectrumChoice.getSelectedItem()=="Late Bins") {
            loadSpectrum("late");
          } else if (spectrumChoice.getSelectedItem()=="Background-Corrected Histogram(s)") {
            loadSpectrum("bkgd");
          } else if (spectrumChoice.getSelectedItem()=="2-Histogram Asymmetry (i-j)(i+j)") {
            loadSpectrum("asy2");
          } else if (spectrumChoice.getSelectedItem()=="Asymmetry from Each Histogram") {
            loadSpectrum("asy1");
          } else if (spectrumChoice.getSelectedItem()=="Asym from Each Bkg-Corr Histogram") {
            loadSpectrum("asy1bc");
          } else if (spectrumChoice.getSelectedItem()=="4-Histogram Complex Asymmetry in RRF") {
            // If the number of phaseRRF angles differs from the histogram length or the number of tokens from the histNumberTextField
            if (phaseRRF.isEmpty() ||
                ( (phaseRRF.size()!=srv.run.hist.length) && (histNumberTextField.getText().trim().equals(new String(""))) ) ||
                ( (phaseRRF.size()!=numberOfTokens) && (!histNumberTextField.getText().trim().equals(new String(""))) ) ) {
              popAlertDialog("Press 'Complex Spectra' Button first to update fields");
            } else {
              loadSpectrum("casyrrfOLD");
            }
          }
          else if (spectrumChoice.getSelectedItem()=="n-Hist Complex Asym w/Phases in RRF") {
            // If the number of phaseRRF angles differs from the histogram length or the number of tokens from the histNumberTextField
            if (phaseRRF.isEmpty() ||
                ( (phaseRRF.size()!=srv.run.hist.length) && (histNumberTextField.getText().trim().equals(new String(""))) ) ||
                ( (phaseRRF.size()!=numberOfTokens) && (!histNumberTextField.getText().trim().equals(new String(""))) ) ) {
              popAlertDialog("Press 'Complex Spectra' Button first to update fields");
            } else {
              // if the Histogram List contains ommited histograms (eg. 1,,2,3) or zeros for histograms (eg. 1,0,2,3) don't allow it. (this is allowed for 4-Hist (casyrrfOLD) only!
              boolean allowCasy = true;
              if (!histNumberTextField.getText().trim().equals(new String(""))) {
                // Use the split method because it acepts "" (zero string lenghts) as a separate token. So it will count if the user didn't put a number in between commas
                String[] fourHists = histNumberTextField.getText().trim().split(",");
                for (int j =0; j<fourHists.length;j++)
                  if (fourHists[j].trim().equals(new String("")) || fourHists[j].trim().equals(new String("0")))
                    allowCasy = false;
              }
              if (allowCasy)  loadSpectrum("casyrrf");
              else  popAlertDialog("Error: zero or blank Histogram values not allowed when loading 'n-Hist' Spectrum");

            }
          }
        }
        else if (command.equals("OK")) {
          infoDialog.hide();
        }

      }
    };
    addRunButton.addActionListener(jbuttonListener);
    addRunButton2.addActionListener(jbuttonListener);
    deleteRunButton.addActionListener(jbuttonListener);
    deleteAllRunsButton.addActionListener(jbuttonListener);
    runHeaderInfoButton.addActionListener(jbuttonListener);
    overridesButton.addActionListener(jbuttonListener);
    RRFCButton.addActionListener(jbuttonListener);
    loadSpectrumButton.addActionListener(jbuttonListener);
    OKButton.addActionListener(jbuttonListener);


    // This Listener is defined outside of the constructor (to keep it separate from the Thread)
    abortButton.addActionListener(buttonListener);

    // Handle events on the Lists
    listListener = new ItemListenerS() {
      public void itemStateChanged(ItemEvent e) {
        ItemSelectable selectedItem = e.getItemSelectable();
        // DataTable List selected
        if (selectedItem == runsList) {
          String selected = runsList.getSelectedItem();
        }
      }
    };
    runsList.addItemListener(listListener);
    spectrumChoice.addItemListener(listListener);
  }
  // All other Methods Below:

  // In order to be able to 'abort' the LoadSpectrum function, start a new SwingWorker (i.e. Thread)
  public void loadSpectrum(String type) {

          // Show a display that the muVIew is busy
          busyDialog.setLocation( (int) getLocation().getX()+50, (int) getLocation().getY()+50);
          ((Label)busyDialog.getComponent(1)).setText(" Please wait, Loading MUD: preparing to load");
          busyDialog.toFront();
          busyDialog.show();
System.out.println("MUDSpectrumFrame:554");
    worker = new SwingWorker(type, this) {

   public Object construct(String type, MUDSpectrumFrame MUDSF) {
System.out.println("MUDSpectrumFrame:556");
          try {
          try {

//--------------------
System.out.println("MUDSpectrumFrame:563");
          // Show a display that the muVIew is busy
          busyDialog.setLocation( (int) getLocation().getX()+50, (int) getLocation().getY()+50);
          ((Label)busyDialog.getComponent(1)).setText(" Please wait, Loading MUD: preparing to load");
          busyDialog.toFront();
          busyDialog.show();

          // Reset abort just in case it is true
          abort=false;

          // Update this dataTable's history (for the XML output later..)
          JTextArea tempTableHistory = new JTextArea();

          // File name
          if (runsList.getItemCount()!=0 && runsList.getSelectedIndex()==-1) runsList.select(0);
            // this string from the runsList contains: beamline, year, run#
          tempTableHistory.append("File: "+runsList.getSelectedItem()+"\n");

          // Spectrum type:
          if (spectrumChoice.getSelectedItem()=="Raw Histogram(s)") {
            tempTableHistory.append(new String(" DataTable loaded as: Raw Histogram(s) \n"));
          } else if (spectrumChoice.getSelectedItem()=="Early Bins") {
            tempTableHistory.append(new String(" DataTable loaded as: the 'Early' Spectrum Histogram(s) \n"));
          } else if (spectrumChoice.getSelectedItem()=="Late Bins") {
            tempTableHistory.append(new String(" DataTable loaded as: 'Late' Spectrum Histogram(s) \n"));
          } else if (spectrumChoice.getSelectedItem()=="Background-Corrected Histogram(s)") {
            tempTableHistory.append(new String(" DataTable loaded as: Background-Corrected Histogram(s) \n"));
          } else if (spectrumChoice.getSelectedItem()=="2-Histogram Asymmetry (i-j)(i+j)") {
            tempTableHistory.append(new String(" DataTable loaded as: 2-Histogram Asymmetry (i-j)(i+j) \n"));
          } else if (spectrumChoice.getSelectedItem()=="Asymmetry from Each Histogram") {
            tempTableHistory.append(new String(" DataTable loaded as: Asymmetry from Each Histogram \n"));
          } else if (spectrumChoice.getSelectedItem()=="Asym from Each Bkg-Corr Histogram") {
           tempTableHistory.append(new String(" DataTable loaded as: Asymmetry from Each Histogram \n"));
          } else if (spectrumChoice.getSelectedItem()=="4-Histogram Complex Asymmetry in RRF") {
            tempTableHistory.append(new String(" DataTable loaded as: 4-Histogram Complex Asymmetry in RRF \n"));
          } else if (spectrumChoice.getSelectedItem()=="n-Hist Complex Asym w/Phases in RRF") {
            tempTableHistory.append(new String(" DataTable loaded as: n-Hist Complex Asym w/Phases in RRF \n"));
          }
//--------------------

/*
          URL precessURL = new java.net.URL("http://musr.org/muview/images/precess.gif");
          Image precessImage = Toolkit.getDefaultToolkit().getImage(precessURL);
          // create a cursor for 'image' with the hotspot at 0, 0.
          Cursor curs = Toolkit.getDefaultToolkit().createCustomCursor(precessImage, new Point(0, 0), "MyCursor");
          // set this cursor on a component
          this.setCursor(curs);
*/
          MuSR_TDiHist iH = null;
          // This is the y Label
          String yLabel = "";
          String xLabel = "";
          // If the runsList has nothing selected but contains items, select the first item (this can happen when loading a GUI state)
          if (runsList.getItemCount()!=0 && runsList.getSelectedIndex()==-1) runsList.select(0);
          // this string from the runsList contains: beamline, year, run#
          final StringTokenizer tokenizer = new StringTokenizer(runsList.getSelectedItem(), " ", false);
          String area = "";
          int year = 0;
          int run = 0;
          if (tokenizer.countTokens()==1) { // this is loaded from a local file that was chosen by the user
            area = tokenizer.nextToken().toLowerCase(); // this will be the directory path with the filename
          } else { // there are 3 tokens
            area = tokenizer.nextToken().toLowerCase();
            year = Integer.parseInt (tokenizer.nextToken());
            run = Integer.parseInt (tokenizer.nextToken());
          }
          // Update this dataTable's history (for the XML output later..)
    //      ((GUI)parent).tempDataColumns.appendToTableHistory(new String(" Run: "+area+" "+year+" "+run));
          //----- Load Run Header -------------------------------
//--------------------- Construct filespec:  (site dependent!)

      String fnm=constructFilepath(area, year, run);

      MuSR_RunHeader rH = srv.getRunHdr(fnm);
//------------------------------------------
          // this is a List of Histograms to load
          List loadHistList = new List();
          // if there is a request for more than one histogram: 'blank' means: all histograms, commas in between means more than one specific histograms
          if (histNumberTextField.getText().trim().equals(new String(""))) {
            for (int i=1; i<=rH.nHists; i++)  {// find the number of existing histograms for this Run
              loadHistList.add(String.valueOf(i));
            }
          } else if (histNumberTextField.getText().trim().indexOf(",")!=-1) {
            StringTokenizer histToken = new StringTokenizer(histNumberTextField.getText().trim(), ",", false);
            int numberOfTokens = histToken.countTokens();
            for (int i=1; i<=numberOfTokens; i++)  {// traverse through the numbers and load them in
              String nextT = histToken.nextToken().trim();
              loadHistList.add(nextT);
            }
          } else { // this means there is only one histogram request, so load that one
            loadHistList.add(histNumberTextField.getText().trim());
          }
          // Create the date to have as a reference for every column created
          Date date = new Date();
          // Create the datatable to load into GUI
          DataTable spectrumDataTable = new DataTable("");

          // If the number of overrides for NT0 vector differs from the histogram length or the number of tokens from the histNumberTextField
          // Note: if the Overrides button hasn't been pressed, these vectors WILL be empty, so just load them with "" TextFields
          //          if a different RUN was selected in the meantime and it has different number of histograms, load with ""
          StringTokenizer histToken = new StringTokenizer(histNumberTextField.getText().trim(), ",", false);
          int numberOfTokens = histToken.countTokens();
          if ( (overrideNT0.size()!=srv.run.hist.length) && (histNumberTextField.getText().trim().equals(new String(""))) ) {
            bkgd1Text.clear();
            bkgd2Text.clear();
            overrideNT0.clear();
            overrideNT1.clear();
            overrideNT2.clear();
            for (int i=0; i<srv.run.hist.length; i++) {
              bkgd1Text.add(new TextField(""));
              bkgd2Text.add(new TextField(""));
              overrideNT0.add(new TextField(""));
              overrideNT1.add(new TextField(""));
              overrideNT2.add(new TextField(""));
            }
          } else if ( (overrideNT0.size()!=numberOfTokens) && (!histNumberTextField.getText().trim().equals(new String(""))) ) {
            bkgd1Text.clear();
            bkgd2Text.clear();
            overrideNT0.clear();
            overrideNT1.clear();
            overrideNT2.clear();
            for (int i=0; i<numberOfTokens; i++) {
              bkgd1Text.add(new TextField(""));
              bkgd2Text.add(new TextField(""));
              overrideNT0.add(new TextField(""));
              overrideNT1.add(new TextField(""));
              overrideNT2.add(new TextField(""));
            }
          }
          //---------------//
          // In case the default setting for goodBIn1, goodBin2 and t0_bin differ for different histograms, calculate the new values that will
          // synchronize all the histograms that load and also make sure they have equal vectro sizes
          // Also, if the the values overriden by the user, update that first
          //---------------//
          int RelGoodBin1 = Integer.MIN_VALUE;
          int RelGoodBin2 = Integer.MAX_VALUE; // put some huge number
          for (int h=1; h<=loadHistList.getItemCount(); h++) {
            MuSR_HistHdr hstHdr =srv.getHistHdr( new String[] {fnm, loadHistList.getItem(h-1) });
            // Update the NT0 value if it exists and it is a digit (ie. a number)
            if ( !((TextField)overrideNT0.get(h-1)).getText().equals("") && (Character.isDigit(((TextField)overrideNT0.get(h-1)).getText().charAt(0))))
              hstHdr.t0_bin = Integer.parseInt (((TextField)overrideNT0.get(h-1)).getText());
            // Update the NT1 value if it exists and it is a digit (ie. a number)
            if (!firstBinTextField.getText().trim().equals("")) {
              if (firstLastChoice.getSelectedItem().equals("bin #")) {
                // Evaluate it just in case it is an expression
                hstHdr.goodBin1= (int) ((GUI)parent).evaluateExpr( firstBinTextField.getText().trim(), -1,-1,"");
              } else if (firstLastChoice.getSelectedItem().equals("Time [s]")) { // Transfer to ns by multiplying with 1000
                // it is in TIme [s] so use the formula: {[(t1_ns-0.5*nsPerBin)+t0_ns]/nsPerBIn}+1 to get the bin #   *1000
                // Evaluate it just in case it is an expression
                hstHdr.goodBin1 = (int)((((
                                                      ((GUI)parent).evaluateExpr( firstBinTextField.getText().trim(), -1,-1,"")
                                                           *1000-0.5*hstHdr.nsPerBin) + hstHdr.t0_ns)/hstHdr.nsPerBin+1));
              } else { // it is in TIme [ns] so use the formula: {[(t1_ns-0.5*nsPerBin)+t0_ns]/nsPerBIn}+1 to get the bin #
                // Evaluate it just in case it is an expression
                hstHdr.goodBin1 = (int)(((
                                                      (int)((GUI)parent).evaluateExpr( firstBinTextField.getText().trim(), -1,-1,"")
                                                           -0.5*hstHdr.nsPerBin) + hstHdr.t0_ns)/hstHdr.nsPerBin)+1;
              }
            }
            if ( !((TextField)overrideNT1.get(h-1)).getText().equals("") && (Character.isDigit(((TextField)overrideNT1.get(h-1)).getText().charAt(0))))
              hstHdr.goodBin1 = Integer.parseInt (((TextField)overrideNT1.get(h-1)).getText());
            // If the user put a value also into the From/To field, update N1!
            // Update the NT2 value if it exists and it is a digit (ie. a number)
            if (!lastBinTextField.getText().trim().equals(""))  {
              if (firstLastChoice.getSelectedItem().equals("bin #")) {
                // Evaluate it just in case it is an expression
                hstHdr.goodBin2= (int) ((GUI)parent).evaluateExpr( lastBinTextField.getText().trim(), -1,-1,"");
               } else if (firstLastChoice.getSelectedItem().equals("Time [s]")) { // Transfer to ns by multiplying with 1000
                // it is in Time [s] so use the formula: ( {[(t1_ns-0.5*nsPerBin)+t)_ns]/nsPerBIn}+1 ) to get the bin #  * 1000
                hstHdr.goodBin2 = (int)((((
                                                     ((GUI)parent).evaluateExpr( lastBinTextField.getText().trim(), -1,-1,"")
                                                              *1000-0.5*hstHdr.nsPerBin) + hstHdr.t0_ns)/hstHdr.nsPerBin+1));
              } else { // it is in TIme [ns] so use the formula: {[(t1_ns-0.5*nsPerBin)+t)_ns]/nsPerBIn}+1 to get the bin #
                hstHdr.goodBin2 = (int)(((
                                                   (int)((GUI)parent).evaluateExpr( lastBinTextField.getText().trim(), -1,-1,"")
                                                               -0.5*hstHdr.nsPerBin) + hstHdr.t0_ns)/hstHdr.nsPerBin)+1;
              }
            }
            if ( !((TextField)overrideNT2.get(h-1)).getText().equals("") && (Character.isDigit(((TextField)overrideNT2.get(h-1)).getText().charAt(0))))
              hstHdr.goodBin2 = Integer.parseInt (((TextField)overrideNT2.get(h-1)).getText());
            // If the user put a value also into the From/To field, update N2!
            // Find the relative differences to use in the calculation later on
            // Max value
            if (RelGoodBin1<hstHdr.goodBin1-hstHdr.t0_bin)
              RelGoodBin1=hstHdr.goodBin1-hstHdr.t0_bin;
            // Min Value
            if (RelGoodBin2>hstHdr.goodBin2-hstHdr.t0_bin)
              RelGoodBin2=hstHdr.goodBin2-hstHdr.t0_bin;
          }

              // clear the temporary Names Vector
              ((GUI)parent).tempNames.removeAllElements();
              // clear the temporary Labels Vector
              ((GUI)parent).tempLabels.removeAllElements();

          // Now, load these Histograms in:
          for (int h=1; h<=loadHistList.getItemCount(); h++) {
            ((Label)busyDialog.getComponent(1)).setText(" Loading histogram "+h);
//          ((Label)busyDialog.getComponent(2)).setText(" Time Left: "+String.valueOf(100-h));
              if (Thread.interrupted()) throw new InterruptedException();
              // This will load the Spectrums
              LoadHistogram loadHistogram = new LoadHistogram();
              // Create a List of all the User defined variables
              List userL = new List();
              userL.add(area);
              userL.add(String.valueOf(year));
              userL.add(String.valueOf(run));
              userL.add(loadHistList.getItem(h-1));
              // --------------------------------------
              // Get the HistHdr associated with this Histogram
              MuSR_HistHdr hstHdr =srv.getHistHdr( new String[] {fnm, loadHistList.getItem(h-1) });

              // Update the NT0 value if it exists and it is a digit (ie. a number)
              if ( !((TextField)overrideNT0.get(h-1)).getText().equals("") && (Character.isDigit(((TextField)overrideNT0.get(h-1)).getText().charAt(0))))
                hstHdr.t0_bin = Integer.parseInt (((TextField)overrideNT0.get(h-1)).getText());
              // Now, update NT1 by using RelGoodBin1 that was calculated earlier
              hstHdr.goodBin1 = hstHdr.t0_bin + RelGoodBin1;
              // Now, update NT2 by using RelGoodBin2 that was calculated earlier
              hstHdr.goodBin2 = hstHdr.t0_bin + RelGoodBin2;
              // Override the background values (if the user specified it)
              if (!((TextField)bkgd1Text.get(h-1)).getText().trim().equals(""))
                hstHdr.bkgd1 = Integer.parseInt(((TextField)bkgd1Text.get(h-1)).getText().trim());
              if (!((TextField)bkgd2Text.get(h-1)).getText().trim().equals(""))
                hstHdr.bkgd2 = Integer.parseInt(((TextField)bkgd2Text.get(h-1)).getText().trim());


              // ---------------------------------------------------------
              // This is where the "Early Bins" Spectrums are 'determined'
              if (type.equals("early")) {
                userL.add( "0" );
                userL.add( String.valueOf( hstHdr.goodBin1 + (int)(hstHdr.goodBin1*0.15) ));
                if (displayChoice.getSelectedItem().equals("(default)") || displayChoice.getSelectedItem().equals("bin #") )
                  yLabel = "counts per raw Bin";
                else if (displayChoice.getSelectedItem().equals("Time [ns]") || displayChoice.getSelectedItem().equals("Time [s]"))
                  yLabel = "counts per ns";
              // This is where the "Late Bins" Spectrums are 'determined'
              } else if (type.equals("late")) {
                userL.add( String.valueOf( hstHdr.goodBin2 - 100 ) ); // Subtract 100 bins as a random value
                userL.add( String.valueOf( hstHdr.nBins ) );
                if (displayChoice.getSelectedItem().equals("(default)") || displayChoice.getSelectedItem().equals("bin #") )
                  yLabel = "counts per raw Bin";
                else if (displayChoice.getSelectedItem().equals("Time [ns]") || displayChoice.getSelectedItem().equals("Time [s]"))
                  yLabel = "counts per ns";
              // Load Raw Spectrum from bin: NT1 to bin: NT2
              } else if (type.equals("raw") || type.equals("bkgd") || type.equals("asy2")) {
                userL.add(String.valueOf( hstHdr.goodBin1));
                userL.add(String.valueOf( hstHdr.goodBin2));
                if (displayChoice.getSelectedItem().equals("(default)") || displayChoice.getSelectedItem().equals("bin #") )
                  yLabel = "counts per raw Bin";
                else if (displayChoice.getSelectedItem().equals("Time [ns]") || displayChoice.getSelectedItem().equals("Time [s]"))
                  yLabel = "counts per ns";
              } else if (type.equals("asy1") || type.equals("asy1bc") || type.equals("casyrrfOLD") || type.equals("casyrrf")) {
                userL.add(String.valueOf( hstHdr.goodBin1));
                userL.add(String.valueOf( hstHdr.goodBin2));
                yLabel = "Asymmetry";
              }

              // Packing:
              if (!packingTextField.getText().trim().equals(new String("")))  {
                if (packingChoice.getSelectedItem().equals("# bins")) {
                  // Evaluate it just in case it is an expression
                  int tempPack = (int) ((GUI)parent).evaluateExpr( packingTextField.getText().trim(), -1,-1,"");
                  if (tempPack==0 && // If the user is loading Copmlex RRF Spectra loadin with zero packing, load "1" instead!
                      ( (!RRFFreq.getText().trim().equals(new String("")) &&  type.equals("casyrrfOLD")) ||
                      (!RRFFreq.getText().trim().equals(new String("")) &&  type.equals("casyrrf")) ) )
                    userL.add("1");
                  else
                    userL.add( String.valueOf(tempPack) );
                } else { // this mean it is in Time [ns] so divide it with nsPerBin to get the integer packing #
                  // Evaluate it just in case it is an expression
                  int tempPack = Math.round( (float) ( ((GUI)parent).evaluateExpr( packingTextField.getText().trim(), -1,-1,"") /hstHdr.nsPerBin ));
                  if (tempPack==0 && // If the user is loading Copmlex RRF Spectra loadin with zero packing, load "1" instead!
                      ( (!RRFFreq.getText().trim().equals(new String("")) &&  type.equals("casyrrfOLD")) ||
                      (!RRFFreq.getText().trim().equals(new String("")) &&  type.equals("casyrrf")) ) )
                    userL.add("1");
                  else
                    userL.add( String.valueOf(tempPack) );
                  }
//System.out.println(" packing/nsPerBin: "+String.valueOf( Math.round((Double.parseDouble(packingTextField.getText().trim())/hstHdr.nsPerBin )) ) );
              } else {
                // This is for Copmlex RRF Spectra loading:
                if ( (!RRFFreq.getText().trim().equals(new String("")) &&  type.equals("casyrrfOLD")) ||
                     (!RRFFreq.getText().trim().equals(new String("")) &&  type.equals("casyrrf")) ) {
                  //int rrfF =  Math.round( Float.parseFloat(RRFFreq.getText().trim())/20 ); // This is an approximation to packing
                  //if (hstHdr.nsPerBin>rrfF)
                    // This will end up not packing the Histogram by any bigger value (like 256.. see MuSR_Server) so the Columns will be pretty big
                    userL.add("1");
                 // else userL.add(String.valueOf(rrfF));
                // This is if all the fields are empty
                // note that this will use a default packing like 256... (see MuSR_Server)
                } else {
                  userL.add("0");
                }
              }

// NOTE: IMPORTANT!!!!, MuSR_Server will also look up "Time [s]", "Time [ns]", bin #
              //----------------------------------------- //
              // Load the Run
              System.out.println("MUDSpectrumFrame:861");
              iH = loadHistogram.main(MUDSF, userL, srv, date, parent);
              System.out.println("iH title:" + iH.title);
              System.out.println("iH label: " + iH.label);      //these are acting the same for TI and TD data - not sure what is going on
              // ----------------------------------------- //

              // Determine the seconds
              // Make sure that the units are converted to 'ns' if time range is <1s (<1000ns). To determine this, use the first x value
              // Set the xLabel
              if (displayChoice.getSelectedItem().equals("(default)")) {
                if (spectrumChoice.getSelectedItem() == "Raw Histogram(s)" ||
                    spectrumChoice.getSelectedItem()=="Early Bins" ||
                    spectrumChoice.getSelectedItem()=="Late Bins" ) {
                  xLabel = "bin";
                  if (h==1) // Create this only once, the first time
                  iH.xxx.setLabel("Time [bin]");
                } else if ( iH.t2_ns<1000) {
                  xLabel = "ns";
                  if (h==1) // Create this only once, the first time
                  iH.xxx.setLabel("Time [ns]");
                } else {
                  xLabel = "s";
                  if (h==1) // Create this only once, the first time
                  iH.xxx.setLabel("Time [s]");
                }
              }
              if (displayChoice.getSelectedItem().equals("Time [s]")) {
                xLabel = "s";
                if (h==1) // Create this only once, the first time
                iH.xxx.setLabel("Time [s]");
              } else if (displayChoice.getSelectedItem().equals("Time [ns]")) {
                xLabel = "ns";
                if (h==1) // Create this only once, the first time
                iH.xxx.setLabel("Time [ns]");
              }
              if (displayChoice.getSelectedItem().equals("bin #")) { // Early and Late types are always graphed showing the bins
                xLabel = "bin";
                if (h==1) // Create this only once, the first time
                iH.xxx.setLabel("Time [bin]");
              }

              // Create new (GUI Compatible) Columns (to convert the iH DataColumns into them)
              LinearColumn xAxisColumn = null;
              ConcreteDataColumn yAxisColumn = null;
              ConcreteDataColumn yAxisPlusErrorColumn = null;
              ConcreteDataColumn yAxisMinusErrorColumn = null;
              // NOTE!! I exchanged the Labels with the units below!!!! This is because PlotApplet uses Labels to displa in the plot and here, we want the units to be displayed
              // Create DataColumns
              if (h==1) { // Create this only once, the first time
                xAxisColumn = iH.xxx;
              }
              // Note: the number "h" is added to the Column Name. This is done to avoid having same names loaded for the columns in the same DataTable
              String yName = "";
              if (type.equals("bkgd") || type.equals("asy2")) yName= iH.nnn.getName()+"BC"+loadHistList.getItem(h-1); // this will be: NBC1, NBC2.. (or different order!)
              else if ( type.equals("asy1") || (type.equals("casyrrf") && useSpectrum.getSelectedItem().equals(new String("ASY1")))
                                                      || (type.equals("casyrrfOLD") && useSpectrum.getSelectedItem().equals(new String("ASY1"))) )
                yName= "ASY"+loadHistList.getItem(h-1); // this will be: ASY1, ASY2.. (or different order!)
              else if ( type.equals("asy1bc") || (type.equals("casyrrf") && useSpectrum.getSelectedItem().equals(new String("ASY1BC")))
                                                          || (type.equals("casyrrfOLD") && useSpectrum.getSelectedItem().equals(new String("ASY1BC"))) )
                yName= "ASYBC"+loadHistList.getItem(h-1); // this will be: ASYBC1, ASYNBC2.. (or different order!)
//              else if (type.equals("casyrrf")) yName= "CASYRRF"+loadHistList.getItem(h-1); // this will be: CASYRRF1, CASYRRF2.. (or different order!)
              else yName = iH.nnn.getName()+loadHistList.getItem(h-1); // this will be: N1, N2.. (or different order!)

              yAxisColumn = new ConcreteDataColumn(yName , yLabel /*iH.nnn.getUnits()*/, yLabel,  date);
              yAxisPlusErrorColumn = new ConcreteDataColumn(yName, yLabel /*iH.nnn.getUnits()*/, yLabel, date);
              yAxisMinusErrorColumn = new ConcreteDataColumn(yName, yLabel /*iH.nnn.getUnits()*/, yLabel, date);

              // ---------------------------- //
              // LOADING THE DATA into the newly Created Columns
              // first, before loading data, Calculate the time-independent background B (in case we need it for the bkgd loading)
              double B = 0; // total numer of counts
              int npacTemp = 0; // average counts per bin
              double M = (hstHdr.bkgd2-hstHdr.bkgd1+1); // # background bins
              double b = 0; // Average number of background bins per count
              if (type.equals("bkgd") || type.equals("asy1bc") || type.equals("asy2")  || type.equals("casyrrfOLD") || type.equals("casyrrf")) {
                for (int w=hstHdr.bkgd1; w<=hstHdr.bkgd2; w++)  B = B + srv.getHist( new String[] {fnm, loadHistList.getItem(h-1) }).bin[w];
                b = B / M;  // divide with the number of bins used to add up into B
                // Calculate the packing (npac) if it hasn't been specified by the user
                npacTemp = iH.packing;
              }
              // ** //
              double tmu = 2.197E+3; // this is in ns (otherwise, in seconds 2.197 multiplies with E-6)
              if (displayChoice.getSelectedItem().equals("Time [s]") || (iH.t2_ns>=1000 && displayChoice.getSelectedItem().equals("(default)"))) {
                tmu = 2.197; // this is in s (otherwise, in seconds 2.197 multiplies with E-6)
              }
              int nb = iH.nnn.getSize(); //  LEN( N )
              // In case, there have already been some truncated columns loaded, find them and adjust nb to that value
              // this will make sure that if there have been some truncated columns (histograms) loaded, that this histogram would load in with that same size
              // note: there is really no need to loop through all of them because they WILL all have the same size regardless
              for (int hist = 0; hist < spectrumDataTable.columnCount() ; hist++) {
                // in case, the new histogram that was loaded is faulty and has fewer elements (this has happened with one of the files while testing!)
                // note: I shouldn't really be responsible for a faulty histogram.. but I will still do this so that it can at least load in
          //      if (spectrumDataTable.getColumn(hist).getSize()<=iH.nnn.getSize())
                  // adjust nb to the shorter length if truncating had occured in previous histograms
                  if (spectrumDataTable.getColumn(hist).getSize()<nb ) nb=spectrumDataTable.getColumn(hist).getSize();
              }
              // Note: if it happens that there is a zero value in the nnn vector, then for the Complex Spectrum (with asy1), there will be a division with zero
              // to avoid this, truncate the vector when a zero appears. To do this, I will just change the value 'nb'.
              if ( ( type.equals("casyrrf") && useSpectrum.getSelectedItem().equals(new String("ASY1")) ) ||
                   ( type.equals("casyrrfOLD") && useSpectrum.getSelectedItem().equals(new String("ASY1")) ) ) {
                for (int i=0; i<iH.nnn.getSize(); i++) {
                  if (Thread.interrupted()) throw new InterruptedException();
                  if ( ((Integer)iH.nnn.getValue(i)).doubleValue()==0 && i<nb) {
                    // Note: i here is the index value, but when assigned to nb, it is assigned as a vector size. So, if assigned as 'i' (and not as 'i-1') it will not
                    // include the array value that contains 0
                    nb=i;
                    ((GUI)parent).popAlertDialog("    Warning: ","  The histogram #"+h+" contains a zero at the row #"+i+"." ," The column will be truncated to avoid division with zero when calculating the Symmetric Spectrum (with ASY1). ");
                    break;
                  }
                }
                // Now make sure all the previous columns get truncated to the same length.
                // Of course, this will happen ONLY IF the nb value changed
                for (int hist = 0; hist < spectrumDataTable.columnCount(); hist++)
                  if (spectrumDataTable.getColumn(hist).getSize()>nb ) {
                    spectrumDataTable.setColumn( spectrumDataTable.getColumn(hist).getSubColumn( new Range(0,nb-1)), hist);
                  }
              }
              // Prepare value for Asy1BC
              double N0BC = 0;
              if ( type.equals("asy1bc") || (type.equals("casyrrf") && useSpectrum.getSelectedItem().equals(new String("ASY1BC")))
                                                   || (type.equals("casyrrfOLD") && useSpectrum.getSelectedItem().equals(new String("ASY1BC"))) ) {
              // Calculate the sum
                for (int s=0; s<nb; s++) {
                  if (Thread.interrupted()) throw new InterruptedException();
                  double xvalue = ((Double)iH.xxx.getValue(s)).doubleValue();  // no need to change units, that was done in Musr_Server
                  if (displayChoice.getSelectedItem().equals("bin #")) {
                    // This is inverse of: (int)((((Double.parseDouble(lastBinTextField.getText().trim())-0.5*hstHdr.nsPerBin) + hstHdr.t0_ns)/hstHdr.nsPerBin+1))
                    xvalue = (((xvalue-1)*hstHdr.nsPerBin)-hstHdr.t0_bin)+0.5*hstHdr.nsPerBin; // Convert bin to ns
                  }
                  /*                  if (displayChoice.getSelectedItem().equals("(default)") && iH.t2_ns>=1000) {
                    xvalue = xvalue*1000; // Convert back from s to ns
                  } else if (displayChoice.getSelectedItem().equals("Time [s]")) {
                    xvalue = xvalue*1000; // Convert back from s to ns
    System.out.println("xvalue: "+xvalue);
                  } else if (displayChoice.getSelectedItem().equals("bin #")) {
                  }*/
                  double expTemp = Math.exp(xvalue/tmu);
                  double yvalue = ((Integer)iH.nnn.getValue(s)).doubleValue();
                  N0BC = N0BC + ( yvalue - ((double)npacTemp*b)) * expTemp;
                }
                N0BC = N0BC/nb;
              }
              // Prepare values for Asy1
              double sumraw = 0;  //sum(N[j],j,1:nb)
              double sume = 0.0;  //sum(exp(-t[j]/tmu),j,1:nb)
              double suminv = 0;  //sum(1/N[j],j,1:nb)
              double sumei = 0;  //sum(exp(-t[j]/tmu)/N[j],j,1:nb)
              double sumeei = 0;  //sum(exp(-t[j]/tmu)^2/N[j],j,1:nb)
              double delta = 0;  //suminv*sumeei - sumei^2
              // Calculate the sums

              if ( type.equals("asy1") || (type.equals("casyrrf") && useSpectrum.getSelectedItem().equals(new String("ASY1")))
                                               || (type.equals("casyrrfOLD") && useSpectrum.getSelectedItem().equals(new String("ASY1"))) )  {
                  for (int s=0; s<nb; s++) {
                    if (Thread.interrupted()) throw new InterruptedException();
                    double xvalue = ((Double)iH.xxx.getValue(s)).doubleValue();
                    if (displayChoice.getSelectedItem().equals("bin #")) {
                      // This is inverse of: (int)((((Double.parseDouble(lastBinTextField.getText().trim())-0.5*hstHdr.nsPerBin) + hstHdr.t0_ns)/hstHdr.nsPerBin+1))
                      xvalue = (((xvalue-1)*hstHdr.nsPerBin)-hstHdr.t0_bin)+0.5*hstHdr.nsPerBin; // Convert bin to ns
                    }
                    double expTemp = Math.exp(-xvalue/tmu);
                    double yvalue = ((Integer)iH.nnn.getValue(s)).doubleValue();
                    // Asy1 variables:
//System.out.println("expTemp: "+expTemp);
//System.out.println("tmu: "+tmu);
//System.out.println("yvalue: "+yvalue);
                    sumraw = sumraw + yvalue;
//            System.out.println("sumraw["+s+"]: "+sumraw);
                    sume = sume + expTemp;
//            System.out.println("sume["+s+"]: "+sume);
                    //if (yvalue!=0) {
                      suminv= suminv + 1/yvalue;
//            System.out.println("suminv["+s+"]: "+suminv);
                      sumei = sumei + expTemp / yvalue;
//            System.out.println("sumei["+s+"]: "+sumei);
                      sumeei = sumeei + Math.pow(expTemp,2) / yvalue;
                    //}
//            System.out.println("sumeei["+s+"]: "+sumeei);
                  }
                  delta = suminv*sumeei - Math.pow(sumei,2);
//            System.out.println("delta"+delta);
              }

              // ** //
              // Now load the data
              for (int i=0; i<nb;i++) {
                if (Thread.interrupted()) throw new InterruptedException();
                // Load X Column
/*         No need for this code since xAxisColumn is Linear (i.e. Virtual)
                if (h==1) { // Load this only once, the first time
                            xAxisColumn.addElement(iH.xxx.getValue(i));
                            xAxisPlusErrorColumn.addElement(iH.xxx.getErrorPlus(i));
                            xAxisMinusErrorColumn.addElement(iH.xxx.getErrorMinus(i));
                }
*/
                // Prepare y values
                Object yValue = null;
                Object yErrorPlus = null;
                Object yErrorMinus = null;
                if (type.equals("raw") || type.equals("early") || type.equals("late")) {
                  // Divide the value with the packing number to get: "Counts per bin"
                  yValue =  new Double( Tools.objToDoub(iH.nnn.getValue(i)) / iH.packing);
                  yErrorPlus = new Double( ((Double)iH.nnn.getErrorPlus(i)).doubleValue() / iH.packing ) ;
                  yErrorMinus = new Double( ((Double)iH.nnn.getErrorMinus(i)).doubleValue() / iH.packing );
                // This is where all the calculations for asy1 and also crrf happens
                } else if (type.equals("asy1") || (type.equals("casyrrf") && useSpectrum.getSelectedItem().equals(new String("ASY1")))
                                                         || (type.equals("casyrrfOLD") && useSpectrum.getSelectedItem().equals(new String("ASY1")))  ) {
                  double xvalue = ((Double)iH.xxx.getValue(i)).doubleValue();    // no need to change units, that was done in Musr_Server
                  if (displayChoice.getSelectedItem().equals("bin #")) {
                    // This is inverse of: (int)((((Double.parseDouble(lastBinTextField.getText().trim())-0.5*hstHdr.nsPerBin) + hstHdr.t0_ns)/hstHdr.nsPerBin+1))
                    xvalue = (((xvalue-1)*hstHdr.nsPerBin)-hstHdr.t0_bin)+0.5*hstHdr.nsPerBin; // Convert bin to ns
                  }
                  double expTemp = Math.exp(-xvalue/tmu);
                  double yvalueTemp = ((Integer)iH.nnn.getValue(i)).doubleValue();
                  double anorm = (sume*suminv - nb*sumei)/delta;
//            System.out.println("anorm"+anorm);
                  double bkgnd = (nb*sumeei - sumei*sume)/(anorm*delta);
//            System.out.println("bkgnd"+bkgnd);
                  yValue = new Double(  (yvalueTemp/anorm - bkgnd)/expTemp - 1.0 );
//            System.out.println("yValue"+yValue);
                  yErrorPlus = new Double( ((Double)iH.nnn.getErrorPlus(i)).doubleValue()/(anorm*expTemp) );  //err/(anorm*EXPT)
                  yErrorMinus = new Double( ((Double)iH.nnn.getErrorMinus(i)).doubleValue()/(anorm*expTemp) );  //err/(anorm*EXPT)
                  //yvalueTemp=0; // release memory
                // This is where all the calculations for asy1bc and also crrf happens
                } else if (type.equals("asy1bc") || (type.equals("casyrrf") && useSpectrum.getSelectedItem().equals(new String("ASY1BC")))
                                                             || (type.equals("casyrrfOLD") && useSpectrum.getSelectedItem().equals(new String("ASY1BC")))   ) {

                  double xvalue = ((Double)iH.xxx.getValue(i)).doubleValue();
                  if (displayChoice.getSelectedItem().equals("bin #")) {
                    // This is inverse of: (int)((((Double.parseDouble(lastBinTextField.getText().trim())-0.5*hstHdr.nsPerBin) + hstHdr.t0_ns)/hstHdr.nsPerBin+1))
                    xvalue = (((xvalue-1)*hstHdr.nsPerBin)-hstHdr.t0_bin)+0.5*hstHdr.nsPerBin; // Convert bin to ns
                  }

                  double expTemp = Math.exp(xvalue/tmu);
                  double yvalueTemp = ((Integer)iH.nnn.getValue(i)).doubleValue();
//if (h==9 && i>50000) {
//  System.out.println("  i:"+i);
//  System.out.println("yValueTemp:"+yvalueTemp);
//  System.out.println("  b:"+b);
//  System.out.println("  npacTemp:"+npacTemp);
//  System.out.println("  npacTemp*b:"+npacTemp*b);
//  }
                  yValue = new Double(  ((yvalueTemp- ((double)npacTemp*b))/N0BC)*expTemp - 1.0 );
                  yErrorPlus = new Double( ( ((Double)iH.nnn.getErrorPlus(i)).doubleValue()/N0BC)*expTemp );  //(err/N0BC)*EXPT
                  yErrorMinus = new Double( ( ((Double)iH.nnn.getErrorMinus(i)).doubleValue()/N0BC)*expTemp );  //(err/N0BC)*EXPT
                  //yvalueTemp=0; // release memory

                } else if (type.equals("bkgd") || type.equals("asy2") /*||
                            type.equals("asy1bc") || type.equals("casyrrf")*/) { // note: asy2, asy1bc, casyrrf load bkgd spectrums to calculate the asymetry to
                  yValue = new Double( ((Integer)iH.nnn.getValue(i)).doubleValue() - ((double)npacTemp*b) );
                  // Error is: sqrt(db^2 + Ni + 1) where db = dB*npac/M (dB=sqrt(B))
                  yErrorPlus = new Double( (double) Math.sqrt( Math.pow((Math.sqrt(B)*npacTemp/M), 2) + ((Integer)iH.nnn.getValue(i)).doubleValue() + 1.0 ) );  //((Double)iH.nnn.getErrorPlus(i)).doubleValue()
                  yErrorMinus = new Double( (double) Math.sqrt( Math.pow((Math.sqrt(B)*npacTemp/M), 2) + ((Integer)iH.nnn.getValue(i)).doubleValue() + 1.0 ) );
                }
                // Load y Column
                yAxisColumn.addElement( yValue );
                yAxisPlusErrorColumn.addElement( yErrorPlus );
                yAxisMinusErrorColumn.addElement( yErrorMinus );
              }

              // add the Column(s) into the DataTable
              if (h==1) { // Load this only once, the first time
                spectrumDataTable.addColumn(xAxisColumn);
                // add the new name into the array,
                ((GUI)parent).tempNames.addElement(xAxisColumn.getName());
                // add a new Label into the array
                ((GUI)parent).tempLabels.addElement(xAxisColumn.getLabel());

                spectrumDataTable.setTitle(iH.title);

              }
              spectrumDataTable.addColumn(new MultiColumn(yAxisColumn, yAxisPlusErrorColumn,yAxisMinusErrorColumn));
              // add the new name into the array
              ((GUI)parent).tempNames.addElement(yAxisColumn.getName());
              // add a new Label into the array
              ((GUI)parent).tempLabels.addElement(yAxisColumn.getLabel());
//System.out.println("h: "+h);
//System.out.println("yAxisPlusErrorColumn: "+yAxisPlusErrorColumn);
//System.out.println("yAxisMinusErrorColumn: "+yAxisMinusErrorColumn);
            // ---------------------------- //

          }  // end of 'for' loop to load all the histograms

          // ---
          // Add the history JTextArea into the DataTable first
          spectrumDataTable.tableHistory = tempTableHistory;
          // Put the DataTable into the dictionary and the dataTable List
          ((GUI)parent).addDataTable(spectrumDataTable, false, "");
          // assign the tempDataColumns DataColumn in the GUI to the new DataTable
          ((GUI)parent).tempDataColumns = (DataTable) ((GUI)parent).dictionary.getElement(((GUI)parent).dictionary.size()-1);
//System.out.println("Console's tempColumns after addDatatTable in GUI: "+((ConcreteDataColumn)((GUI)parent).tempColumns.get(0) ).getSize());

          // Now that all the columns have been loaded, if there is a need to calculate Asymetric Spectrum, do it and then erase the initial y (bcakground) columns
          if (type.equals("asy2"))  {

            ((Label)busyDialog.getComponent(1)).setText(" Calculating Asymetric Spectrum ");
//            ((Label)busyDialog.getComponent(2)).setText(" Time Left: "+60);

            DataTable table = ((GUI)parent).tempDataColumns;
            // every second column, note, if there are 3 items, it will enter only once! (in case input-ed 3 histograms)
            // Note that loadHistList.getItemCount() doesn't include the Time column
            for (int a=0; a<(loadHistList.getItemCount()-1); a++) {
              if (Thread.interrupted()) throw new InterruptedException();
              // Create the expression from the column names
              String N1 = ((DataColumn)table.getColumn(a+1)).getName(); // note: 1 is added to a because the very first item in the list is the x column.
              String N2 = ((DataColumn)table.getColumn(a+2)).getName();
              System.out.println("MUDSpectrumFrame:1162 - N1 = " + N1);
              System.out.println("MUDSpectrumFrame:1163 - N2 = " + N2);
              String genExpr = "("+N1+"-"+N2+")/("+N1+"+"+N2+")";
              String genName = "Asy"+N1.substring(N1.length()-1)+N2.substring(N2.length()-1);  // this will look like: ASY21, ASY12 ..
              System.out.println("MUDSpectrumFrame:1167");
              ((GUI)parent).console.generateColumn(genExpr, genName, date);
              a++; // increase (i.e. double) the value of 'a', so that it skips one loop
            }
            // finally, remove the columns that Asy was generated from
            for (int a=0; a<(loadHistList.getItemCount()); a++) {
              if (Thread.interrupted()) throw new InterruptedException();
              table.removeColumnAt(1); // Remove the second column (index=1) everytime. (not the first one, that is the x column)
                // add the new name into the array
                ((GUI)parent).tempNames.removeElementAt(1);
                // add an new dummy label which should be changed in the db file right after
                ((GUI)parent).tempLabels.removeElementAt(1);
            }
          }

          // Now that all the columns have been loaded, if there is a need to calculate RRF Complex Spectrum, do it and then erase the initial y (asy1 or asy1bc) columns
          if (type.equals("casyrrf") || type.equals("casyrrfOLD")) {

                  ((Label)busyDialog.getComponent(1)).setText(" Calculating Complex Asymmetry, Please Wait" );
//                  ((Label)busyDialog.getComponent(2)).setText(" Time Left: "+50);

            int dataTableKey = ((GUI)parent).getDataTableIndex();
            DataTable table = (DataTable)((GUI)parent).dictionary.getElement(dataTableKey); // retreive info from here
            ConcreteDataColumn realColumn = new ConcreteDataColumn("RASY","CAsymmetry","CAsymmetry", date); // this is the new real column
            ConcreteDataColumn realColumnMinusErr = new ConcreteDataColumn();
            ConcreteDataColumn realColumnPlusErr = new ConcreteDataColumn();
            ConcreteDataColumn imagColumn = new ConcreteDataColumn("IASY","CAsymmetry","CAsymmetry", date); // this is the new imaginary column
            ConcreteDataColumn imagColumnMinusErr = new ConcreteDataColumn();
            ConcreteDataColumn imagColumnPlusErr = new ConcreteDataColumn();
            int columnCount = ((DataColumn)table.getColumn(0)).getSize(); // The size is retreived from the first column in the DataTable
            // Traverse through all the columns and for each, calculate the CASYRRF
            for (int i=0; i<columnCount; i++) {
              if (Thread.interrupted()) throw new InterruptedException();
              // For each coulmn, use all the Columns (ASY1 or ASY1BC) in the DataTable to calculate the value
              double realValue = 0;
              double realErrMinValue = 0;
              double realErrPlusValue = 0;
              double imagValue = 0;
              double imagErrMinValue = 0;
              //double imagErrPlusValue = 0;
              // these are sum values:
              double sumW = 0;
              double sumSin = 0;
              double sumCos = 0;
              // This is the algorithm to derive realValue and imagValue the old way, withouth rotating frames
              if (type.equals("casyrrfOLD")) {
                // Note: this Spectrum will accept additional formats for the Histograms numbers input
                // Ex: if Histograms: 1,3,2,4 then Real=(A1-A3)/2, Imag=(A2-A4)/2
                //         Higtograms: 1 then Real=A1, Imag=0
                //         Histograms: 1,,3,4 then Real=A1, Imag=(A3-A4)/2

                String[] fourHists = null;
                if (histNumberTextField.getText().equals(new String(""))) { // this means that the user didn't specify any histogram so load them all
                  fourHists = new String[loadHistList.getItemCount()];
                  for (int a=0; a<(loadHistList.getItemCount()); a++)
                    fourHists[a] = loadHistList.getItem(a);
                } else { // the user specified histograms to load
                  // Use the split method because it acepts "" (zero string lenghts) as a separate token. So it will count if the user didn't put a number in between commas
                  fourHists = histNumberTextField.getText().trim().split(",");
                }
                Vector fourHistVector = new Vector();
                for (int j =0; j<fourHists.length;j++)
                  fourHistVector.add(fourHists[j]);

                // The Spectrum will always be calculated by using four histograms.
                // If the user didn't input all the Histograms, add zeros into the Vector until there are four elements in the Vector
                // Example:1,2 - the Vector will then contain: {1,2,0,0}
                if (fourHistVector.size()<4)
                  while (fourHistVector.size() < 4) // until there are 4 elements in the vector
                    fourHistVector.add(new String(""));
                // Note: the order of the loaded histograms is already adjusted, we don't have to worry about the oreder really, just where the zeros are
                int count = 1; // this will be the index to load from. Note: it starts from 1 because we want to skip the first time column
                double A1 = 0;
                double dA1m = 0;
                double dA1p = 0;
                if (!((String)fourHistVector.elementAt(0)).equals("")) {
                  A1 = Tools.objToDoub( ((DataColumn)table.getColumn(count)).getValue(i) ); // count=1
                  dA1m = Tools.objToDoub( ((DataColumn)table.getColumn(count)).getErrorMinus(i) );
                  //dA1p = Tools.objToDoub( ((DataColumn)table.getColumn(count)).getErrorPlus(i) );
                  count++;
                }
                double A2 = 0;
                double dA2m = 0;
                double dA2p = 0;
                if (!((String)fourHistVector.elementAt(1)).equals("")) {
                  A2 = Tools.objToDoub( ((DataColumn)table.getColumn(count)).getValue(i) ); // count=2
                  dA2m = Tools.objToDoub( ((DataColumn)table.getColumn(count)).getErrorMinus(i) );
                  dA2p = Tools.objToDoub( ((DataColumn)table.getColumn(count)).getErrorPlus(i) );
                  count++;
                }
                double A3 = 0;
                double dA3m = 0;
                double dA3p = 0;
                if (!((String)fourHistVector.elementAt(2)).equals("")) {
                  A3 = Tools.objToDoub( ((DataColumn)table.getColumn(count)).getValue(i) ); // count=3
                  dA3m = Tools.objToDoub( ((DataColumn)table.getColumn(count)).getErrorMinus(i) );
                  dA3p = Tools.objToDoub( ((DataColumn)table.getColumn(count)).getErrorPlus(i) );
                  count++;
                }
                double A4 = 0;
                double dA4m = 0;
                double dA4p = 0;
                if (!((String)fourHistVector.elementAt(3)).equals("")) {
                  A4 = Tools.objToDoub( ((DataColumn)table.getColumn(count)).getValue(i) ); // count=4
                  dA4m = Tools.objToDoub( ((DataColumn)table.getColumn(count)).getErrorMinus(i) );
                  dA4p = Tools.objToDoub( ((DataColumn)table.getColumn(count)).getErrorPlus(i) );
                  count++;
                }
                // Now calcualte realValue and imagValue
                if (A1!=0 && A2!=0)
                  realValue = (A1-A2)/2;
                else // one or both are zero. Don't divide by 2
                  realValue = A1-A2;
                if (A3!=0 && A4!=0)
                  imagValue = (A3-A4)/2;
                else // one or both are zero. Don't divide by 2
                  imagValue = A3-A4;

                // calculate casyrrfOLD errors
                realErrMinValue = Math.sqrt( ( dA1m*dA1m + dA2m*dA2m )/2 );
                //realErrPlusValue = Math.sqrt( ( dA1p*dA1p + dA2p*dA2p )/2 );
                imagErrMinValue = Math.sqrt( ( dA3m*dA3m + dA4m*dA4m )/2 );
                //imagErrPlusValue = Math.sqrt( ( dA3p*dA3p + dA4p*dA4p )/2 );

              // This is the algorithm to derive realValue and imagValue when there are rotating frames (i.e. using angles)
              } else if (type.equals("casyrrf")) {
                for (int a=0; a<(loadHistList.getItemCount()); a++) {
                  if (Thread.interrupted()) throw new InterruptedException();
                  // Create two new Columns (real and imaginery) by summing the values of the columns
                  // Note: the number of angles in Vector phaseRRF should match the number of coulumns
                  double theta = ((GUI)parent).evaluateExpr(  ((TextField)phaseRRF.get(a)).getText().trim(), -1,-1,"");
                  double columnValue  = Tools.objToDoub( ((DataColumn)table.getColumn(a+1)).getValue(i) );
                  double columnMinError = Tools.objToDoub( ((DataColumn)table.getColumn(a+1)).getErrorMinus(i) );
                  double columnPlusError = Tools.objToDoub( ((DataColumn)table.getColumn(a+1)).getErrorPlus(i) );
//System.out.println("");
//System.out.println("columnMinError: "+columnMinError);
//System.out.println("columnPlusError: "+columnPlusError);
                  double W = 0;
                  if (columnMinError==0 || columnPlusError==0 ) // Note: this algorithm could be better. It doesn't take into account if only one error is zero
                    W = 1;
                  else
                    W = (1/Math.pow(columnMinError,2)+1/Math.pow(columnPlusError,2))/2;
//System.out.println("W: "+W);
                  sumW = sumW +W;
                  sumCos = sumCos + Math.pow( Math.cos( Math.toRadians(theta)), 2 ) * W;
                  sumSin = sumSin + Math.pow( Math.sin( Math.toRadians(theta)), 2 ) * W;
                  //note: it starts from 1 because the very first item in the list is the x column.
                  realValue = realValue +  columnValue * W * Math.cos( Math.toRadians(theta) );
                  imagValue = imagValue + columnValue * W * Math.sin( Math.toRadians(theta) );
                }
                if  (loadHistList.getItemCount()==1 && Double.parseDouble( (String) ((TextField)phaseRRF.get(0)).getText().trim() )==0) { // avoid division with zero
                } else {
                  if (sumCos!=0)
                    realValue = realValue/sumCos;
                    realErrMinValue = 1/Math.sqrt(sumCos);
                    //realErrPlusValue = 1/Math.sqrt(sumCos);
                  if (sumSin!=0)
                    imagValue = imagValue/sumSin;
                    imagErrMinValue = 1/Math.sqrt(sumSin);
                    //imagErrPlusValue = 1/Math.sqrt(sumSin);
                }
              }
//System.out.println("realValue: "+realValue);
//System.out.println("imagValue: "+imagValue);
              // If there is no RRF then ignore the multiplication with it AND RePacking as well!
              if (!RRFFreq.getText().trim().equals( new String("") )) {
                // Change the name of the Columns:
                realColumn.name = "RRFR";
                imagColumn.name = "RRFI";
                // Get the RRF frequency. Evaluate it just in case it is an expression
                // In case it is a negative value, take its absolute value
                double rrfF = Math.abs( ((GUI)parent).evaluateExpr( RRFFreq.getText().trim(), -1,-1,"") );
                // Multiply the values with the RRF frequency in MHz (convert the frequency appropriately)
                if (xLabel.equals(new String("s"))) {} // do nothing since1MHz == 1000000(1/s) == 1 (1/s)
                else rrfF = rrfF*0.001; // in case it is is "ns" (1MHz == 0.0001 (1/ns))  Note: that for "bin" the calculation is the same, this is probably incorrect but I assume user won't go there
                double th = -rrfF * 2* Math.PI * Tools.objToDoub( ((DataColumn)table.getColumn(0)).getValue(i) );
                double realValueTemp = realValue;
                double imagValueTemp = imagValue;
//System.out.println("Math.cos(th): "+Math.cos(th));
                realValue = realValueTemp * Math.cos(th) + imagValueTemp * Math.sin(th);  // *t
                imagValue = -realValueTemp * Math.sin(th) + imagValueTemp * Math.cos(th);  // *t
                // calculate the error
                if ((sumCos!=0 && sumSin!=0 && Math.cos(th)!=0 && Math.sin(th)!=0) || !type.equals("casyrrfOLD") ) {  // Note: 1/(realErrMinValue)^2 = sumCos
                  realErrMinValue = 1/(Math.sqrt( sumCos * Math.pow(Math.cos(th),2) + sumSin * Math.pow(Math.sin(th),2)) );
                  //realErrPlusValue = 1/(Math.sqrt( sumCos * Math.pow(Math.cos(th),2) + sumSin * Math.pow(Math.sin(th),2)) );
                  imagErrMinValue = 1/( Math.sqrt( sumCos * Math.pow(Math.sin(th),2) + sumSin * Math.pow(Math.cos(th),2)) );
                  //imagErrPlusValue = 1/( Math.sqrt( sumCos * Math.pow(Math.sin(th),2) + sumSin * Math.pow(Math.cos(th),2)) );
                } else if (type.equals("casyrrfOLD") && Math.cos(th)!=0 && Math.sin(th)!=0 && realErrMinValue!=0 && realErrMinValue!=0) {
                  realErrMinValue = 1/(Math.sqrt( (1/Math.pow(realErrMinValue,2)) * Math.pow(Math.cos(th),2) + (1/Math.pow(imagErrMinValue,2)) * Math.pow(Math.sin(th),2)) );
                  //realErrPlusValue = 1/(Math.sqrt( (1/Math.pow(realErrPlusValue,2)) * Math.pow(Math.cos(th),2) + (1/Math.pow(imagErrPlusValue,2)) * Math.pow(Math.sin(th),2)) );
                  imagErrMinValue = 1/( Math.sqrt( (1/Math.pow(realErrMinValue,2)) * Math.pow(Math.sin(th),2) + (1/Math.pow(imagErrMinValue,2)) * Math.pow(Math.cos(th),2)) );
                  //imagErrPlusValue = 1/( Math.sqrt( (1/Math.pow(realErrPlusValue,2)) * Math.pow(Math.sin(th),2) + (1/Math.pow(imagErrPlusValue,2)) * Math.pow(Math.cos(th),2)) );
                }
              }
              realColumn.addElement(Tools.doubToObj(realValue) );
              realColumnMinusErr.addElement(Tools.doubToObj(realErrMinValue));
              realColumnPlusErr.addElement(Tools.doubToObj(realErrMinValue));
              imagColumn.addElement(Tools.doubToObj(imagValue) );
              imagColumnMinusErr.addElement(Tools.doubToObj(imagErrMinValue));
              imagColumnPlusErr.addElement(Tools.doubToObj(imagErrMinValue));
            }
            // If required, repack
            if (!RRFFreq.getText().trim().equals( new String("") ) ) {

                ((Label)busyDialog.getComponent(1)).setText("  Repacking ");
//                ((Label)busyDialog.getComponent(2)).setText(" Time Left: 5");

              // If user didn't enter anything for RRFPackBIn, use packing factor 1
              double getRRFBin = 1;
              if (!RRFPackBin.getText().trim().equals(new String(""))) {
              // Evaluate it just in case it is an expression
                getRRFBin= ((GUI)parent).evaluateExpr( RRFPackBin.getText().trim(), -1,-1,"");
              }
              int packBin = 0;
              MuSR_HistHdr hstHdr =srv.getHistHdr( new String[] {fnm, loadHistList.getItem(0) }); // use the first histogram to get nsPerBin
              if (RRFMenu.getSelectedItem().trim().equals(new String("RRF Periods"))) {
                // RRFPackBin/RRFFreq is in s so multiply with 1000
                // Note: In case RRF Frequency is a negative value, take its absolute value
                packBin = (int)(  ( getRRFBin/
                                             Math.abs( ((GUI)parent).evaluateExpr(RRFFreq.getText().trim(), -1,-1,"") ) )
                                             *1000/hstHdr.nsPerBin / iH.packing );
              } else if (RRFMenu.getSelectedItem().trim().equals(new String("ns"))) {
                packBin = (int)Math.round( (float)getRRFBin/hstHdr.nsPerBin  / iH.packing );
              } else if (RRFMenu.getSelectedItem().trim().equals(new String("s"))) {
                packBin = (int)Math.round( (float)getRRFBin*1000/hstHdr.nsPerBin  / iH.packing );
              } else if (RRFMenu.getSelectedItem().trim().equals(new String("# Bins"))) {
                packBin = (int)getRRFBin;
              }

              // Convert the packBin to ns
              // **  Repack **
//System.out.println("Time: ######### ");
              LinearColumn timeRepackedColumn = (LinearColumn) ((GUI)parent).repack(packBin, table.getColumn(0) , date );
              table.addColumn(timeRepackedColumn);
                // add the new name into the array
                ((GUI)parent).tempNames.addElement(timeRepackedColumn.getName());
                // add a new Label into the array
                ((GUI)parent).tempLabels.addElement(timeRepackedColumn.getLabel());
//System.out.println("Real: ######### ");
              MultiColumn realRepackedColumn =
                (MultiColumn) ((GUI)parent).repack(
                                                                  packBin,
                                                                  new MultiColumn(((ConcreteDataColumn)realColumn),
                                                                                             (ConcreteDataColumn)realColumnPlusErr,
                                                                                             (ConcreteDataColumn)realColumnMinusErr),
                                                                  date);
              table.addColumn(realRepackedColumn);
                // add the new name into the array
                ((GUI)parent).tempNames.addElement(realColumn.getName());
                // add a new Label into the array
                ((GUI)parent).tempLabels.addElement(realColumn.getLabel());
//System.out.println("Imag: ######### ");
              MultiColumn imagRepackedColumn =
                (MultiColumn) ((GUI)parent).repack(
                                                                  packBin,
                                                                  new MultiColumn(((ConcreteDataColumn)imagColumn),
                                                                                             (ConcreteDataColumn)imagColumnPlusErr,
                                                                                             (ConcreteDataColumn)imagColumnMinusErr),
                                                                  date);

                ((Label)busyDialog.getComponent(1)).setText(" Loading Data Tables");
//                ((Label)busyDialog.getComponent(2)).setText(" Time Left: 4");

              table.addColumn(imagRepackedColumn);
                // add the new name into the array
                ((GUI)parent).tempNames.addElement(imagColumn.getName());
                // add a new Label into the array
                ((GUI)parent).tempLabels.addElement(imagColumn.getLabel());
              // finally, remove the columns that Asy was generated from (including the first (Time) column
              for (int a=0; a<=(loadHistList.getItemCount()); a++) {
                if (Thread.interrupted()) throw new InterruptedException();
                table.removeColumnAt(0); // Remove the first column (index=0) everytime.
                // add the new name into the array
                ((GUI)parent).tempNames.removeElementAt(0);
                // add a new Label into the array
                ((GUI)parent).tempLabels.removeElementAt(0);
              }
            } else {
                ((Label)busyDialog.getComponent(1)).setText(" Loading Data Tables");
//                ((Label)busyDialog.getComponent(2)).setText(" Time Left: 4");
              // There was no RRFFreq and repacking
              table.addColumn(new MultiColumn(((ConcreteDataColumn)realColumn),
                                                        (ConcreteDataColumn)realColumnPlusErr,
                                                        (ConcreteDataColumn)realColumnMinusErr));
                // add the new name into the array
                ((GUI)parent).tempNames.addElement(realColumn.getName());
                // add a new Label into the array
                ((GUI)parent).tempLabels.addElement(realColumn.getLabel());
              table.addColumn(new MultiColumn(((ConcreteDataColumn)imagColumn),
                                                        (ConcreteDataColumn)imagColumnPlusErr,
                                                        (ConcreteDataColumn)imagColumnMinusErr));
                // add the new name into the array
                ((GUI)parent).tempNames.addElement(imagColumn.getName());
                // add a new Label into the array
                ((GUI)parent).tempLabels.addElement(imagColumn.getLabel());
              // finally, remove the columns that Asy was generated from
              for (int a=0; a<(loadHistList.getItemCount()); a++) {
                table.removeColumnAt(1); // Remove the second column (index=1) everytime. (not the first one, that is the x column)
                // remove name from vector
                ((GUI)parent).tempNames.removeElementAt(1);
                // remove label from vector
                ((GUI)parent).tempLabels.removeElementAt(1);
              }
            }
            // assign the tempDataColumns DataColumn in the GUI to this DataTable
            ((GUI)parent).tempDataColumns = table;
          }

          // show dataTable's column contents in the ColumnList
          ((GUI)parent).reloadColumnList();

     /*     if (type.equals("asy1bc"))  {
            int dataTableKey = ((GUI)parent).getDataTableIndex();
            DataTable table = (DataTable)((GUI)parent).dictionary.getElement(dataTableKey);
            for (int a=0; a<(loadHistList.getItemCount()); a++) { // for every column generate on asy1bc
              // Create the expression from the column names
              String N1 = ((DataColumn)table.getColumn(a+1)).getName(); // note: 1 is added to a because the very first item in the list is the x column.
              String genExpr = "("+N1+"/"+20+")*exp("+")";
              String genName = "Asy"+N1.substring(N1.length()-1);  // this will look like: ASY2, ASY1 ..
              ((GUI)parent).console.generateColumn(genExpr, genName);
            }
            // finally, remove the columns that Asy was generated from
            for (int a=0; a<(loadHistList.getItemCount()); a++) {
              table.removeColumnAt(1); // Remove the second column (index=1) everytime. (not the first one, that is the x column)
            }
            // show dataTable's column contents in the ColumnList
            ((GUI)parent).reloadColumnList();
          }*/
          // -----

          // Add the X column into X Axis List and the Rest into the Y Axis Column
          if ( ((GUI)parent).columnList.getItemCount()!=0 && ((GUI)parent).layoutChoice.getSelectedIndex()!=-1 ) {
            // Add the first column in the Column List into the X Axis
            int selectedLayoutIndex = ((GUI)parent).layoutChoice.getSelectedIndex();
            ((GUI)parent).addX(0, selectedLayoutIndex);
            // Add the rest of the columns into the Y Axis
            for (int yh=1; yh<=((GUI)parent).columnList.getItemCount()-1; yh++) { // note: yh starts from 1, which is the second index in columnList, therefore, the first y axis column
              if (Thread.interrupted()) throw new InterruptedException();
              int selectedxAxisIndex = ((GUI)parent).xAxisList.getItemCount()-1;// This is 'buggy', I exchanged it for the code on the left:    ((GUI)parent).xAxisList.getSelectedIndex();
              ((GUI)parent).addY(yh, selectedxAxisIndex,selectedLayoutIndex);
            }
          }

          // Hide the display that the muVIew is busy
          busyDialog.hide();

          } catch (InterruptedException ei) {
             //popAlertDialog("Loading aborted");
            return "Interrupted";  // SwingWorker.get() returns this

          } catch (OutOfMemoryError e) {
                busyDialog.hide();
                popAlertDialog("Error: Out of memory.");
                System.out.println("Memory leak occured. e: "+e);
                return "Out of Memory";  // SwingWorker.get() returns this
          }

          } catch (Exception e2) {
                busyDialog.hide();
                System.err.println("Exception in MUDSpectrumFrame loading TDiHist: "+ e2.getMessage());
                // also pop-up an alert about this
                displayError();
                return new String("Exception occured: "+e2);  // SwingWorker.get() returns this
          }
          System.out.println("MUDSpectrumFrame:1531");
          return "All Done";
     }

    };
    System.out.println("MUDSpectrumFrame:1537");
    worker.start();
    //worker.construct(type);
  }

  public void LoadRunHeader() {

            try {
                System.out.println(1540);
            // this string from the runsList contains: beamline, year, run#
            final StringTokenizer tokenizer = new StringTokenizer(runsList.getSelectedItem(), " ", false);
          String area = "";
          int year = 0;
          int run = 0;
          if (tokenizer.countTokens()==1) { // this is loaded from a local file that was chosen by the user
            area = tokenizer.nextToken().toLowerCase(); // this will be the directory path with the filename
          } else { // there are 3 tokens
            area = tokenizer.nextToken().toLowerCase();
            year = Integer.parseInt (tokenizer.nextToken());
            run = Integer.parseInt (tokenizer.nextToken());
          }
          System.out.println(1553);

//----- Load Run Header -------------------------------
//--------------------- Construct filespec:  (site dependent!)

      String fnm=constructFilepath(area, year, run);
      System.out.println(1559);
      MuSR_RunHeader rH = srv.getRunHdr(fnm);
//------------------------------------------
            System.out.println(1562);
              final JFrame runHeaderFrame = new JFrame("Run Header: "+runsList.getSelectedItem());
              TextArea runHeaderTextArea = new TextArea();
              // fill the text area
              runHeaderFrame.getContentPane().add(runHeaderTextArea, "Center");
              System.out.println(1567);
/*
              // Load Header info
              runHeaderTextArea.append(" Run Header Info: ");
              runHeaderTextArea.append("\n "+Format.get(System.out, "fmtID=0x%08x", rH.fmtID));
              runHeaderTextArea.append( "\t runNumber "+rH.runNumber );
              runHeaderTextArea.append("\n title \""+rH.title+"\"");
              runHeaderTextArea.append("\t nHists: "+rH.nHists );
              runHeaderTextArea.append("\n subtitle:  "+rH.subtitle );
              runHeaderTextArea.append("\n comment1: "+rH.comment1);
              runHeaderTextArea.append("\n comment2: "+rH.comment2 );
              runHeaderTextArea.append("\n comment3: "+rH.comment3);
              runHeaderTextArea.append( "\n lab \""+rH.lab+"\"");
              runHeaderTextArea.append("\t exptNumber "+rH.exptNumber);
              runHeaderTextArea.append( "\t experimenter \""+rH.experimenter+"\"" );
              runHeaderTextArea.append("\n sample \""+rH.sample+"\"");
              runHeaderTextArea.append(", orient \""+rH.orient+"\"");
              if ( rH.temperature != null) runHeaderTextArea.append ( ", temperature \""+rH.temperature+"\"" );
              if ( rH.field != null) runHeaderTextArea.append ( ", field \""+rH.field+"\"" );
              runHeaderTextArea.append("\n (timeEnd:"+rH.timeEnd+") - ");
              runHeaderTextArea.append("(timeBegin:" +rH.timeBegin+") = ");
              runHeaderTextArea.append("(elapsedSec:"+rH.elapsedSec+")");
              runHeaderTextArea.append( "\n method \""+rH.method+"\"" );
              runHeaderTextArea.append( "\t das \""+rH.das+"\"" );
              runHeaderTextArea.append( "\n area \""+rH.area+"\"" );
              runHeaderTextArea.append( "\t apparatus \""+rH.apparatus+"\"" );
              runHeaderTextArea.append( "\t insert \""+rH.insert+"\"" );
              runHeaderTextArea.append( "\t nSclrs: "+rH.nSclrs+"\n");
              runHeaderTextArea.append( "\t nIndVar: "+rH.nIndVar+"\n");
*/
              // Load Header info
              Date StartTime = new Date( (long) 1000*rH.timeBegin );
              int Year = Calendar.YEAR;
              if (Year < 71) { Year = 2000 + Year; }
                else { Year = 1900 + Year; } // Our very own Y2K bug!
              Date EndTime = new Date( (long) 1000*rH.timeEnd );
         //     runHeaderTextArea.append(" Run Header Info: ");
              runHeaderTextArea.append ( "\n "+rH.lab );
              runHeaderTextArea.append( " Experiment # "+rH.exptNumber );
              runHeaderTextArea.append ( "\t["+rH.experimenter+"]" );
              runHeaderTextArea.append ( "\ton "+rH.area );
              runHeaderTextArea.append ( "\tin "+Year+":" );
              runHeaderTextArea.append("\n "+Format.get(System.out, "fmtID=0x%08x", rH.fmtID));
              runHeaderTextArea.append ( "\n\n"+rH.das );
              runHeaderTextArea.append ( "\t"+rH.method );
              runHeaderTextArea.append ( "\t Run # "+rH.runNumber );
              runHeaderTextArea.append ( "\t ["+rH.nHists+" Histograms" );
              if (rH.nSclrs > 0 ) runHeaderTextArea.append ( ", "+rH.nSclrs+" Scalers");
              if ( rH.nIndVar > 0 ) runHeaderTextArea.append ( ", "+rH.nIndVar+" Indep. Var.");
              runHeaderTextArea.append ( "]" );
              runHeaderTextArea.append ( "\n\n \""+rH.title+"\"\n" );
              if ( rH.subtitle != null) runHeaderTextArea.append ( "\n "+rH.subtitle );
              if ( rH.comment1 != null) runHeaderTextArea.append ( "\n "+rH.comment1 );
              if ( rH.comment2 != null) runHeaderTextArea.append ( "\n "+rH.comment2 );
              if ( rH.comment3 != null) runHeaderTextArea.append ( "\n "+rH.comment3 );
              runHeaderTextArea.append ( "\n" );
              if ( rH.sample != null ) runHeaderTextArea.append ( "Sample: \""+rH.sample+"\"  " );
              if ( rH.orient != null ) runHeaderTextArea.append ( "Orientation: \""+rH.orient+"\"  " );
              if ( rH.temperature != null) runHeaderTextArea.append ( "Temperature = "+rH.temperature+"  " );
              if ( rH.field != null) runHeaderTextArea.append ( "Field = "+rH.field );
              runHeaderTextArea.append ( "\nStarted "+StartTime );
              runHeaderTextArea.append ( ".\tEnded "+EndTime );
              runHeaderTextArea.append ( "\nElapsed: "+rH.elapsedSec+" s.  " );
              runHeaderTextArea.append ( "\tApparatus = \""+rH.apparatus+"\"" );
              runHeaderTextArea.append ( "\tInsert = \""+rH.insert+"\"" );
              runHeaderTextArea.append ( "\n" );
              System.out.println(1633);

              // Load Histpgram Header info
              for (int i=1; i<=rH.nHists; i++)  { // find the number of existing histograms for this Run
                // retreive the Header info for the i-th histogram
                MuSR_HistHdr hstHdr =srv.getHistHdr( new String[] {fnm, String.valueOf(i) });
                runHeaderTextArea.append( "\n Histogram #"+i); // also can use: hstHdr.histNumber
                runHeaderTextArea.append(" \"" +hstHdr.title +"\"");
                runHeaderTextArea.append("\t" +hstHdr.nEvents +" evts");
                runHeaderTextArea.append(" in " +hstHdr.nBins +" bins\n");
                if ( hstHdr.nsPerBin != 0.0 ) runHeaderTextArea.append(hstHdr.nsPerBin +" ns/bin; ");
                if ( hstHdr.t0_bin != 0 ) runHeaderTextArea.append("t=0 in bin " +hstHdr.t0_bin+" ");
                if ( hstHdr.t0_ns != 0.0 )
                  runHeaderTextArea.append("[" +hstHdr.t0_ns +" ns past left edge of bin 1]\n");
                if ( hstHdr.bkgd1 != 0 ) runHeaderTextArea.append("t<0 background in bins " +hstHdr.bkgd1);
                if ( hstHdr.bkgd2 != 0 ) runHeaderTextArea.append(" to " +hstHdr.bkgd2+"; ");
                runHeaderTextArea.append("good data in bins " +hstHdr.goodBin1);
                runHeaderTextArea.append(" to " +hstHdr.goodBin2 +")\n");
              }
              System.out.println(1652);

              runHeaderFrame.pack();
              runHeaderFrame.setSize((int)((GUI)parent).screenDimen.getWidth()/2, (int)((GUI)parent).screenDimen.getHeight()/2);
              // Show the frame with the TextArea and the Button in it
              runHeaderFrame.show();

              Button runHeaderButton = new Button("Close");
              runHeaderFrame.getContentPane().add(runHeaderButton, "South");
               ActionListenerS buttonlistenerHeader = new ActionListenerS() {
                         public void actionPerformed(ActionEvent e) {
                            if (e.getActionCommand().equals("Close")) {
                              runHeaderFrame.hide();
                              runHeaderFrame.dispose();
                            }
                          }
                };
                System.out.println(1669);
              runHeaderButton.addActionListener(buttonlistenerHeader);

            } catch (Exception e2) {
                System.err.println("Exception in MUDSpectrumFrame loading TDiHist (LoadRunHeader Method): "+ e2.getMessage());
                // also pop-up an alert about this
                displayError();
            }
            System.out.println(1677);
  }

  /**
   * This displays the overrides pop-up window with all the overrides in it.
   */
        public void overridesFrame() {

          try {
          // this is a List of Histograms to load
          List loadHistList = new List();
          String fnm = "";
          // If the Run List is empty, disable skip everyting. The user needs to create a run before being able to set the parameters
          // OR if the user entered the number of hists to load
          if ((runsList.getItemCount()>0) || (!histNumberTextField.getText().trim().equals(new String("")))) {
            // Find out how many histograms the run contains
            if (runsList.getItemCount()>0) {
              // this string from the runsList contains: beamline, year, run#
              final StringTokenizer tokenizer = new StringTokenizer(runsList.getSelectedItem(), " ", false);
          String area = "";
          int year = 0;
          int run = 0;
          if (tokenizer.countTokens()==1) { // this is loaded from a local file that was chosen by the user
            area = tokenizer.nextToken().toLowerCase(); // this will be the directory path with the filename
          } else { // there are 3 tokens
            area = tokenizer.nextToken().toLowerCase();
            year = Integer.parseInt (tokenizer.nextToken());
            run = Integer.parseInt (tokenizer.nextToken());
          }

//----- Load Run Header -------------------------------
//--------------------- Construct filespec:  (site dependent!)

      fnm=constructFilepath(area, year, run);

      MuSR_RunHeader rH = srv.getRunHdr(fnm);
//------------------------------------------

               // if there is a request for more than one histogram: 'blank' means: all histograms, commas in between means more than one specific histograms
               if (histNumberTextField.getText().trim().equals(new String(""))) {
                 for (int i=1; i<=rH.nHists; i++)  {// find the number of existing histograms for this Run
                   loadHistList.add(String.valueOf(i));
                 }
               } else if (histNumberTextField.getText().trim().indexOf(",")!=-1) {
                 StringTokenizer histToken = new StringTokenizer(histNumberTextField.getText().trim(), ",", false);
                 int numberOfTokens = histToken.countTokens();
                 for (int i=1; i<=numberOfTokens; i++)  {// traverse through the numbers and load them in
                   loadHistList.add(histToken.nextToken().trim());
                 }
               } else { // this means there is only one histogram request, so load that one
                  loadHistList.add(histNumberTextField.getText().trim());
               }
          }

          int histNumber = loadHistList.getItemCount();

          // If the number of Histograms requested is new, resize and overvrite the overrideNT0 Vector
          // Note: this is done in case the overrides button has not been pressed yet or a new RUN with different # of histoframs was selected
          if (overrideNT0.size()!=histNumber) {
            // Erase any previous values that were stored by the user
            bkgd1Text.clear();
            bkgd2Text.clear();
            overrideNT0.clear();
            overrideNT1.clear();
            overrideNT2.clear();
            for (int i=0; i<histNumber; i++) {
              bkgd1Text.add(new TextField(""));
              bkgd2Text.add(new TextField(""));
              overrideNT0.add(new TextField(""));
              overrideNT1.add(new TextField(""));
              overrideNT2.add(new TextField(""));
            }
          }
                 // Clean up the earlier contents of the frame
                 overridesFrameScrollbar.removeAll();
                 overridesFrame.setBackground(Color.lightGray);

                        // Contains settings to add or remove plot captions
                        JPanel aPanel = new JPanel();
                        aPanel.setLayout(gridbag);

                        c.gridx = 0; c.gridy = 0; aPanel.add(new Label(" Hist #"), c);
                        c.gridx = 1; c.gridy = 0; aPanel.add(new Label(" NT0    "), c);
	  c.gridx = 2; c.gridy = 0; aPanel.add(new Label(" NT1    "), c);
	  c.gridx = 3; c.gridy = 0; aPanel.add(new Label(" NT2    "), c);
	  c.gridx = 4; c.gridy = 0; aPanel.add(new Label(" bkgd1    "), c);
	  c.gridx = 5; c.gridy = 0; aPanel.add(new Label(" bkgd2    "), c);

                        // Load the histograms and the fields for them
                        for (int i=1; i<=histNumber; i++) {
                          MuSR_HistHdr hstHdr =srv.getHistHdr( new String[] {fnm, String.valueOf(i) });
                          // Insert Histogram #
                            c.gridx = 0; c.gridy = i;
                          aPanel.add(new Label( loadHistList.getItem(i-1) ), c);
                          // Insert NT0
                              c.gridx = 1; c.gridy = i;
                          if (overrideNT0.isEmpty()) // Note: it should never be empty, but just in case
                            aPanel.add(new Label(""), c);
                          else
                            aPanel.add((TextField)overrideNT0.get(i-1), c);
                          // Insert NT1
                              c.gridx = 2; c.gridy = i;
                          if (overrideNT1.isEmpty()) // Note: it should never be empty, but just in case
                            aPanel.add(new Label(""), c);
                          else
                            aPanel.add((TextField)overrideNT1.get(i-1), c);
                          // Insert NT2
                              c.gridx = 3; c.gridy = i;
                          if (overrideNT2.isEmpty()) // Note: it should never be empty, but just in case
                            aPanel.add(new Label(""), c);
                          else
                            aPanel.add((TextField)overrideNT2.get(i-1), c);
                          // Insert bkgd1Text
                              c.gridx = 4; c.gridy = i;
                          if (bkgd1Text.isEmpty()) // Note: it should never be empty, but just in case
                            aPanel.add(new Label(""), c);
                          else
                            aPanel.add((TextField)bkgd1Text.get(i-1), c);
                          // Insert bkgd2Text
                              c.gridx = 5; c.gridy = i;
                          if (bkgd2Text.isEmpty()) // Note: it should never be empty, but just in case
                            aPanel.add(new Label(""), c);
                          else
                            aPanel.add((TextField)bkgd2Text.get(i-1), c);
                        }

                            c.gridx = 0; c.gridy = histNumber+4;
                        Button aButton = new Button("Close");
                        aPanel.add(aButton, c);
                            c.gridx = 1; c.gridy = histNumber+4;
                        Button nt0Button = new Button("NT0-fill");
                        aPanel.add(nt0Button, c);
                            c.gridx = 2; c.gridy = histNumber+4;
                        Button nt1Button = new Button("NT1-fill");
                        aPanel.add(nt1Button, c);
                            c.gridx = 3; c.gridy = histNumber+4;
                        Button nt2Button = new Button("NT2-fill");
                        aPanel.add(nt2Button, c);
                            c.gridx = 4; c.gridy = histNumber+4;
                        Button bkg1Button = new Button("BG1-fill");
                        aPanel.add(bkg1Button, c);
                            c.gridx = 5; c.gridy = histNumber+4;
                        Button bkg2Button = new Button("BG2-fill");
                        aPanel.add(bkg2Button, c);


                            c.gridx = 1; c.gridy = histNumber+5;
                        Button nt0Buttonclear = new Button("NT0-clear");
                        aPanel.add(nt0Buttonclear, c);
                            c.gridx = 2; c.gridy = histNumber+5;
                        Button nt1Buttonclear = new Button("NT1-clear");
                        aPanel.add(nt1Buttonclear, c);
                            c.gridx = 3; c.gridy = histNumber+5;
                        Button nt2Buttonclear = new Button("NT2-clear");
                        aPanel.add(nt2Buttonclear, c);
                            c.gridx = 4; c.gridy = histNumber+5;
                        Button bkg1Buttonclear = new Button("BG1-clear");
                        aPanel.add(bkg1Buttonclear, c);
                            c.gridx = 5; c.gridy = histNumber+5;
                        Button bkg2Buttonclear = new Button("BG2-clear");
                        aPanel.add(bkg2Buttonclear, c);

                        overridesFrameScrollbar.add(aPanel);
                        overridesFrame.getContentPane().add(overridesFrameScrollbar);

                        overridesFrame.setSize(600,500);
                        overridesFrame.isEnabled();

                        // In order to use these values inside inner class, we need to have them as 'final'
                        final String fnmF = fnm;
                        final int histNumberF = histNumber;

                        ActionListenerS buttonlistenerTitle = new ActionListenerS() {
                         public void actionPerformed(ActionEvent e) {
        try {
                          if (e.getActionCommand().equals("Close"))
                            overridesFrame.dispose();
                          // Refill NT0 column with default values
                          if (e.getActionCommand().equals("NT0-fill")) {
                            for (int i=1; i<=histNumberF; i++) {
                              MuSR_HistHdr hstHdr =srv.getHistHdr( new String[] {fnmF, String.valueOf(i) });
                              ((TextField)overrideNT0.get(i-1)).setText( Integer.toString(hstHdr.t0_bin) );
                            }
                          }
                          // Refill NT1 column with default values
                          if (e.getActionCommand().equals("NT1-fill")) {
                            for (int i=1; i<=histNumberF; i++) {
                              MuSR_HistHdr hstHdr =srv.getHistHdr( new String[] {fnmF, String.valueOf(i) });
                              ((TextField)overrideNT1.get(i-1)).setText( Integer.toString(hstHdr.goodBin1) );
                            }
                          }
                          // Refill NT2 column with default values
                          if (e.getActionCommand().equals("NT2-fill")) {
                            for (int i=1; i<=histNumberF; i++) {
                              MuSR_HistHdr hstHdr =srv.getHistHdr( new String[] {fnmF, String.valueOf(i) });
                              ((TextField)overrideNT2.get(i-1)).setText( Integer.toString(hstHdr.goodBin2) );
                            }
                          }
                          // Refill Bkgd1 column with default values
                          if (e.getActionCommand().equals("BG1-fill")) {
                            for (int i=1; i<=histNumberF; i++) {
                              MuSR_HistHdr hstHdr =srv.getHistHdr( new String[] {fnmF, String.valueOf(i) });
                              ((TextField)bkgd1Text.get(i-1)).setText( Integer.toString(hstHdr.bkgd1) );
                            }
                          }
                          // Refill Bkgd2 column with default values
                          if (e.getActionCommand().equals("BG2-fill")) {
                            for (int i=1; i<=histNumberF; i++) {
                              MuSR_HistHdr hstHdr =srv.getHistHdr( new String[] {fnmF, String.valueOf(i) });
                              ((TextField)bkgd2Text.get(i-1)).setText( Integer.toString(hstHdr.bkgd2) );
                            }
                          }
                          // clear NT0
                          if (e.getActionCommand().equals("NT0-clear")) {
                            for (int i=1; i<=histNumberF; i++) {
                              ((TextField)overrideNT0.get(i-1)).setText("");
                            }
                          }
                          // clear NT1
                          if (e.getActionCommand().equals("NT1-clear")) {
                            for (int i=1; i<=histNumberF; i++) {
                              ((TextField)overrideNT1.get(i-1)).setText("");
                            }
                          }
                          // clear NT2
                          if (e.getActionCommand().equals("NT2-clear")) {
                            for (int i=1; i<=histNumberF; i++) {
                              ((TextField)overrideNT2.get(i-1)).setText("");
                            }
                          }
                          // clear bkgd1
                          if (e.getActionCommand().equals("BG1-clear")) {
                            for (int i=1; i<=histNumberF; i++) {
                              ((TextField)bkgd1Text.get(i-1)).setText("");
                            }
                          }
                          // clear bkgd2
                          if (e.getActionCommand().equals("BG2-clear")) {
                            for (int i=1; i<=histNumberF; i++) {
                              ((TextField)bkgd2Text.get(i-1)).setText("");
                            }
                          }
          } catch (Exception e2) {
              System.err.println("Exception in MUDSpectrumFrame loading TDiHist (OverridesFrame Method, button listener): "+ e2.getMessage());
          }
                         }
                        };
                        aButton.addActionListener(buttonlistenerTitle);
                        nt0Button.addActionListener(buttonlistenerTitle);
                        nt1Button.addActionListener(buttonlistenerTitle);
                        nt2Button.addActionListener(buttonlistenerTitle);
                        bkg1Button.addActionListener(buttonlistenerTitle);
                        bkg2Button.addActionListener(buttonlistenerTitle);
                        nt0Buttonclear.addActionListener(buttonlistenerTitle);
                        nt1Buttonclear.addActionListener(buttonlistenerTitle);
                        nt2Buttonclear.addActionListener(buttonlistenerTitle);
                        bkg1Buttonclear.addActionListener(buttonlistenerTitle);
                        bkg2Buttonclear.addActionListener(buttonlistenerTitle);

                        overridesFrame.setLocation(200,200);
                        overridesFrame.show();
            }
          } catch (Exception e2) {
              System.err.println("Exception in MUDSpectrumFrame loading TDiHist (OverridesFrame Method): "+ e2.getMessage());
              // also pop-up an alert about this
              displayError();
          }
        }


        /**
         * This displays the Frame with the Complex Spectra variables in it
         */
        public void complexSpectrumFrame() {

          try {
          // this is a List of Histograms to load
          List loadHistList = new List();
          // If the Run List is empty, disable skip everyting. The user needs to create a run before being able to set the parameters
          // OR if the user entered the number of hists to load
          if ((runsList.getItemCount()>0) || (!histNumberTextField.getText().trim().equals(new String("")))) {
            // Find out how many histograms the run contains
            if (runsList.getItemCount()>0) {
              // this string from the runsList contains: beamline, year, run#
              final StringTokenizer tokenizer = new StringTokenizer(runsList.getSelectedItem(), " ", false);
          String area = "";
          int year = 0;
          int run = 0;
          if (tokenizer.countTokens()==1) { // this is loaded from a local file that was chosen by the user
            area = tokenizer.nextToken().toLowerCase(); // this will be the directory path with the filename
          } else { // there are 3 tokens
            area = tokenizer.nextToken().toLowerCase();
            year = Integer.parseInt (tokenizer.nextToken());
            run = Integer.parseInt (tokenizer.nextToken());
          }

//----- Load Run Header -------------------------------
//--------------------- Construct filespec:  (site dependent!)

      String fnm=constructFilepath(area, year, run);

      MuSR_RunHeader rH = srv.getRunHdr(fnm);
//------------------------------------------

               // if there is a request for more than one histogram: 'blank' means: all histograms, commas in between means more than one specific histograms
               if (histNumberTextField.getText().trim().equals(new String(""))) {
                 for (int i=1; i<=rH.nHists; i++)  {// find the number of existing histograms for this Run
                   loadHistList.add(String.valueOf(i));
                 }
               } else if (histNumberTextField.getText().trim().indexOf(",")!=-1) {
                 StringTokenizer histToken = new StringTokenizer(histNumberTextField.getText().trim(), ",", false);
                 int numberOfTokens = histToken.countTokens();
                 for (int i=1; i<=numberOfTokens; i++)  {// traverse through the numbers and load them in
                   loadHistList.add(histToken.nextToken().trim());
                 }
               } else { // this means there is only one histogram request, so load that one
                  loadHistList.add(histNumberTextField.getText().trim());
               }
            }

          int histNumber = loadHistList.getItemCount();

          // If the number of Histograms requested is new, resize and overvrite the PhaseRRF Vector
          if (phaseRRF.size()!=histNumber) {
            phaseRRF.clear(); // Erase any previous angles that were stored by the user
            for (int i=0; i<histNumber; i++) {
              // Note: for histNumber=1, we get 0, histNumber=2, we get 0;180, histNumber=3, we get 0; 120; 240, histNumber=4, we get 0;90;180;270 and so on
              phaseRRF.add(new TextField(new String( Integer.toString( (int) i*360/histNumber )) ));
            }
          }
                 // Clean up the earlier contents of the frame
                 aFrameScrollbar.removeAll();
                 aFrame.setBackground(Color.lightGray);

                        // Contains settings to add or remove plot captions
                        JPanel aPanel = new JPanel();
                        aPanel.setLayout(gridbag);

                        // the choice where user chooses the asy1 or asy1bc spectrum
                            c.gridx = 0; c.gridy = 0;
                        aPanel.add(new Label( new String("Using Spectrum:") ), c);
                            c.gridx = 1; c.gridy = 0;
                        aPanel.add(useSpectrum, c);
                        // RRF frequency
                            c.gridx = 0; c.gridy = 1;
                        aPanel.add(new Label( new String("RRF frequency [MHz]:") ), c);
                            c.gridx = 1; c.gridy = 1;
                        aPanel.add(RRFFreq, c);
                        // RRF frequency
                            c.gridx = 0; c.gridy = 2;
                        aPanel.add(new Label( new String("Packed bin size in RRF:") ), c);
                            c.gridx = 1; c.gridy = 2;
                        aPanel.add(RRFPackBin, c);
                            c.gridx = 2; c.gridy = 2;
                        aPanel.add(RRFMenu, c);

                            c.gridx = 1; c.gridy = 3;
                        Button aButton = new Button("Close");
                        aPanel.add(aButton, c);

                        // Enable the RRF Angles only if the Comlex Spectrum with Phases is requested
                        if (spectrumChoice.getSelectedItem()=="n-Hist Complex Asym w/Phases in RRF") {
                          for (int i=0; i<histNumber; i++) {
                            ((TextField)phaseRRF.get(i)).setEnabled(true);
                          }
                        } else  {
                          for (int i=0; i<histNumber; i++) {
                            ((TextField)phaseRRF.get(i)).setEnabled(false);
                          }
                        }

                        // add notice that this is only for the Complex Spextra using Angles
                            c.gridx = 0; c.gridy = 4;
                        aPanel.add(new Label("To specify Phase Angles:"), c);
                            c.gridx = 1; c.gridy = 4;
                        aPanel.add(new Label(""), c);
                        // add the Frame Label
                            c.gridx = 0; c.gridy = 5;
                        aPanel.add(new Label(" Hist #"), c);
                            c.gridx = 1; c.gridy = 5;
                        aPanel.add(new Label(" Phase [deg] "), c);
                        // Load the phase angles
                        for (int i=1; i<=histNumber; i++) {
                              c.gridx = 0; c.gridy = 5+i;
                          aPanel.add(new Label( loadHistList.getItem(i-1) ), c);
                              c.gridx = 1; c.gridy =5+i;
                          aPanel.add((TextField)phaseRRF.get(i-1), c);
                        }


                        aFrameScrollbar.add(aPanel);
                        aFrame.getContentPane().add(aFrameScrollbar);

                        aFrame.setSize(500,400);
                        //aFrame.pack();
                        aFrame.isEnabled();
                        ActionListenerS buttonlistenerTitle = new ActionListenerS() {
                         public void actionPerformed(ActionEvent e) {
                          if (e.getActionCommand().equals("Close"))
                            // Store the phase angle values (in case they were changed)
                            aFrame.dispose();
                         }
                        };
                        aButton.addActionListener(buttonlistenerTitle);

                        aFrame.setLocation(200,200);
                        aFrame.show();
            }
          } catch (Exception e2) {
              System.err.println("Exception in MUDSpectrumFrame loading TDiHist (complexSpectrumFrame Method): "+ e2.getMessage());
              // also pop-up an alert about this
              displayError();
          }
        }
        
  /**
   *This method alters the features of the GUI according to whether it will be used to load time-integral mode data or 
   *time-differential mode data.
   */
  public void setDataType(int dataType)
  {
      if(dataType==INTEGRAL_MODE)
      {
            //figure out what goes here!
      }
      else if(dataType==DIFFERENTIAL_MODE)
      {
            //undoes all the features of INTEGRAL_MODE
      }
      else
          System.out.println("Error in data type selection.");
  }

  /**
   * This method creates the proper filepath for loading a MUD spectrum
   */
  private String constructFilepath(String area, int year, int run) {
      System.out.println(2120);
    String fnm="";
    // If these two are 0, the area field will hold the directory path with the file name
    if (year==0 && run==0) {
      fnm = area;
    } else {
      fnm = run+".msr";
      if ( run < 100000 ) fnm = "0"+fnm;
      if ( run < 10000 ) fnm = "0"+fnm;
      if ( run < 1000 ) fnm = "0"+fnm;
      if ( run < 100 ) fnm = "0"+fnm;
      if ( run < 10 ) fnm = "0"+fnm;
      fnm = filebase+"/"+area+"/"+year+"/"+fnm;
    }
    System.out.println(2134);
//System.out.println(" fnm:"+fnm);
    return fnm;
  }

  /**
   * This method pops up a dialog box informing the user that the MUD Spectrum loading had failed
   */

  private void popAlertDialog(String alert) {
                                     infoDialog.removeAll();
                                     infoDialog.setLayout(new GridLayout(4,1));
                                     infoDialog.add(new Label(alert));
                                     infoDialog.add(OKButton);
                                     infoDialog.setSize(new Dimension(450, 150));
                                     infoDialog.setLocation(100,200);
                                     infoDialog.show();
  }

  /**
   * This method pops up a dialog box informing the user that the MUD Spectrum loading had failed
   */

  private void displayError() {
                                     busyDialog.hide();
                                     infoDialog.removeAll();
                                     infoDialog.setLayout(new GridLayout(4,1));
                                     infoDialog.add(new Label("ERROR:"));
                                     infoDialog.add(new Label("Failed to load the requested Spectrum properly"));
                                     infoDialog.add(new Label(" Check the settings and try again"));
                                     infoDialog.add(OKButton);
                                     infoDialog.setSize(new Dimension(450, 150));
                                     infoDialog.setLocation(100,200);
                                     infoDialog.show();
  }


    // Handle events on the JButtons
    // Add/Removing Columns and their information in/from x-axis/y-axis Lists
    ActionListener buttonListener = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
            worker.interrupt();
            busyDialog.hide();

      }
    };

}

