package musr.client.plot2d;
import java.applet.*;
import java.awt.*;

/**
 * It sets up all the axis information in its superclass Axis and then, using all that information,
 * it draws the appropriate axis in a linear domain.
 *
 * @author Ivan Tucakov, Nikola Tucakov
 */
public class LinearAxis extends Axis implements java.io.Serializable {

  static final long serialVersionUID = 0L; // SUID

	/* Constructor methods */
  public LinearAxis(double min, double max, int numTics, int pixMin, int pixMax, PlotOptionsFrame optionVariables) {
    super(min, max, numTics, pixMin, pixMax, false, optionVariables);
  }
  public LinearAxis() { super(); }

 	/* Instance Methods */

  public void drawAxis(Graphics g) {

	try {

		/* Create Horizontal Linear Axis */

	  if (orient.equals("horizontal")) {

		  // Set the Fonts
                  Font font = new Font( optionVariables.numbersFontType,
                                        optionVariables.numbersFontStyle,
                                        optionVariables.numbersFontSize );
                  g.setFont(font);
                  FontMetrics fMetric = getFontMetrics(font);

                // Draw horizontal line
                g.setColor(optionVariables.borderlineColor);
                if (ticOutput.equals("dash"))
                  g.drawLine(pixMin, appletHeight-pixDist, pixMax, appletHeight-pixDist);

                // Draw Ticks and write Labels numTics times
                for(int i=0; i<numTics; i++) {

                // Draw Tics
                // Don't draw the edge Tics.
                // When ticOutput = "dash", draw regular tics on the line;
                // -Tics can be drawn above or below the axis ("up" or "down")
                // When ticOutput = "dot", draw small dots inside the plot.
                int x = pix(tics[i].getValue(), true);
                if ((i!=0) && (i!=numTics-1) && (pixMin<=x && x<=pixMax)) {
                  if (ticOutput.equals("dash")) {
                    if (ticSide.equals("up")) g.drawLine( x, appletHeight-(pixDist-5),
                                                                                x, appletHeight-pixDist );
                    else if (ticSide.equals("down")) g.drawLine( x, appletHeight-pixDist,
                                                                                       x, appletHeight-(pixDist+5) );
                    else
                      Tools.fail("Error in horizontal LinearAxis: wrong String for ticSide: "+ ticSide);
                  }
                  else if (ticOutput.equals("dot"))
                    g.drawOval(x, appletHeight-pixDist, 1, 1);
                   else
                     Tools.fail("Error in horizontal LinearAxis: wrong String for ticOutput: "+ ticOutput);
                  }

			  // Draw subTics:
			  // if ticOutput is "dash"
			  // For every Tic, draw eight linTics (don't add linTics to the last Tic)
			  // If j==5, make a larger linTic in between Tics
			  if (appletWidth > 150+pixMin-pixMax+appletWidth) {
                            if (ticOutput.equals("dash")) {
                              for(int j=1; j<10; j++) {
                                int linTic =pix( tics[i].getValue() + ticSpacing * j / 10 , true);
                                // Make sure that, if the labels are 'strict' to draw these within the plot
                                if (pixMin<=linTic && linTic<=pixMax) {
                                  if (i!=numTics-1) {
                                    if (j==5) {
                                      if (ticSide.equals("up"))
                                        g.drawLine( linTic, appletHeight-(pixDist-3),
                                                    linTic, appletHeight-pixDist );
                                      if (ticSide.equals("down"))
                                        g.drawLine( linTic, appletHeight-pixDist,
                                                    linTic, appletHeight-(pixDist+3) );
                                    }
                                    else if (appletWidth > 250+pixMin-pixMax+appletWidth) {
                                        if (ticSide.equals("up"))
                                          g.drawLine( linTic, appletHeight-(pixDist-2),
                                                      linTic, appletHeight-pixDist );
                                        if (ticSide.equals("down"))
                                          g.drawLine( linTic, appletHeight-pixDist,
                                                      linTic, appletHeight-(pixDist+2) );
                                    }
                                  }
                                }
                              }
                            }
	                  }

                          // Write Labels (Numbers)
                          // make sure they are within the plot
                          if ((labelOutput) && (ticOutput.equals("dash")) && (pixMin<=x && x<=pixMax)) {
                            String str = Double.toString(Tools.fraction(tics[i].getValue(),numFraction));
                            // Fix up the number if it needs to look nice
                            if (str.substring(str.length()-2).equals(new String(".0"))) str = str.substring(0,str.length()-2); // Cut off a ".0" from an end if it is there
                            g.drawString( str,x - fMetric.stringWidth(str)/2,appletHeight-(pixDist-25) );
                            // if the Label is equal to 0.0, draw a vertical dashed line there
                            if (tics[i].getValue()==0) {
                              if (optionVariables.yZeroAxisMenuItem.getState()) {
                                Graphics2D g2 = (Graphics2D) g;
                               g2.setStroke(dashed);
                                g2.drawLine(x, appletHeight-optionVariables.downDistance, x, optionVariables.upDistance);
                                g2.setStroke(new BasicStroke());
                              }
                            }
			  }
		  }
	  }

		/* Create Vertical Linear Axis */

	  else if (orient.equals("vertical")) {

		  // Set the Fonts
                  Font font = new Font( optionVariables.numbersFontType,
                                        optionVariables.numbersFontStyle,
                                        optionVariables.numbersFontSize );
                  g.setFont(font);
                  FontMetrics fMetric = getFontMetrics(font);

		  // Draw vertical line
		  g.setColor(optionVariables.borderlineColor);
		  if (ticOutput.equals("dash"))
			  g.drawLine(pixDist, pixMin, pixDist, pixMax);

		  // Draw Ticks and write Labels numTics times
		  for(int i=0; i<numTics; i++) {

			  // Draw Tics
			  // Don't draw the edge tics
 				// When ticOutput = "dash", draw regular tics on the line;
				// -Tics can be drawn left or right the axis ("left" or "right")
 				// When ticOutput = "dot", draw small dots inside the plot.
			  int y = appletHeight - pix(tics[i].getValue(), false);
			  if ((i!=0) && (i!=numTics-1) && (pixMin<=y && y<=pixMax)) {
				  if (ticOutput.equals("dash")) {
					  if (ticSide.equals("left")) g.drawLine(pixDist-5, y, pixDist, y);
					  else if (ticSide.equals("right")) g.drawLine(pixDist, y, pixDist+5, y);
					  else
						  Tools.fail("Error in vertical LinearAxis: wrong String for ticSide: "
												 + ticSide);
				  }
				  else if (ticOutput.equals("dot"))
					  g.drawOval(pixDist, y, 1, 1);
				  else
					  Tools.fail("Error in vertical LinearAxis: wrong String for ticOutput: "
											 + ticOutput);
			  }

			  // Draw SubTics:
			  // if ticOutput is "dash"
			  // For every Tic, draw eight linTics (don't add linTics to the last Tic)
			  // If j==5, make a larger linTic in between Tics
  			  if (appletHeight > 150+pixMin-pixMax+appletHeight){
                            if (ticOutput.equals("dash")) {
                              for(int j=1; j<10; j++) {
                                int linTic = appletHeight -
					pix( tics[i].getValue() + ticSpacing * j / 10 , false);
                                // Make sure that, if the labels are 'strict' to draw these within the plot
                                if (pixMin<=linTic && linTic<=pixMax) {
                                  if (i!=numTics-1) {
                                    if (j==5) {
                                      if (ticSide.equals("left"))
                                        g.drawLine(pixDist-3, linTic, pixDist, linTic);
                                      if (ticSide.equals("right"))
                                          g.drawLine(pixDist, linTic, pixDist+3, linTic);
                                    }
                                    else
                                    if (appletHeight > 250+pixMin-pixMax+appletHeight) {
                                      if (ticSide.equals("left"))
                                        g.drawLine(pixDist-2, linTic, pixDist, linTic);
                                      if (ticSide.equals("right"))
                                        g.drawLine(pixDist, linTic, pixDist+2, linTic);
                                    }
                                  }
                                }
                              }
                            }
			  }

                          // Write Labels
                          // make sure they are within the plot
                          if ((labelOutput) && (ticOutput.equals("dash")) && (pixMin<=y && y<=pixMax)) {
                            String str = Double.toString(Tools.fraction(tics[i].getValue(), numFraction));
                            // Fix up the number if it needs to look nice
                            if (str.substring(str.length()-2).equals(new String(".0"))) str = str.substring(0,str.length()-2); // Cut off a ".0" from an end if it is there
                            g.drawString( str,pixDist - fMetric.stringWidth(str) - 15,y + fMetric.getMaxAscent()/2 ); // use Ascent instead of Height
//System.out.println("getHeight: "+fMetric.getHeight()+"getHeight/2: "+fMetric.getHeight()/2+" Maxascent: "+fMetric.getMaxAscent()+" Maxascent/2: "+fMetric.getMaxAscent()/2);
                            // if the Label is equal to 0.0, draw a horizontal dashed line there
                            if (tics[i].getValue()==0.0) {
                              if (optionVariables.xZeroAxisMenuItem.getState()) {
                                Graphics2D g2 = (Graphics2D) g;
                                g2.setStroke(dashed);
                                g2.drawLine(optionVariables.leftDistance, y, appletWidth - optionVariables.rightDistance, y);
                                g2.setStroke(new BasicStroke());
                              }
                            }
			  }
		  }
	  }
	  else Tools.fail("Error in LinearAxis: wrong String for orient: " + orient);
	}
	catch (Exception e) {System.err.println(e.getMessage());}
  }

}
