package musr.client.plot2d;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.lang.*;
import java.util.*;
import javax.swing.*;
import musr.client.datacolumn.*;
import musr.client.gui.*;

/**
 * A program that uses all the standard AWT components
 * Creates a frame to set-up various plotting properties of the graphs
 * and the overall appearance of the plot.
 * This class is later used in other classes as a 'package' that contains
 * all the necessary information to draw the plot.orizontal"
 *
 * it contains:
 *
 * @author Ivan Tucakov, Nikola Tucakov
 */
public class PlotOptionsFrame extends Frame implements java.io.Serializable {

  static final long serialVersionUID = 0L; // SUID

		/* Constants */
		public static final String COLORS[] = {"Black", "Red", "Blue", "Green", "Orange", "Magenta",
                                               "Cyan", "Dark Gray", "Pink", "Gray", "White",
                                               "Light Gray" };
                                           public static final String POINT_TYPE[] =
                                             { "square", "circle", "triangle", "nabla", "diamond", "star", "cross", "heart" };
		public static final String FONT_TYPES[] = { "Serif (TimesRoman)", "SansSerif (Helvetica)",
                                               "Monospaced (Courier)","Dialog", "DialogInput" };
		public static final String FONT_STYLES[] = { "PLAIN", "BOLD", "ITALIC" };

		/* Variables */

		/* Helper */
              /* General */
              public Vector graphTypeLine = new Vector();
              public Vector graphTypeOther = new Vector();
              public Vector graphSymbolSize = new Vector();
              public Vector graphSymbolFilled = new Vector();
              public String axisTypeX = "LinearAxis";
              public String axisTypeY = "LinearAxis";
              /* Colors */
              public Color backgroundColor = Color.white;
              public Color borderlineColor = Color.black;
              public Vector graphColor = new Vector();

              /* Fonts */
              public String labelFontType = "Helvetica";
              public int labelFontStyle = Font.PLAIN;
              public int labelFontSize = 24;
              public String titleFontType = "Helvetica";
              public int titleFontStyle = Font.BOLD;
              public int titleFontSize = 24;
              public String numbersFontType = "Helvetica";
              public int numbersFontStyle = Font.PLAIN;
              public int numbersFontSize = 18;

              /* Plot setup - just make sure that verticals equal horizontals when added
                  note (not applied here): the amount that is subtracted horizontally has to be in the ratio of 5 to 4
                  to the ammount subtracted vertically so that the landscape print size ratio would be preserved.
                  ALso, there is extra ~40 pixels from the top o fframe to where plot begins
                  90+80+20/130+40  ~  4/5  (it doesn't really apply.... changes hav been made, it doesn't really matter) */
              public int upDistance = 90;
              public int downDistance = 80;
              public int leftDistance = 130;
              public int rightDistance = 40;

                /** Used for zooming features, it is loaded only from the Plot2D class */
                public Stack stack = new Stack();

		/* Buttons */
    Button okay = new Button("OK");
    Button cancel = new Button("Cancel");

    /* Checkboxes, grouped in a CheckboxGroups to give them "radio button" behavior.*/
    // Make xAxisType
    CheckboxGroup xType = new CheckboxGroup();
    public Checkbox[] xTypes = new Checkbox[3];
    // Make yAxisType
    CheckboxGroup yType = new CheckboxGroup();
    public Checkbox[] yTypes = new Checkbox[3];
    // Make graphType
    public Checkbox gTypeLine = new Checkbox("Connect with line", true);

    public Checkbox filledSymbol = new Checkbox("Fill symbols", true);

		/* Choices */
		public Choice groupNumberChoice = new Choice();
	                      public Choice graphNumberChoice = new Choice();
		public Choice gTypeOtherChoice = new Choice();
		public Choice graphSymbolSizeChoice = new Choice();
		public Choice graphColorChoice = new Choice();

		/* Labels */
		// General Status Information Labels
		public JLabel backgroundColorLabel = new JLabel("White");
		public JLabel borderlineColorLabel = new JLabel("Black");

		public JLabel labelTypeLabel = new JLabel(labelFontType);
		public JLabel labelStyleLabel = new JLabel(FONT_STYLES[0]); //PLAION
		public JLabel labelSizeLabel = new JLabel(Long.toString(this.labelFontSize));
		public JLabel titleTypeLabel = new JLabel(titleFontType);
		public JLabel titleStyleLabel = new JLabel(FONT_STYLES[1]); //BOLD
		public JLabel titleSizeLabel = new JLabel(Long.toString(this.titleFontSize));
		public JLabel numbersTypeLabel = new JLabel(numbersFontType);
		public JLabel numbersStyleLabel = new JLabel(FONT_STYLES[0]); //PLAIN
		public JLabel numbersSizeLabel = new JLabel(Long.toString(this.numbersFontSize));

		/* Menus */
              
                
                // "Global Defaults" CheckboxMenus
                // Strict Limits checkbox determines whether the graph labels will be Strict (tight) or Loose
                public CheckboxMenuItem strictLimitsMenuItem = new CheckboxMenuItem("Strict Limits", false);
                // Connect all points in graphs with lines
                public CheckboxMenuItem allPointsLineMenuItem = new CheckboxMenuItem("Connect Points", false);
                // make X Axis Monotonic (accending order)
                public CheckboxMenuItem monotonicMenuItem = new CheckboxMenuItem("Plot with X Sorted", true);
                // Display details for power of ten in log axis (e.g. 2x10^3, 3x10^4..)
                public CheckboxMenuItem detailedPowerTenMenuItem = new CheckboxMenuItem("Display Details for Power of Ten in Log Axis", true);
                // Make the plot symbols Automatically filled
                public CheckboxMenuItem symbolsFilledMenuItem = new CheckboxMenuItem("Fill Plot Symbols", true);

                // "Color Options" Menus
		public Menu colorMenu = new Menu("Color Options");
		  public Menu backgroundMenu = new Menu("Background Color ("+backgroundColorLabel.getText()+")");
		  public Menu borderlineMenu = new Menu("Borderline Color ("+borderlineColorLabel.getText()+")");
		// "Font Options" Menus
		public Menu labelFontMenu = new Menu("Label Font");
		  public Menu labelTypeMenu = new Menu("Type ("+labelTypeLabel.getText()+")");
		  public Menu labelStyleMenu = new Menu("Style ("+labelStyleLabel.getText()+")");
		  public Menu labelSizeMenu = new Menu("Size ("+labelSizeLabel.getText()+")");
		public Menu titleFontMenu = new Menu("Title Font");
		  public Menu titleTypeMenu = new Menu("Type ("+titleTypeLabel.getText()+")");
		  public Menu titleStyleMenu = new Menu("Style ("+titleStyleLabel.getText()+")");
		  public Menu titleSizeMenu = new Menu("Size ("+titleSizeLabel.getText()+")");
                public CheckboxMenuItem adjustToFitMenuItem = new CheckboxMenuItem("Adjust to fit", true);
		public Menu numbersFontMenu = new Menu("Numbers Font");
		  public Menu numbersTypeMenu = new Menu("Type ("+numbersTypeLabel.getText()+")");
		  public Menu numbersStyleMenu = new Menu("Style ("+numbersStyleLabel.getText()+")");
		  public Menu numbersSizeMenu = new Menu("Size ("+numbersSizeLabel.getText()+")");
                // Dashed Lines zero axes Menu
                public Menu setupMenuA = new Menu("Zero Axes");
                  public CheckboxMenuItem xZeroAxisMenuItem = new CheckboxMenuItem("Horizontal", true);
                  public CheckboxMenuItem yZeroAxisMenuItem = new CheckboxMenuItem("Vertical", true);
		// "Plot Setup Options" Menu
                public Menu setupMenuP = new Menu("Plot Location (pixels)");
              // "Graph Setup Options" Menu
              public Menu setupMenuG = new Menu("Graph Setup");
              public CheckboxMenuItem errorBarsMenuItem = new CheckboxMenuItem("Error Bars", true);
              public CheckboxMenuItem errorBarTicsMenuItem = new CheckboxMenuItem("Error Bar Edge Tics", false);
              public CheckboxMenuItem dotFiducialsMenuItem = new CheckboxMenuItem("Dot Fiducials", false);

              GridBagConstraints gridConstr = new GridBagConstraints();

              /* TextFields */
              public TextField text = new TextField();
              public TextField xMinText = new TextField();
              public TextField xMaxText = new TextField();
              public TextField yMinText = new TextField();
              public TextField yMaxText = new TextField();

              public TextField typeColorTextField = new TextField(5);
              
              /*Spinners for point type and colour*/
              /*public JSpinner graphPointSpinner = new JSpinner(new SpinnerListModel(PlotOptionsFrame.POINT_TYPE));
              public JSpinner graphColourSpinner = new JSpinner(new SpinnerListModel(PlotOptionConstants.COLOR));*/

  //These are the Event Listeners
  private class ItemListenerS implements java.awt.event.ItemListener, java.io.Serializable {
    public void itemStateChanged(ItemEvent e) {}
  };
  ItemListenerS checkbox_listenerX;
  ItemListenerS checkbox_listenerY;
  ItemListenerS checkbox_listenerG;
  ItemListenerS checkbox_listenerF;
  ItemListenerS choice_listenerG;
  ItemListenerS choice_listenerS;
  ItemListenerS choice_listenerC;
  ItemListenerS checkbox_listenerI;
  ItemListenerS checkboxMenu_listenerLine;
  ItemListenerS choice_listenerN1;
  ItemListenerS choice_listenerN2;

  private class MouseListenerS implements java.awt.event.MouseListener, java.io.Serializable {
    public void mousePressed(MouseEvent e) {}
    public void mouseReleased(MouseEvent e) {}
    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}
    public void mouseClicked(MouseEvent e) {}
  };
  MouseListenerS choice_mouselisnrN;

  private class ActionListenerS implements java.awt.event.ActionListener, java.io.Serializable {
    public void actionPerformed(ActionEvent e) {}
  };
  ActionListenerS buttonListener;
  ActionListenerS moreOptionsListener;

  private class KeyListenerF implements java.awt.event.KeyListener, java.io.Serializable {
    public void keyReleased(KeyEvent e) {}
    public void keyPressed(KeyEvent e) {}
    public void keyTyped(KeyEvent e) {}
  };
  KeyListenerF frameListener;

  // This Label lets the user know which Frame is currently being used
  public JLabel layoutFrameLabel;

  // these are the Frames that will contain different components
  /** This Label is for the purpose of 'recognizing' which Plot Frame is this Window for in the GUI */
  public JPanel axisTypesPanel;

  /** This Label is for the purpose of 'recognizing' which Plot Frame is this Window for in the GUI */
  public JPanel graphTypesPanel;

  /** This Label is for the purpose of 'recognizing' which Plot Frame is this Window for in the GUI */
  public JPanel titleSetupPanel;
  public TextField titleText = new TextField();
  public TextField xLabelText = new TextField();
  public TextField yLabelText = new TextField();

  /** This List holds the vaues for the captions that will be displayed in the plot */
  public JTextArea captionsTextArea = new JTextArea();

  /** This Label is for the purpose of 'recognizing' which Plot Frame is this Window for in the GUI */
  public JPanel minMaxPanel;

  /** This Label is for the purpose of 'recognizing' which Plot Frame is this Window for in the GUI */
  public JPanel statusInfoPanel;

  /**
   * This is the actual GUI that this Frame is being called from, it is defined through the
   * setParentGUI(Frame) method in the GUI class
   */
  private musr.client.gui.GUI parentGUI;

  /**
   * Create the whole GUI, and set up event listeners
   */
  public PlotOptionsFrame() {
    super("Plot Options");  // set frame title.
    this.setBackground(Color.lightGray);

    // Set a default font
    this.setFont(new Font("SansSerif", Font.PLAIN, 12));

		//
		/* setLayout where to put contents */
		//

    // Now that we've done the menu, we can begin work on the contents of
    // the frames.
    this.setLayout(new BorderLayout(10, 10));

    layoutFrameLabel = new JLabel("*** Frame #1 ***");

    // Create all the Panels for this frame
    axisTypesPanel = new JPanel();
    axisTypesPanel.setLayout(new GridBagLayout());

    graphTypesPanel = new JPanel();
    graphTypesPanel.setLayout(new GridBagLayout());

    titleSetupPanel = new JPanel();
    titleSetupPanel.setLayout(new GridBagLayout());

    minMaxPanel = new JPanel();
    minMaxPanel.setLayout(new GridBagLayout());

    statusInfoPanel = new JPanel();
    statusInfoPanel.setLayout(new GridBagLayout());

    // Some default values for the constraints object
    gridConstr.fill = GridBagConstraints.BOTH;
    gridConstr.insets = new Insets(0,5,0,5);

		/* General Status Information */
		// Inside statusInfoPanel with a GridBagLayout

		gridConstr.gridx = 0; gridConstr.gridy = 0;
		statusInfoPanel.add(new JLabel("*General Status*"), gridConstr);


		gridConstr.gridx = 0; gridConstr.gridy = 1;
		statusInfoPanel.add(new JLabel("Colors:"), gridConstr);

		gridConstr.gridx = 0; gridConstr.gridy = 2;
		statusInfoPanel.add(new JLabel("  Background Color:"), gridConstr);
		gridConstr.gridx = 1; gridConstr.gridy = 2;
		statusInfoPanel.add(backgroundColorLabel, gridConstr);

		gridConstr.gridx = 0; gridConstr.gridy = 3;
		statusInfoPanel.add(new JLabel("  Borderline Color:"), gridConstr);
		gridConstr.gridx = 1; gridConstr.gridy = 3;
		statusInfoPanel.add(borderlineColorLabel, gridConstr);

		gridConstr.gridx = 0; gridConstr.gridy = 4;
		statusInfoPanel.add(new JLabel("Fonts:"), gridConstr);

		gridConstr.gridx = 0; gridConstr.gridy = 5;
		statusInfoPanel.add(new JLabel("  LABEL Font Type:"), gridConstr);
		gridConstr.gridx = 1; gridConstr.gridy = 5;
		statusInfoPanel.add(labelTypeLabel, gridConstr);

		gridConstr.gridx = 0; gridConstr.gridy = 6;
		statusInfoPanel.add(new JLabel("  LABEL Font Style:"), gridConstr);
		gridConstr.gridx = 1; gridConstr.gridy = 6;
		statusInfoPanel.add(labelStyleLabel, gridConstr);

		gridConstr.gridx = 0; gridConstr.gridy = 7;
		statusInfoPanel.add(new JLabel("  LABEL Font Size:"), gridConstr);
		gridConstr.gridx = 1; gridConstr.gridy = 7;
		statusInfoPanel.add(labelSizeLabel, gridConstr);

		gridConstr.gridx = 0; gridConstr.gridy = 8;
		statusInfoPanel.add(new JLabel("  TITLE Font Type:"), gridConstr);
		gridConstr.gridx = 1; gridConstr.gridy = 8;
		statusInfoPanel.add(titleTypeLabel, gridConstr);

		gridConstr.gridx = 0; gridConstr.gridy = 9;
		statusInfoPanel.add(new JLabel("  TITLE Font Style:"), gridConstr);
		gridConstr.gridx = 1; gridConstr.gridy = 9;
		statusInfoPanel.add(titleStyleLabel, gridConstr);

		gridConstr.gridx = 0; gridConstr.gridy = 10;
		statusInfoPanel.add(new JLabel("  TITLE Font Size:"), gridConstr);
		gridConstr.gridx = 1; gridConstr.gridy = 10;
		statusInfoPanel.add(titleSizeLabel, gridConstr);

		gridConstr.gridx = 0; gridConstr.gridy = 11;
		statusInfoPanel.add(new JLabel("  NUMBERS Font Type:"), gridConstr);
		gridConstr.gridx = 1; gridConstr.gridy = 11;
		statusInfoPanel.add(numbersTypeLabel, gridConstr);

		gridConstr.gridx = 0; gridConstr.gridy = 12;
		statusInfoPanel.add(new JLabel("  NUMBERS Font Style:"), gridConstr);
		gridConstr.gridx = 1; gridConstr.gridy = 12;
		statusInfoPanel.add(numbersStyleLabel, gridConstr);

		gridConstr.gridx = 0; gridConstr.gridy = 13;
		statusInfoPanel.add(new JLabel("  NUMBERS Font Size:"), gridConstr);
		gridConstr.gridx = 1; gridConstr.gridy = 13;
		statusInfoPanel.add(numbersSizeLabel, gridConstr);
    /* Axis Types Options */
    // Inside axisTypesPanel with a GridBagLayout

    // Set xAxisType
    xTypes[0] = new Checkbox("Linear", xType, true);
    xTypes[1] = new Checkbox("Logarithmic", xType, false);
    xTypes[2] = new Checkbox("Inverse", xType, false);
    // Set GridBagConstraints
    gridConstr.gridx = 0; gridConstr.gridy = 0;
    axisTypesPanel.add(new JLabel("X-axis Type:"), gridConstr);
    for(int i = 0; i < xTypes.length; i++) {
				gridConstr.gridx = 0; gridConstr.gridy = i+1;
				axisTypesPanel.add(xTypes[i], gridConstr);
    }

    // Handle events on the X Axis Type checkboxes
    checkbox_listenerX = new ItemListenerS() {
      public void itemStateChanged(ItemEvent e) {

        String command = ((Checkbox)e.getItemSelectable()).getLabel();
        if (command.equals("Linear"))
          axisTypeX = PlotOptionConstants.AXIS_TYPE[0];
        if (command.equals("Logarithmic"))
          axisTypeX = PlotOptionConstants.AXIS_TYPE[1];
        if (command.equals("Inverse"))
          axisTypeX = PlotOptionConstants.AXIS_TYPE[2];
      }
    };
    for(int i = 0; i < xTypes.length; i++)
      xTypes[i].addItemListener(checkbox_listenerX);

		// Set yAxisType
    yTypes[0] = new Checkbox("Linear", yType, true);
    yTypes[1] = new Checkbox("Logarithmic", yType, false);
    yTypes[2] = new Checkbox("Inverse", yType, false);
		// Set GridBagConstraints
		gridConstr.gridx = 0; gridConstr.gridy = 4;
    axisTypesPanel.add(new JLabel("Y-axis Type:"), gridConstr);
    for(int i = 0; i < yTypes.length; i++) {
				gridConstr.gridx = 0; gridConstr.gridy = i+5;
				axisTypesPanel.add(yTypes[i], gridConstr);
		}
    // Handle events on the Y Axis Type checkboxes
    checkbox_listenerY = new ItemListenerS() {
      public void itemStateChanged(ItemEvent e) {
        String command = ((Checkbox)e.getItemSelectable()).getLabel();
        if (command.equals("Linear")) axisTypeY = PlotOptionConstants.AXIS_TYPE[0];
        if (command.equals("Logarithmic"))	axisTypeY = PlotOptionConstants.AXIS_TYPE[1];
        if (command.equals("Inverse"))	axisTypeY = PlotOptionConstants.AXIS_TYPE[2];
      }
    };
    for(int i = 0; i < yTypes.length; i++)
      yTypes[i].addItemListener(checkbox_listenerY);

        // Set graphType and Colors

        // Check GroupNumberChoice and GraphNumberChoice for Mouse Events
        choice_mouselisnrN = new MouseListenerS() {
          public void mousePressed(MouseEvent e) {
            if ( graphNumberChoice.getItemCount()!=0 ) {
              // If the typeColorTextField is not empty, load its values into appropriate group and graph vectors:  (int)type+2.(int)color
              if (!typeColorTextField.getText().trim().equals("")) {
                int tempTypeIndex = 0;
                int tempColorIndex = 0;
                if (typeColorTextField.getText().trim().indexOf(".")!=-1 ) { // the form is "(int)type.(int)color"
                  tempTypeIndex = Integer.parseInt( typeColorTextField.getText().trim().substring(0,typeColorTextField.getText().trim().indexOf(".")) );
                  tempColorIndex = Integer.parseInt( typeColorTextField.getText().trim().substring(typeColorTextField.getText().trim().indexOf(".")+1) );
                } else { // the form is "(int)type"
                  tempTypeIndex = Integer.parseInt( typeColorTextField.getText().trim() )%14; // divide using MOD, since there are 14 types of Symbols
                  // Calculate the total number of graphs
                  int tempCount = 0;
                  for (int x =0; x<groupNumberChoice.getSelectedIndex(); x++) {
                    for (int y=0; y<((DataTable)((Vector) parentGUI.yAxisDataTable.elementAt(parentGUI.layoutChoice.getSelectedIndex())).elementAt(x)).columnCount();y++) {// count them all
                      tempCount = tempCount+1;
                    }
                  }
                  tempCount = tempCount+graphNumberChoice.getSelectedIndex(); // add last group items
                  tempColorIndex = (tempCount)%12; // divide using MOD, since there are 12 types of Colors
                }
                // Set Symbol Type   - note: '+2' is because there is already 'none' and 'dot' items in the gTypeOtherChoice
                ((Vector)graphTypeOther.elementAt(groupNumberChoice.getSelectedIndex())).
                                                     setElementAt( POINT_TYPE[tempTypeIndex] , graphNumberChoice.getSelectedIndex() );
                // Set Symbol (Graph) Color
                ((Vector)graphColor.elementAt(groupNumberChoice.getSelectedIndex())).
                                                setElementAt(PlotOptionConstants.COLOR[tempColorIndex], graphNumberChoice.getSelectedIndex() );
              }
            }
            
              /*String pointType = (String)graphPointSpinner.getModel().getValue();
              Color pointColour = (Color)graphColourSpinner.getModel().getValue();
              
            ((Vector)graphTypeOther.elementAt(groupNumberChoice.getSelectedIndex())).setElementAt(pointType, graphNumberChoice.getSelectedIndex());
            ((Vector)graphColor.elementAt(groupNumberChoice.getSelectedIndex())).setElementAt(pointColour, graphNumberChoice.getSelectedIndex());*/
            
          }
        };
        groupNumberChoice.addMouseListener(choice_mouselisnrN);
        graphNumberChoice.addMouseListener(choice_mouselisnrN);
        // Handle events on the GroupNumberChoice Choice
        //      (Which was created by calling method setNumberOfGraphs)
        choice_listenerN1 = new ItemListenerS() {
          public void itemStateChanged(ItemEvent e) {
            // Show the 'Graph Number' Choice if for the selected group
            graphNumberChoice.removeAll();
            for(int i=0; i<((DataTable)((Vector)parentGUI.yAxisDataTable.elementAt(parentGUI.layoutChoice.getSelectedIndex()))
                                                              .elementAt(groupNumberChoice.getSelectedIndex())).columnCount(); i++) {
              graphNumberChoice.addItem("Graph #" + Integer.toString(i+1)+"  ("+
                ((DataColumn) ((DataTable)
                ((Vector) parentGUI.yAxisDataTable.elementAt(parentGUI.layoutChoice.getSelectedIndex())).
                                                       elementAt(groupNumberChoice.getSelectedIndex())).getColumn(i)).getName()+ ")"  ); }
            // If there are graphs in the selected group
            if ( graphNumberChoice.getItemCount()==0 ) {
              gTypeLine.setEnabled(false);
              gTypeOtherChoice.setEnabled(false);
              typeColorTextField.setEnabled(false);
              graphSymbolSizeChoice.setEnabled(false);
              graphColorChoice.setEnabled(false);
              filledSymbol.setEnabled(false);
            }
            else {
              // Set their values for the selected graph
              gTypeLine.setState( ( (Boolean) ((Vector) graphTypeLine.
                                                    elementAt(groupNumberChoice.getSelectedIndex())).
                                                      elementAt(graphNumberChoice.getSelectedIndex()) ).
                                                        booleanValue() );
              gTypeOtherChoice.select( (String)((Vector) graphTypeOther.
                                                    elementAt(groupNumberChoice.getSelectedIndex())).
                                                      elementAt(graphNumberChoice.getSelectedIndex()) );
              graphSymbolSizeChoice.select( (String)((Vector) graphSymbolSize.
                                                    elementAt(groupNumberChoice.getSelectedIndex())).
                                                      elementAt(graphNumberChoice.getSelectedIndex()) );

              for(int i=0; i<12; i++)
                if ( (Color)((Vector) graphColor.
                                                    elementAt(groupNumberChoice.getSelectedIndex())).
                                                      elementAt(graphNumberChoice.getSelectedIndex()) ==
                                                          PlotOptionConstants.COLOR[i] ) {
                  graphColorChoice.select(i);
                  break;
                }
              filledSymbol.setState( ( (Boolean) ((Vector) graphSymbolFilled.
                                                    elementAt(groupNumberChoice.getSelectedIndex())).
                                                      elementAt(graphNumberChoice.getSelectedIndex()) ).
                                                        booleanValue() );
              // Fill the typeColorTextField with appropriate numbers: (int)type.(int)color     (note: '-2' because there is types: 'none' and 'dot' before all other)
              typeColorTextField.setText( String.valueOf(gTypeOtherChoice.getSelectedIndex()-2)+"."+String.valueOf(graphColorChoice.getSelectedIndex()) );
              // Enable the Options
              gTypeLine.setEnabled(true);
              gTypeOtherChoice.setEnabled(true);
              // If the selected graph doesn't have some symbol for it's points set 'Size' Choice to false
              if (gTypeOtherChoice.getSelectedItem().equals("none") || gTypeOtherChoice.getSelectedItem().equals("dot")) {
                graphSymbolSizeChoice.setEnabled(false);
                filledSymbol.setEnabled(false);
              } else {
                filledSymbol.setEnabled(true);
                graphSymbolSizeChoice.setEnabled(true);
              }
              graphColorChoice.setEnabled(true);
            }
          }
        };
    groupNumberChoice.addItemListener(choice_listenerN1);

    // Handle events on the GraphNumberChoice Choice
    //      (Which was created by calling method setNumberOfGraphs)
    choice_listenerN2 = new ItemListenerS() {
      public void itemStateChanged(ItemEvent e) {
        // Set the 'Line' CheckBox and the 'Other' Choice to properties of the selected graph.
        // Values are taken from Vectors tempGraphTypeLine and Vector TempGraphTypeOther by first
        // taking appropriate Vector inside these Vectors at position
        // groupNumberChoice.getSelectedIndex(). Then values in these new Vectors are called by
        // finding the index value of selected graphNumberChoice.
        // Same thing is done for the 'Color' Choice.
        gTypeLine.setState( ( (Boolean) ((Vector) graphTypeLine.
                                          elementAt(groupNumberChoice.getSelectedIndex())).
                                            elementAt(graphNumberChoice.getSelectedIndex()) ).
                                        booleanValue() );
        gTypeOtherChoice.select( (String)((Vector) graphTypeOther.
                                          elementAt(groupNumberChoice.getSelectedIndex())).
                                            elementAt(graphNumberChoice.getSelectedIndex()) );

        graphSymbolSizeChoice.select( (String)((Vector) graphSymbolSize.
                                          elementAt(groupNumberChoice.getSelectedIndex())).
                                            elementAt(graphNumberChoice.getSelectedIndex()) );
        for(int i=0; i<12; i++)
          if ( (Color)((Vector) graphColor.elementAt(groupNumberChoice.getSelectedIndex())).elementAt(graphNumberChoice.getSelectedIndex()) ==
				PlotOptionConstants.COLOR[i] ) {
                                graphColorChoice.select(i);
                                break;
          }
          filledSymbol.setState( ( (Boolean) ((Vector) graphSymbolFilled.
                                                    elementAt(groupNumberChoice.getSelectedIndex())).
                                                      elementAt(graphNumberChoice.getSelectedIndex()) ).
                                                        booleanValue() );
        // Fill the typeColorTextField with appropriate numbers: (int)type.(int)color     (note: '-2' because there is types: 'none' and 'dot' before all other)
        typeColorTextField.setText( String.valueOf(gTypeOtherChoice.getSelectedIndex()-2)+"."+String.valueOf(graphColorChoice.getSelectedIndex()) );
        // If the selected graph doesn't have some symbol for it's points set 'Size' Choice to false
        if (gTypeOtherChoice.getSelectedItem().equals("none") || gTypeOtherChoice.getSelectedItem().equals("dot")) {
          filledSymbol.setEnabled(false);
          graphSymbolSizeChoice.setEnabled(false);
        } else {
        filledSymbol.setEnabled(true);
          graphSymbolSizeChoice.setEnabled(true);
        }
      }
    };
    // Add an item Listener to graphNumberChoice
    graphNumberChoice.addItemListener(choice_listenerN2);

    // Graph Type Settings:
    gridConstr.gridx = 0; gridConstr.gridy = 2;
    graphTypesPanel.add(gTypeLine, gridConstr);
    // Handle events on the GraphTypeLine checkbox
    // Assign the 'Line' boolean value to the appropriate tempGraphTypeLine
    checkbox_listenerG = new ItemListenerS() {
      public void itemStateChanged(ItemEvent e) {
        boolean command = ((Checkbox)e.getItemSelectable()).getState();
        ((Vector)graphTypeLine.elementAt(groupNumberChoice.getSelectedIndex())).
	setElementAt( new Boolean(command), graphNumberChoice.getSelectedIndex() );
      }
    };
    gTypeLine.addItemListener(checkbox_listenerG);

    // Symbol Size Settings
    gridConstr.gridx = 0; gridConstr.gridy = 3;
    graphTypesPanel.add(new JLabel("Set Symbol Size:"), gridConstr);
    // graphSymbolSizeChoice
    graphSymbolSizeChoice.removeAll();
    for(int i=1; i<=30; i++)
      graphSymbolSizeChoice.addItem(Integer.toString(i));
    // Set GridBagConstraints for 'Symbol Size' Choice
    gridConstr.gridx = 0; gridConstr.gridy = 4;
    // Don't show the 'Symbol Size' Choice if there are no graphs in the plot
    // and if the selected graph doesn't have some symbol for it's points
    graphTypesPanel.add(graphSymbolSizeChoice, gridConstr);
    // Handle events on the Graph Symbol Size Choice
    choice_listenerS = new ItemListenerS() {
      public void itemStateChanged(ItemEvent e) {
        String command = ((Choice)e.getItemSelectable()).getSelectedItem();
        // Assign the size of the graph symbol selected
        ((Vector)graphSymbolSize.elementAt(groupNumberChoice.getSelectedIndex())).
        	setElementAt(command, graphNumberChoice.getSelectedIndex() );
      }
    };
    graphSymbolSizeChoice.addItemListener(choice_listenerS);
    // Since there are initially no Graphs, disable the 'Graph Symbol Size' Choice
    graphSymbolSizeChoice.setEnabled(false);


    gridConstr.gridx = 0; gridConstr.gridy = 12;
    graphTypesPanel.add(typeColorTextField, gridConstr);
    
    /*
    gridConstr.gridx = 0; gridConstr.gridy = 6;
    graphTypesPanel.add(graphPointSpinner, gridConstr);
    
    gridConstr.gridx = 1; gridConstr.gridy = 6;
    graphTypesPanel.add(graphColourSpinner, gridConstr);
     **/

    gridConstr.gridx = 0; gridConstr.gridy = 6;
    graphTypesPanel.add(new JLabel("Set Graph Type:"), gridConstr);
    gTypeOtherChoice.removeAll();
    // Insert Graph Type Choice
    // 'Other' Choice:
    gTypeOtherChoice.addItem("none");
    gTypeOtherChoice.addItem("dot");
    // the settings actually start from here
    for(int i=0; i<POINT_TYPE.length; i++)
    	gTypeOtherChoice.addItem(POINT_TYPE[i]);
    gridConstr.gridx = 0; gridConstr.gridy = 7;
    graphTypesPanel.add(gTypeOtherChoice, gridConstr);

    // Handle events on the GraphTypeOther Choice
    choice_listenerG = new ItemListenerS() {
      public void itemStateChanged(ItemEvent e) {
        String command = ((Choice)e.getItemSelectable()).getSelectedItem();
        /*if (command.equals("none")) {*/
          // Switch the Line CheckBox on and draw the line
          /*((Vector)graphTypeLine.elementAt(groupNumberChoice.getSelectedIndex())).
            setElementAt( new Boolean(true), graphNumberChoice.getSelectedIndex() );*/
            //gTypeLine.setState(true); // this will make graph invisible but that should be allowed
          /*}
          else {*/
            // Erase the line graph and switch off the Line CheckBox
            ((Vector)graphTypeLine.elementAt(groupNumberChoice.getSelectedIndex())).
            setElementAt( new Boolean(false), graphNumberChoice.getSelectedIndex() );
          /*}*/
          if (command.equals("none") || command.equals("dot")) {
            // Disable the Symbol Size Choice
            graphSymbolSizeChoice.setEnabled(false);
            // clear the TextField
            typeColorTextField.setText("");
          } else {
            // Enable the Symbol Size Choice and select it's value for the chosen graph
            graphSymbolSizeChoice.setEnabled(true);
            graphSymbolSizeChoice.select( (String)((Vector) graphSymbolSize.
               elementAt(groupNumberChoice.getSelectedIndex())).
                   elementAt(graphNumberChoice.getSelectedIndex()) );
            // Add the Type and Color 'numbers' that correspond to this Group and Graph to the TextFIeld
            typeColorTextField.setText( String.valueOf(gTypeOtherChoice.getSelectedIndex()-2)+"."+String.valueOf(graphColorChoice.getSelectedIndex()) );
            filledSymbol.setState( ( (Boolean) ((Vector) graphSymbolFilled.
                                                    elementAt(groupNumberChoice.getSelectedIndex())).
                                                      elementAt(graphNumberChoice.getSelectedIndex()) ).
                                                        booleanValue() );
          }
          // Assign the type of graph selected
          ((Vector)graphTypeOther.elementAt(groupNumberChoice.getSelectedIndex())).
               setElementAt(command, graphNumberChoice.getSelectedIndex() );
      }
    };
    gTypeOtherChoice.addItemListener(choice_listenerG);
    // Since there are initially no Graphs, disable the 'Set graphType' Options
    gTypeLine.setEnabled(false);
    gTypeOtherChoice.setEnabled(false);

    // Color Settings
    gridConstr.gridx = 0; gridConstr.gridy = 8;
    graphTypesPanel.add(new JLabel("Set Graph Color:"), gridConstr);
    // graphColorChoice
    graphColorChoice.removeAll();
    for(int i=0; i<12; i++)
      graphColorChoice.addItem(COLORS[i]);
    // Set GridBagConstraints for 'Graph Color' Choice
    gridConstr.gridx = 0; gridConstr.gridy = 9;
    graphTypesPanel.add(graphColorChoice, gridConstr);
    // Handle events on the Graph Color Choice
    choice_listenerC = new ItemListenerS() {
      public void itemStateChanged(ItemEvent e) {
        String command = ((Choice)e.getItemSelectable()).getSelectedItem();
        // Assign the type of graph selected
        for(int i=0; i<12; i++)
          if (command.equals(COLORS[i])) {
            ((Vector)graphColor.elementAt(groupNumberChoice.getSelectedIndex())).
                setElementAt(PlotOptionConstants.COLOR[i],
                graphNumberChoice.getSelectedIndex() );
              break;
          }
          // Add the Type and COlor 'numbers' that correspond to this Group and Graph to the TextFIeld
          typeColorTextField.setText( String.valueOf(gTypeOtherChoice.getSelectedIndex()-2)+"."+String.valueOf(graphColorChoice.getSelectedIndex()) );
      }
    };
    graphColorChoice.addItemListener(choice_listenerC);
    // Since there are initially no Graphs, disable the 'graph Color' Choice
    graphColorChoice.setEnabled(false);

    // Filled/Open Symbol Checkbox setup
    gridConstr.gridx = 0; gridConstr.gridy = 10;
    graphTypesPanel.add(filledSymbol, gridConstr);
    // Handle events on the GraphSymbolFilled checkbox
    // Assign the 'Filled' boolean value to the appropriate GraphSymbolFilled
    checkbox_listenerF = new ItemListenerS() {
      public void itemStateChanged(ItemEvent e) {
        boolean command = ((Checkbox)e.getItemSelectable()).getState();
        ((Vector)graphSymbolFilled.elementAt(groupNumberChoice.getSelectedIndex())).
	setElementAt( new Boolean(command), graphNumberChoice.getSelectedIndex() );
      }
    };
    filledSymbol.addItemListener(checkbox_listenerF);
    // Since there are initially no Graphs, disable the checkbox
    filledSymbol.setEnabled(false);
        
    gridConstr.gridx = 0; gridConstr.gridy = 11;
    graphTypesPanel.add(new JLabel("point '(int)type.(int)color':"), gridConstr);


                /* Title and Label Manual Setup Options */

		gridConstr.gridx = 1; gridConstr.gridy = 0;
                titleSetupPanel.add(new JLabel("Note: if these fields are left blank, GUI will use column values"),gridConstr);
		gridConstr.gridx = 0; gridConstr.gridy = 1;
                titleSetupPanel.add(new JLabel("Title:"), gridConstr);
		gridConstr.gridx = 1; gridConstr.gridy = 1;
                titleText.setColumns(50);
                titleSetupPanel.add(titleText, gridConstr);
		gridConstr.gridx = 0; gridConstr.gridy = 2;
                titleSetupPanel.add(new JLabel("x-axis Label:"), gridConstr);
		gridConstr.gridx = 1; gridConstr.gridy = 2;
                xLabelText.setColumns(50);
                titleSetupPanel.add(xLabelText, gridConstr);
		gridConstr.gridx = 0; gridConstr.gridy = 3;
                titleSetupPanel.add(new JLabel("y-axis Label:"), gridConstr);
		gridConstr.gridx = 1; gridConstr.gridy = 3;
                yLabelText.setColumns(50);
                titleSetupPanel.add(yLabelText, gridConstr);

		/* Min and Max Setup Options */

		// Set GridBagConstraints
		gridConstr.gridx = 0; gridConstr.gridy = 0;
    minMaxPanel.add(new JLabel("Min & Max Setup"), gridConstr);
    // Set the color of the textfields
    //xMinText.setBackground(Color.lightGray);
    //xMaxText.setBackground(Color.lightGray);
    //yMinText.setBackground(Color.lightGray);
    //yMaxText.setBackground(Color.lightGray);
		// Set xMin and xMax Options
		gridConstr.gridx = 0; gridConstr.gridy = 1;
    minMaxPanel.add(new JLabel("X-axis:"), gridConstr);
		gridConstr.gridx = 0; gridConstr.gridy = 2;
    minMaxPanel.add(new JLabel("Minimum: "), gridConstr);
		gridConstr.gridx = 1; gridConstr.gridy = 2;
		xMinText.setColumns(10);
    minMaxPanel.add(xMinText, gridConstr);
		gridConstr.gridx = 0; gridConstr.gridy = 3;
    minMaxPanel.add(new JLabel("Maximum: "), gridConstr);
		gridConstr.gridx = 1; gridConstr.gridy = 3;
    minMaxPanel.add(xMaxText, gridConstr);
		// If Manual, set yMin and yMax Options
		gridConstr.gridx = 0; gridConstr.gridy = 4;
    minMaxPanel.add(new JLabel("Y-axis:"), gridConstr);
		gridConstr.gridx = 0; gridConstr.gridy = 5;
    minMaxPanel.add(new JLabel("Minimum: "), gridConstr);
		gridConstr.gridx = 1; gridConstr.gridy = 5;
    minMaxPanel.add(yMinText, gridConstr);
		gridConstr.gridx = 0; gridConstr.gridy = 6;
    minMaxPanel.add(new JLabel("Maximum: "), gridConstr);
		gridConstr.gridx = 1; gridConstr.gridy = 6;
    minMaxPanel.add(yMaxText, gridConstr);

      /* Plot all plot Lines Setup */

       // Handle events on the Graph Color Choice
    checkboxMenu_listenerLine = new ItemListenerS() {
      public void itemStateChanged(ItemEvent e) {
        String command = ((CheckboxMenuItem)e.getItemSelectable()).getLabel();
          if (command.equals("Connect Points")) {
                                                  if (allPointsLineMenuItem.getState()) {
                                                    // Convert all the line checkboxes for all the graphs to TRUE
                                                    for (int i=0; i<graphTypeLine.size(); i++)
                                                      for (int j=0; j<((Vector)graphTypeLine.get(i)).size(); j++)
                                                      ((Vector)graphTypeLine.elementAt(i)).setElementAt( new Boolean(true), j );
                                                      gTypeLine.setState(true);
                                                      //((Checkbox)graphTypesPanel.getComponentAt(0,3)).setState(true);
                                                  } else {
                                                    // Convert all the line checkboxes for all the graphs to FALSE
                                                    for (int i=0; i<graphTypeLine.size(); i++)
                                                      for (int j=0; j<((Vector)graphTypeLine.get(i)).size(); j++)
                                                      ((Vector)graphTypeLine.elementAt(i)).setElementAt( new Boolean(false), j );
                                                      gTypeLine.setState(false);
                                                      //((Checkbox)graphTypesPanel.getComponentAt(0,3)).setState(false);
                                                  }
          }
          else if (command.equals("Fill Plot Symbols")) {
                                                  if (symbolsFilledMenuItem.getState()) {
                                                    // Convert all the checkboxes for all the graphs to TRUE
                                                    for (int i=0; i<graphSymbolFilled.size(); i++)
                                                      for (int j=0; j<((Vector)graphSymbolFilled.get(i)).size(); j++)
                                                      ((Vector)graphSymbolFilled.elementAt(i)).setElementAt( new Boolean(true), j );
                                                      filledSymbol.setState(true);
                                                  } else {
                                                    // Convert all the line checkboxes for all the graphs to FALSE
                                                    for (int i=0; i<graphSymbolFilled.size(); i++)
                                                      for (int j=0; j<((Vector)graphSymbolFilled.get(i)).size(); j++)
                                                      ((Vector)graphSymbolFilled.elementAt(i)).setElementAt( new Boolean(false), j );
                                                      filledSymbol.setState(false);
                                                  }
          }
          // This displays the plot in the GUI once the action is over
          // Note that this action will happen in certain cases where checkboxMenuItems are only selected without any other action being performed
          parentGUI.display();
      }
    };
    strictLimitsMenuItem.addItemListener(checkboxMenu_listenerLine);
    monotonicMenuItem.addItemListener(checkboxMenu_listenerLine);
    allPointsLineMenuItem.addItemListener(checkboxMenu_listenerLine);
    symbolsFilledMenuItem.addItemListener(checkboxMenu_listenerLine);
    adjustToFitMenuItem.addItemListener(checkboxMenu_listenerLine);

		/* Color Options */
		// Add the "Color Options" Menu to "More Options..."
		// Fill the "Color Options" Menu
		colorMenu.add(backgroundMenu);
		colorMenu.add(borderlineMenu);
		for(int i=0; i<12; i++) {
				// Fill Background Color Menu
				backgroundMenu.add(COLORS[i]);
				// Fill Borderline Color Menu
				borderlineMenu.add(COLORS[i]);
		}

		// Fill the "Font Options" Menu
		labelFontMenu.add(labelTypeMenu);
		labelFontMenu.add(labelStyleMenu);
		labelFontMenu.add(labelSizeMenu);
		titleFontMenu.add(titleTypeMenu);
		titleFontMenu.add(titleStyleMenu);
		titleFontMenu.add(titleSizeMenu);
		titleFontMenu.add(adjustToFitMenuItem);
		numbersFontMenu.add(numbersTypeMenu);
		numbersFontMenu.add(numbersStyleMenu);
		numbersFontMenu.add(numbersSizeMenu);
		for(int i=0; i<5; i++) {
				// Fill Label Font Type Menu
				labelTypeMenu.add(FONT_TYPES[i]);
				// Fill Title Font Type Menu
				titleTypeMenu.add(FONT_TYPES[i]);
				// Fill Numbers Font Type Menu
				numbersTypeMenu.add(FONT_TYPES[i]);
		}
		for(int i=0; i<3; i++) {
				// Fill Label Font Style Menu
				labelStyleMenu.add(FONT_STYLES[i]);
				// Fill Title Font Style Menu
				titleStyleMenu.add(FONT_STYLES[i]);
				// Fill Numbers Font Style Menu
				numbersStyleMenu.add(FONT_STYLES[i]);
		}
		for(int i=6; i<=35; i++) {
				// Fill Label Font Size Menu
				labelSizeMenu.add(Integer.toString(i));
				// Fill Title Font Size Menu
				titleSizeMenu.add(Integer.toString(i));
				// Fill Numbers Font Size Menu
				numbersSizeMenu.add(Integer.toString(i));
		}

		/* Plot Setup Options */

		// Fill the "Plot Setup Options" Menu
		setupMenuP.add("Top Distance ("+Integer.toString(upDistance)+")");
		setupMenuP.add("Bottom Distance ("+Integer.toString(downDistance)+")");
		setupMenuP.add("Left Distance ("+Integer.toString(leftDistance)+")");
		setupMenuP.add("Right Distance ("+Integer.toString(rightDistance)+")");

		/* Graph Setup Options */
		// Fill the "Graph Setup Options" Menu
		setupMenuG.add(errorBarsMenuItem);
		setupMenuG.add(errorBarTicsMenuItem);
                setupMenuG.add(dotFiducialsMenuItem);

                /* Zero Axis Setup Options */
                setupMenuA.add(xZeroAxisMenuItem);
                setupMenuA.add(yZeroAxisMenuItem);

		//
		/* Event Handlers */
		//

                /* Handle a Frame event */
                frameListener = new KeyListenerF() {
                  public void keyReleased(KeyEvent e) {
                    // An event occured within the Frame
                    int command = e.getKeyCode();
//System.out.println(command);
                  }
                };
                // Add the ActionListener to the Frame
                this.addKeyListener(frameListener);
		okay.addKeyListener(frameListener);

		/* Handle all "More Option" (Menu) events */
		moreOptionsListener = new ActionListenerS() {
				public void actionPerformed(ActionEvent e) {

                                // Label of the menu (inside which menuItem was pressed)
                                String command1 = new String( ((Menu)e.getSource()).getLabel() );
                                // The ActionCommand of the menuitem pressed in the menu
                                String command2 = new String( e.getActionCommand() );


                                              /* Handle events on the "Color Options" Menu */

                                              // Handle event for Background Color
                                              if ( command1.equals("Background Color ("+backgroundColorLabel.getText()+")") ) {
                                                  for(int i=0; i<12; i++)
                                                    if (e.getActionCommand().equals(COLORS[i])) {
                                                      PlotOptionsFrame.this.backgroundColor=PlotOptionConstants.COLOR[i];
                                                      PlotOptionsFrame.this.backgroundColorLabel.setText(COLORS[i]);
                                                      break;
                                                    }
                                                    // If Background is Black, Set the Borderline to White
                                                    if (e.getActionCommand().equals("Black")) {
                                                      PlotOptionsFrame.this.borderlineColor	=	Color.white;
                                                        PlotOptionsFrame.this.borderlineColorLabel.setText("White");
                                                    }
                                                    // Finally, rename the Menubox Item
                                                    backgroundMenu.setLabel("Background Color ("+backgroundColorLabel.getText()+")");
						}

                                              // Handle event for Borderline Color
                                              else if ( command1.equals("Borderline Color ("+borderlineColorLabel.getText()+")") ) {
                                                  for(int i=0; i<12; i++)
                                                    if (e.getActionCommand().equals(COLORS[i])) {
                                                      PlotOptionsFrame.this.borderlineColor=PlotOptionConstants.COLOR[i];
                                                      PlotOptionsFrame.this.borderlineColorLabel.setText(COLORS[i]);
                                                      break;
                                                    }
                                                    // Finally, rename the Menubox Item
                                                    borderlineMenu.setLabel("Borderline Color ("+borderlineColorLabel.getText()+")");
						}

                                                /*Address changes to the font settings.*/
                                                //check which menu has been called
                                                String fontMenuText = ((MenuItem)(((MenuItem)e.getSource()).getParent())).getLabel();
                                                //System.out.println("fontMenuText:\t" + fontMenuText);
                                                if(fontMenuText.equals("Label Font"))
                                                {
                                                    if(command1.indexOf("Type")>=0)          //change font type
                                                    {
                                                        for(int i=0; i<5; i++)
                                                            if (e.getActionCommand().equals(FONT_TYPES[i])) 
                                                            {
                                                                PlotOptionsFrame.this.labelFontType=PlotOptionConstants.FONT_TYPE[i];
                                                                PlotOptionsFrame.this.labelTypeLabel.setText(PlotOptionsFrame.this.labelFontType);
                                                                break;
                                                            }
                                                            // Finally, Rename the menubox Item
                                                            labelTypeMenu.setLabel("Type ("+labelTypeLabel.getText()+")");                                                        
                                                    }
                                                    else if(command1.indexOf("Style")>=0)    //change font style
                                                    {
                                                        for(int i=0; i<3; i++)
                                                            if (e.getActionCommand().equals(FONT_STYLES[i])) 
                                                            {
                                                                PlotOptionsFrame.this.labelFontStyle=PlotOptionConstants.FONT_STYLE[i];
                                                                PlotOptionsFrame.this.labelStyleLabel.setText(FONT_STYLES[i]);
                                                                break;
                                                            }
                                                        // Finally, rename the menubox item
                                                        labelStyleMenu.setLabel("Style ("+labelStyleLabel.getText()+")");                                                        
                                                    }
                                                    else if(command1.indexOf("Size")>=0)     //change font size
                                                    {
                                                        PlotOptionsFrame.this.labelFontSize = Integer.parseInt(e.getActionCommand());
                                                        PlotOptionsFrame.this.labelSizeLabel.setText(e.getActionCommand());
                                                        // Finally, rename the menubox item
                                                        labelSizeMenu.setLabel("Size ("+labelSizeLabel.getText()+")");
                                                    }
                                                }
                                                else if(fontMenuText.equals("Title Font"))
                                                {
                                                    if(command1.indexOf("Type")>=0)          //change font type
                                                    {
                                                        for(int i=0; i<5; i++)
                                                            if (e.getActionCommand().equals(FONT_TYPES[i])) 
                                                            {
                                                                PlotOptionsFrame.this.titleFontType=PlotOptionConstants.FONT_TYPE[i];
                                                                PlotOptionsFrame.this.titleTypeLabel.setText(PlotOptionsFrame.this.titleFontType);
                                                                break;
                                                            }
                                                        // Finally, rename the menubox item
                                                        titleTypeMenu.setLabel("Type ("+titleTypeLabel.getText()+")");
                                                    }
                                                    else if(command1.indexOf("Style")>=0)    //change font style
                                                    {
                                                        for(int i=0; i<3; i++)
                                                            if (e.getActionCommand().equals(FONT_STYLES[i])) 
                                                            {
                                                                PlotOptionsFrame.this.titleFontStyle=PlotOptionConstants.FONT_STYLE[i];
                                                                PlotOptionsFrame.this.titleStyleLabel.setText(FONT_STYLES[i]);
                                                                break;
                                                            }
                                                        // Finally, rename the menubox item
                                                        titleStyleMenu.setLabel("Style ("+titleStyleLabel.getText()+")");
                                                    }
                                                    else if(command1.indexOf("Size")>=0)     //change font size
                                                    {
                                                        PlotOptionsFrame.this.titleFontSize = Integer.parseInt(e.getActionCommand());
                                                        PlotOptionsFrame.this.titleSizeLabel.setText(e.getActionCommand());
                                                        // Finally, rename the menubox item
                                                        titleSizeMenu.setLabel("Size ("+titleSizeLabel.getText()+")");
                                                        // Disable the plot to fit menu Choice
                                                        adjustToFitMenuItem.setState(false);                                                        
                                                    }
                                                    else if(e.getActionCommand().equals("Adjust to fit"))   //adjust title font size as necessary
                                                    {
                                                        adjustToFitMenuItem.setState(true);
                                                    }
                                                }
                                                else if(fontMenuText.equals("Numbers Font"))
                                                {
                                                    if(command1.indexOf("Type")>=0)          //change font type
                                                    {
                                                        for(int i=0; i<5; i++)
                                                            if (e.getActionCommand().equals(FONT_TYPES[i])) 
                                                            {
                                                                PlotOptionsFrame.this.numbersFontType = PlotOptionConstants.FONT_TYPE[i];
                                                                PlotOptionsFrame.this.numbersTypeLabel.setText(PlotOptionsFrame.this.numbersFontType);
                                                                break;
                                                            }
                                                        // Finally, rename the menubox item
                                                        numbersTypeMenu.setLabel("Type ("+numbersTypeLabel.getText()+")");                                                        
                                                    }
                                                    else if(command1.indexOf("Style")>=0)    //change font style
                                                    {
                                                        for(int i=0; i<3; i++)
                                                            if (e.getActionCommand().equals(FONT_STYLES[i])) 
                                                            {
                                                                PlotOptionsFrame.this.numbersFontStyle = PlotOptionConstants.FONT_STYLE[i];
                                                                PlotOptionsFrame.this.numbersStyleLabel.setText(FONT_STYLES[i]);
                                                                break;
                                                            }
                                                        // Finally, rename the menubox item
                                                        numbersStyleMenu.setLabel("Style ("+numbersStyleLabel.getText()+")");
                                                    }
                                                    else if(command1.indexOf("Size")>=0)     //change font size
                                                    {
                                                        PlotOptionsFrame.this.numbersFontSize = Integer.parseInt(e.getActionCommand());
                                                        PlotOptionsFrame.this.numbersSizeLabel.setText(e.getActionCommand());
                                                        // Finally, rename the menubox item
                                                        numbersSizeMenu.setLabel("Size ("+numbersSizeLabel.getText()+")");                                                        
                                                    }                                                    
                                                }

						/* Handle events on the "Plot Setup Options" Menu */

						// Handle event for Up Distance
						if ( command2.equals("Top Distance ("+Integer.toString(upDistance)+")") ) {
								// Create a Dialog
								final Dialog dialog = new Dialog(PlotOptionsFrame.this, "Top Distance:", true);
								// Add the TextField with default value in it
								text.setText(Integer.toString(upDistance));
								dialog.add(text, "Center");
								// Add the OK and Cancel buttons
								dialog.add(new JLabel(dialog.getTitle()), "North");
								Panel box = new Panel();
								Button ok = new Button("OK");
								Button cancel = new Button("Cancel");
								box.add(ok);
								box.add(cancel);
								dialog.add(box, "South");
								// Set the size of the Dialog
								dialog.setSize(150, 150);
								ActionListenerS buttonlistener = new ActionListenerS() {
										public void actionPerformed(ActionEvent e) {
												try {
														if (e.getActionCommand().equals("OK")) {
																upDistance = Integer.parseInt(text.getText());
																setupMenuP.remove(0);
																setupMenuP.insert("Top Distance ("+Integer.toString(upDistance)+")", 0);

																dialog.dispose();
														}
														else if (e.getActionCommand().equals("Cancel")) dialog.dispose();
												}
												catch (Exception ex) {
														System.err.println("Top Distance should be integer value");
												}
										}
								};

								ok.addActionListener(buttonlistener);
								cancel.addActionListener(buttonlistener);
								dialog.show();
						}
						// Handle event for Down Distance
						else if ( command2.equals("Bottom Distance ("+Integer.toString(downDistance)+")") ) {
								// Create a Dialog
								final Dialog dialog = new Dialog(PlotOptionsFrame.this, "Bottom Distance:", true);
								// Add the TextField with default value in it
								text.setText(Integer.toString(downDistance));
								dialog.add(text, "Center");
								// Add the OK and Cancel buttons
								dialog.add(new JLabel(dialog.getTitle()), "North");
								Panel box = new Panel();
								Button ok = new Button("OK");
								Button cancel = new Button("Cancel");
								box.add(ok);
								box.add(cancel);
								dialog.add(box, "South");
								// Set the size of the Dialog
								dialog.setSize(150, 150);
								ActionListenerS buttonlistener = new ActionListenerS() {
										public void actionPerformed(ActionEvent e) {
												try {
														if (e.getActionCommand().equals("OK")) {
																downDistance = Integer.parseInt(text.getText());
																setupMenuP.remove(1);
																setupMenuP.insert("Bottom Distance ("+Integer.toString(downDistance)+")", 1);

																dialog.dispose();
														}
														else if (e.getActionCommand().equals("Cancel")) dialog.dispose();
												}
												catch (Exception ex) {
														System.err.println("Bottom Distance should be integer value");
												}
										}
								};
								ok.addActionListener(buttonlistener);
								cancel.addActionListener(buttonlistener);
								dialog.show();
						}
						// Handle event for Left Distance
						else if ( command2.equals("Left Distance ("+Integer.toString(leftDistance)+")") ) {
								// Create a Dialog
								final Dialog dialog = new Dialog(PlotOptionsFrame.this, "Left Distance:", true);
								// Add the TextField with default value in it
								text.setText(Integer.toString(leftDistance));
								dialog.add(text, "Center");
								// Add the OK and Cancel buttons
								dialog.add(new JLabel(dialog.getTitle()), "North");
								Panel box = new Panel();
								Button ok = new Button("OK");
								Button cancel = new Button("Cancel");
								box.add(ok);
								box.add(cancel);
								dialog.add(box, "South");
								// Set the size of the Dialog
								dialog.setSize(150, 150);
								ActionListenerS buttonlistener = new ActionListenerS() {
										public void actionPerformed(ActionEvent e) {
												try {
														if (e.getActionCommand().equals("OK")) {
																leftDistance = Integer.parseInt(text.getText());
																setupMenuP.remove(2);
																setupMenuP.insert("Left Distance ("+Integer.toString(leftDistance)+")", 2);

																dialog.dispose();
														}
														else if (e.getActionCommand().equals("Cancel")) dialog.dispose();
												}
												catch (Exception ex) {
														System.err.println("Left Distance should be integer value");
												}
										}
								};
								ok.addActionListener(buttonlistener);
								cancel.addActionListener(buttonlistener);
								dialog.show();
						}
						// Handle event for Right Distance
						else if ( command2.equals("Right Distance ("+Integer.toString(rightDistance)+")") ) {
								// Create a Dialog
								final Dialog dialog = new Dialog(PlotOptionsFrame.this, "Right Distance:", true);
								// Add the TextField with default value in it
								text.setText(Integer.toString(rightDistance));
								dialog.add(text, "Center");
								// Add the OK and Cancel buttons
								dialog.add(new JLabel(dialog.getTitle()), "North");
								Panel box = new Panel();
								Button ok = new Button("OK");
								Button cancel = new Button("Cancel");
								box.add(ok);
								box.add(cancel);
								dialog.add(box, "South");
								// Set the size of the Dialog
								dialog.setSize(150, 150);
								ActionListenerS buttonlistener = new ActionListenerS() {
										public void actionPerformed(ActionEvent e) {
												try {
														if (e.getActionCommand().equals("OK")) {
																rightDistance = Integer.parseInt(text.getText());
																setupMenuP.remove(3);
																setupMenuP.insert("Right Distance ("+Integer.toString(rightDistance)+")", 3);

																dialog.dispose();
														}
														else if (e.getActionCommand().equals("Cancel")) dialog.dispose();
												}
												catch (Exception ex) {
														System.err.println("Right Distance should be integer value");
												}
										}
								};
								ok.addActionListener(buttonlistener);
								cancel.addActionListener(buttonlistener);
								dialog.show();
						}
                                // This displays the plot int he GUI once the action in the menubar is over
                                parentGUI.display();
				}
		};
		// Add the ActionListener to colorMenu's Menus
		backgroundMenu.addActionListener(moreOptionsListener);
		borderlineMenu.addActionListener(moreOptionsListener);
		// Add the ActionListener to fontMenu's Menus
		labelTypeMenu.addActionListener(moreOptionsListener);
		labelStyleMenu.addActionListener(moreOptionsListener);
		labelSizeMenu.addActionListener(moreOptionsListener);
		titleTypeMenu.addActionListener(moreOptionsListener);
		titleStyleMenu.addActionListener(moreOptionsListener);
		titleSizeMenu.addActionListener(moreOptionsListener);
		numbersTypeMenu.addActionListener(moreOptionsListener);
		numbersStyleMenu.addActionListener(moreOptionsListener);
		numbersSizeMenu.addActionListener(moreOptionsListener);
		// Add the ActionListener to Plot setupMenuP checkobx
		setupMenuP.addActionListener(moreOptionsListener);
		// Add the ActionListener to Plot setupMenuG checkbox
		setupMenuG.addActionListener(moreOptionsListener);
                // Add the ActionListener to the vertical and horizontal axis setup checkbox
                setupMenuA.addActionListener(moreOptionsListener);

	}

      /*  Methods */

      /**
      * This method sets variables which need to be initialized when new graph is added to plot2D
       * The size of the symbol matters so it is passed as a parameter
       *  int symbSize: is the size of the symbols in the graph, the Calculation for it's size is done before.
      */
      public void AddXOptions() {

        // add new vectors (i.e. X Axis fields)
        graphTypeLine.addElement(new Vector());
        graphSymbolSize.addElement(new Vector());
        graphSymbolFilled.addElement(new Vector());
        graphTypeOther.addElement(new Vector());
        graphColor.addElement(new Vector());

        updateGraphTypesWindow();
      }

      public void AddYOptions(int selectedxAxisIndex, int symbSize) {
        // add default values to these new vectors (i.e. Y Axis fields)
        ((Vector)graphTypeLine.elementAt(selectedxAxisIndex)).addElement(new Boolean(allPointsLineMenuItem.getState())); // it is set to the global default
        ((Vector)graphSymbolSize.elementAt(selectedxAxisIndex)).addElement(Integer.toString(symbSize));
        ((Vector)graphSymbolFilled.elementAt(selectedxAxisIndex)).addElement(new Boolean(true));
        // Find total number of the Graph for the purpose of choosing a new, different graph symbol and color

        int numberOfGraphs=0;
        for(int k=0; k<((DataTable) parentGUI.xAxisDataTable.elementAt(parentGUI.layoutChoice.getSelectedIndex())).columnCount(); k++)
          numberOfGraphs+=((DataTable)((Vector) parentGUI.yAxisDataTable.elementAt(parentGUI.layoutChoice.getSelectedIndex())).elementAt(k)).columnCount();
        // Set the new Symbol
        // note: subtract 2 and add a 2 to avoid using "none" and "dot" symbols which
        // are 0 and 1 in the gTypeOtherChoice
        int orderHelper = ((numberOfGraphs-1)%(gTypeOtherChoice.getItemCount()-2))+2;
        ((Vector)graphTypeOther.elementAt(selectedxAxisIndex)).
            addElement(gTypeOtherChoice.getItem(orderHelper));
        // Set the new Colors
        // Use colors in sequence, avoid using the color of the background
        int back=0;
        for (int l=0; l<12; l++)
          if (backgroundColor==PlotOptionConstants.COLOR[l]) back=l;
        // 11 is color of black (that's default graph and labels color) so don't use it for graph color
        // Note: this will rarely be used since the background will probably never be set to black
        if (back==11 && numberOfGraphs==12) {
          ((Vector)graphColor.elementAt(selectedxAxisIndex)).
                   addElement(PlotOptionConstants.COLOR[0]); // any other color (eg.0:blue)
        }
        // this will be the color of the background (default bacground is actually white: 9)
        else if (back==numberOfGraphs) {
          ((Vector)graphColor.elementAt(selectedxAxisIndex)).
                addElement(PlotOptionConstants.COLOR[0]); // any other color (eg.0:blue)
        }
        else {
          ((Vector)graphColor.elementAt(selectedxAxisIndex)).   // There are 12 colors
                  addElement(PlotOptionConstants.COLOR[(numberOfGraphs-1)%12]);
        }

        updateGraphTypesWindow();
      }

      /**
       *
       * Note: This Method will also remove the Y Options that are within X options
       */
      public void removeXOptions(int selectedxAxisIndex) {
        // update the fields in PlotOptionsFrame
        graphTypeLine.removeElementAt(selectedxAxisIndex);
        graphTypeOther.removeElementAt(selectedxAxisIndex);
        graphSymbolSize.removeElementAt(selectedxAxisIndex);
        graphColor.removeElementAt(selectedxAxisIndex);
        graphSymbolFilled.removeElementAt(selectedxAxisIndex);

        updateGraphTypesWindow();
      }

      public void removeYOptions(int selectedxAxisIndex, int selectedyAxisIndex) {
        ((Vector)graphTypeLine.elementAt(selectedxAxisIndex)).removeElementAt(selectedyAxisIndex);
        ((Vector)graphTypeOther.elementAt(selectedxAxisIndex)).removeElementAt(selectedyAxisIndex);
        ((Vector)graphSymbolSize.elementAt(selectedxAxisIndex)).removeElementAt(selectedyAxisIndex);
        ((Vector)graphColor.elementAt(selectedxAxisIndex)).removeElementAt(selectedyAxisIndex);
        ((Vector)graphSymbolFilled.elementAt(selectedxAxisIndex)). removeElementAt(selectedyAxisIndex);

        updateGraphTypesWindow();
      }

      /* Update the fileds in the 'Graph Types' Window */
      public void updateGraphTypesWindow() {

        // Add items into 'Group Number' Choice
        groupNumberChoice.removeAll();
        for (int i=0; i<((DataTable) parentGUI.xAxisDataTable.elementAt(parentGUI.layoutChoice.getSelectedIndex())).columnCount(); i++)
          groupNumberChoice.addItem("X Axis #" + Integer.toString(i+1)+"  ("+
                ((DataColumn) ((DataTable)parentGUI.xAxisDataTable.elementAt(parentGUI.layoutChoice.getSelectedIndex())).getColumn(i)).getName()+ ")" );

        // Add Choices into 'Graph Number' Choice if there are any elements in the group
        graphNumberChoice.removeAll();
        if (((DataTable) parentGUI.xAxisDataTable.elementAt(parentGUI.layoutChoice.getSelectedIndex())).columnCount()!=0)
          // note: we have elementAt(0), because we are displaying only the first column when updating
          for (int j=0; j<((DataTable)((Vector) parentGUI.yAxisDataTable.elementAt(parentGUI.layoutChoice.getSelectedIndex())).elementAt(0)).columnCount(); j++)
            graphNumberChoice.addItem("Graph #" + Integer.toString(j+1)+"  ("+((DataColumn) ((DataTable)
                    ((Vector) parentGUI.yAxisDataTable.elementAt(parentGUI.layoutChoice.getSelectedIndex())).elementAt(0)).getColumn(j)).getName()+ ")"  );
        // Add the Type and COlor 'numbers' that correspond to this Group and Graph to the TextFIeld
        typeColorTextField.setText( String.valueOf(gTypeOtherChoice.getSelectedIndex()-2)+"."+String.valueOf(graphColorChoice.getSelectedIndex()) );
        // Set GridBagConstraints for 'Group Number' Choice
        gridConstr.gridx = 0; gridConstr.gridy = 0;
        // Don't show the 'Group Number' Choice if there are no graphs in the plot
        if (((DataTable) parentGUI.xAxisDataTable.elementAt(parentGUI.layoutChoice.getSelectedIndex())).columnCount()!=0) graphTypesPanel.add(groupNumberChoice, gridConstr);
        // Set GridBagConstraints for 'Graph Number' Choice
        gridConstr.gridx = 0; gridConstr.gridy = 1;
        // Don't show the 'Graph Number' Choice if there are no groups
        if (graphNumberChoice.getItemCount()!=0) {
          graphTypesPanel.add(graphNumberChoice, gridConstr);
          // Set 'Line' and 'Other' Options
          gTypeLine.setState( ( (Boolean) ((Vector) graphTypeLine.elementAt(0)).elementAt(0) ).booleanValue() );
          gTypeOtherChoice.select( (String)((Vector) graphTypeOther.elementAt(0)).elementAt(0) );
          graphSymbolSizeChoice.select( (String)((Vector) graphSymbolSize.elementAt(0)).elementAt(0) );
          for(int i=0; i<12; i++) {
            if ( (Color)((Vector) graphColor.elementAt(0)).elementAt(0) == PlotOptionConstants.COLOR[i] ) {
              graphColorChoice.select(i);
              break;
            }
          }
          filledSymbol.setState( ( (Boolean) ((Vector) graphSymbolFilled.elementAt(0)).elementAt(0) ).booleanValue() );
        }


        // If there are no Graphs, disable the 'Set graphType' and 'Graph Color' Options
        if (((DataTable) parentGUI.xAxisDataTable.elementAt(parentGUI.layoutChoice.getSelectedIndex())).columnCount()==0 ||
            ((Vector) parentGUI.yAxisDataTable.elementAt(parentGUI.layoutChoice.getSelectedIndex())).size()==0) {
          gTypeLine.setEnabled(false);
          gTypeOtherChoice.setEnabled(false);
          graphSymbolSizeChoice.setEnabled(false);
          graphColorChoice.setEnabled(false);
          filledSymbol.setEnabled(false);
        }
        else {
          gTypeLine.setEnabled(true);
          gTypeOtherChoice.setEnabled(true);
          // If the selected (first) graph doesn't have some symbol for it's points set 'Size' Choice to false
          if (gTypeOtherChoice.getSelectedItem().equals("none") || gTypeOtherChoice.getSelectedItem().equals("dot")) {
            filledSymbol.setEnabled(false);
            graphSymbolSizeChoice.setEnabled(false);
          } else {
            filledSymbol.setEnabled(true);
            graphSymbolSizeChoice.setEnabled(true);
          }
          graphColorChoice.setEnabled(true);
        }
      }


              /** This is the action listener method which is just overloaded here */
              public void actionPerformed(ActionEvent e) {}

              // These are the methods of the WindowListener object.  Only
              // windowClosing() is implemented
              public void windowClosing(WindowEvent e) { PlotOptionsFrame.this.setVisible(false); }
              public void windowOpened(WindowEvent e) {}
              public void windowClosed(WindowEvent e) {}
              public void windowIconified(WindowEvent e) {}
              public void windowDeiconified(WindowEvent e) {}
              public void windowActivated(WindowEvent e) {}
              public void windowDeactivated(WindowEvent e) {}

              public Object clone() throws CloneNotSupportedException {
                  PlotOptionsFrame returnValue = (PlotOptionsFrame)super.clone();
                  /*returnValue.values = (Vector)values.clone();
                                //note: we don't need to clone the Objects (which are converted doubles) within the Vector!
				String cloneFrom[] = new String[3];
				String cloneTo[] = new String[3];
                                cloneFrom[0] = super.label;
				cloneFrom[1] = super.name;
				cloneTo = (String[])cloneFrom.clone();
                                returnValue.label = cloneTo[0];
				returnValue.name = cloneTo[1];*/
				return returnValue;
		}

  public int getGraphNumberChoiceIndex() {return graphNumberChoice.getSelectedIndex();}
  public void setGTypeLine(boolean bool) {gTypeLine.setEnabled(bool);}
  public void setParentGUI(musr.client.gui.GUI parent) {parentGUI = parent;}

  public static void main(String[] args) {
    PlotOptionsFrame f = new PlotOptionsFrame();
    f.show();
  }

}
