//: MuSR_Histogram.java

package musr.spect;

public class MuSR_Histogram implements java.io.Serializable {

  public int histNumber;    // Histogram number [instance number]
  public int histType;      // Histogram type [should be 0x02010002 hex]
  public int nBytes;        // number of bytes in this (packed?) Histogram
  public int nBins;         // number of bins in this Histogram
  public int bytesPerBin;   // bytes per bin [0 for "packed" data]
  public double nsPerBin;      // nanoseconds per bin
  public double t0_ns;         // left edge of bin "1" to t=0 [nanosec]
  public int t0_bin;        // bin number where t=0
  public int goodBin1;      // first bin of good t>0 data
  public int goodBin2;      // last bin of good t>0 data
  public int bkgd1;         // first bin of t<0 Background data
  public int bkgd2;         // last bin of t<0 Background data
  public int nEvents;       // total number of events in Histogram
  public String title;      // Histogram name [like "Left" or "Right"]

  public int bin[]; // Array of integer bins.  NOTE: bin[0] UNUSED!
  // Data starts in bin[1].  Therefore these Histograms are 1 bin
  // longer than those in the data files.

  /* histogram header copier getHdr method */
  public MuSR_HistHdr getHdr () {
    MuSR_HistHdr hdr = new MuSR_HistHdr();
    hdr.histNumber = histNumber;
    hdr.histType = histType;
    hdr.nBytes = nBytes;
    hdr.nBins = nBins;
    hdr.bytesPerBin = bytesPerBin;
    hdr.nsPerBin = nsPerBin;
    hdr.t0_ns = t0_ns;
    hdr.t0_bin = t0_bin;
    hdr.goodBin1 = goodBin1;
    hdr.goodBin2 = goodBin2;
    hdr.bkgd1 = bkgd1;
    hdr.bkgd2 = bkgd2;
    hdr.nEvents = nEvents;
    hdr.title = title;
    return hdr;
  }

  /* histogram header prettyPrint method */
  public void prettyPrint () {
    System.out.print("\nHistogram #" +histNumber);
    System.out.print(" \"" +title +"\"");
    System.out.print("\t" +nEvents +" evts");
    System.out.print(" in " +nBins +" bins\n");
    if ( nsPerBin != 0.0 ) {  // Otherwise must be MuSR_TI histogram.
      System.out.print(nsPerBin +" ns/bin; ");
      System.out.print("t=0 in bin " +t0_bin+" ");
      System.out.print("[" +t0_ns +" ns past left edge of bin 1]\n");
      System.out.print("t<0 background in bins " +bkgd1);
      System.out.print(" to " +bkgd2+"; ");
      System.out.print("good data in bins " +goodBin1);
      System.out.print(" to " +goodBin2 +"\n");
    }
  }

  /* histogram header printHdr method */
  public void printHdr () {
    Format.print(System.out, "\ntype 0x%08x", histType);
    System.out.print(" Histogram #"+histNumber);
    System.out.print(":\t title \""+ title+"\"");
    System.out.print("\t nEvents= "+nEvents);
    System.out.print("\nnBins= "+nBins);
    System.out.print(", nsPerBin= "+nsPerBin);
    System.out.print("\t(t0_ns= "+t0_ns);
    System.out.print("; t0_bin= "+t0_bin+")");
    System.out.print("\n(goodBin1= "+goodBin1);
    System.out.print(", goodBin2= "+goodBin2+")");
    System.out.print("\t(bkgd1= "+bkgd1);
    System.out.print(", bkgd2= "+bkgd2+")\n");
  }

  /* histogram header flexdbHdr (print in flexdb format) method */
  public void flexdbHdr () {
    System.out.print("H"+histNumber+"title="+title+"\n");
    System.out.print("H"+histNumber+"evts="+nEvents+"\n");
    System.out.print("H"+histNumber+"nBins="+nBins+"\n");
    System.out.print("H"+histNumber+"ns_bin="+nsPerBin+"\n");
    System.out.print("H"+histNumber+"t0ns="+t0_ns+"\n");
    System.out.print("H"+histNumber+"t0bin="+t0_bin+"\n");
    System.out.print("H"+histNumber+"good1="+goodBin1+"\n");
    System.out.print("H"+histNumber+"good2="+goodBin2+"\n");
    System.out.print("H"+histNumber+"bkgd1="+bkgd1+"\n");
    System.out.print("H"+histNumber+"bkgd2="+bkgd2+"\n");
  }

  /* histogram header csvHdr (print in CSV format) method */
  public void csvHdr () {
    System.out.print("\"[HIST]\",\""+histNumber+"\",");
    System.out.print("\""+title+"\",");
    System.out.print("\""+nEvents+"\",");
    System.out.print("\""+nBins+"\",");
    System.out.print("\""+nsPerBin+"\",");
    System.out.print("\""+t0_ns+"\",");
    System.out.print("\""+t0_bin+"\",");
    System.out.print("\""+goodBin1+"\",");
    System.out.print("\""+goodBin2+"\",");
    System.out.print("\""+bkgd1+"\",");
    System.out.print("\""+bkgd2+"\"\n");
  }

  /* histogram header sqlHdr (print in SQL format) method */
/*  public void sqlHdr () {
    System.out.print("\"[HIST]\",\""+histNumber+"\",");
    System.out.print("\""+title+"\",");
    System.out.print("\""+nEvents+"\",");
    System.out.print("\""+nBins+"\",");
    System.out.print("\""+nsPerBin+"\",");
    System.out.print("\""+t0_ns+"\",");
    System.out.print("\""+t0_bin+"\",");
    System.out.print("\""+goodBin1+"\",");
    System.out.print("\""+goodBin2+"\",");
    System.out.print("\""+bkgd1+"\",");
    System.out.print("\""+bkgd2+"\"\n");
  }
*/

  /* histogram printBins method */
  public void printBins ( int jj1, int jj2 ) {
    if (jj1<0) jj1 = goodBin1 - 1;
    if (jj2<0) jj2 = goodBin2 - 1;
    if (jj1<0) jj1 = 0;
    if (jj1>=bin.length) jj1 = bin.length - 1;
    if (jj2>=bin.length) jj2 = bin.length - 1;
    if (jj2<jj1) jj2 = jj1;
    System.out.print("\n Histogram "+histNumber+", bins ");
    System.out.print((jj1+1)+" to "+(jj2+1)+":");
    if ((jj1%10) != 1) Format.print(System.out, "\n % 6d: ", jj1);
    for (int i=jj1; (i<=jj2); i++) {
      if ((i%10) == 1) {
	Format.print(System.out, "\n % 6d: ", (i+1));
      }
      Format.print(System.out, "% 7d", bin[i]);
    }
    System.out.print("\n");
  }

  /* histogram data unPack method */
  public void unPack ( int loc, byte bb[] )
    throws MuSR_PackingException {

    int nbn, bpb, i, j = 0, k = loc;

    if (bytesPerBin == 0) {  // Compressed format (normal for TD hists)
      i = Byte2Int(k, bb); k = k + 4;
      if (i != nBytes) {
          //System.out.println("YES JACOB YOU HAVE FOUND THE ERROR. YOU ARE BRILLIANT");
	//System.out.print(i +" bytes doesn't match " +nBytes +".  Exit!\n");
	//System.exit(3);
      }
      //     System.out.print("nBytes:[" +i +"]\n");
      while (k<(loc+nBytes) && j<nBins) {
	nbn = (int)(Byte2Short(k, bb)); k+=2;
	if (nbn<0) nbn = 65536+nbn; // Watch those oferflows!
	bpb = (int)(bb[k]); k++;
	if (bpb<0) bpb = 256+bpb;
	if (nbn > (nBins-j)) nbn = nBins - j;
	//	System.out.print("Next " +nbn +" bins are ");
	//	if (bpb == 0) System.out.print("zero.\n");
	//	else System.out.print(bpb +" bytes per bin.\n");
	if (bpb == 0) {
	  for (i=0; i<nbn; i++) {
	    bin[j] = 0; j++;
	  }
	}
	else if (bpb == 1) {
	  for (i=0; i<nbn; i++) {
	    bin[j] = (int)(bb[k]); if (bin[j]<0) bin[j]=256+bin[j];
	    j++; k++;
	  }
	}
	else if (bpb == 2) {
	  for (i=0; i<nbn; i++) {
	    bin[j] = (int)(Byte2Short(k, bb));
	    j++; k+=2;
	  }
	}
	else if (bpb == 4) {
	  for (i=0; i<nbn; i++) {
	    bin[j] = Byte2Int(k, bb);
	    j++; k+=4;
	  }
	}
	else {
	  System.out.print("*** " +bpb +" bytes per bin??!!\n");
	  //System.exit(4);
          //System.out.println("MuSR_Histogram:193");
	}
      }
    }
    else if (bytesPerBin == 1) {  // Fixed byte-sized bins.
      for (i=0; i<nBins; i++) {
	bin[i] = (int)(bb[k]);
	k++;
      }
    }
    else if (bytesPerBin == 2) {  // Fixed 2-byte bins.
      for (i=0; i<nBins; i++) {
	bin[i] = (int)(Byte2Short(k, bb));
	k+=2;
      }
    }
    else if (bytesPerBin == 4) {  // Fixed 4-byte bins.
      for (i=0; i<nBins; i++) {
	bin[i] = Byte2Int(k, bb);
	k+=4;
      }
    }
    else {
      /*System.out.print("*** " +bytesPerBin +" bytesPerBin ?? !!\n");
      System.exit(4);*/
        //System.out.println("MuSR_Histogram:217");
    }

    //    System.out.print("\n");
  }

  short Byte2Short ( int loc, byte b[] ) {
    short m1, m2;
    m1 = (short)(b[loc]); if (m1<0) m1=(short)(256+m1);
    m2 = (short)(b[loc+1]); if (m2<0) m2=(short)(256+m2);
    m2 = (short)(256*m2 + m1);
    return m2;
  }

  int Byte2Int ( int loc, byte b[] ) {
    int i1, i2, i3, i4;
    i1 = (int)(b[loc]); if (i1<0) i1=(int)(256+i1);
    i2 = (int)(b[loc+1]); if (i2<0) i2=(int)(256+i2);
    i3 = (int)(b[loc+2]); if (i3<0) i3=(int)(256+i3);
    i4 = (int)(b[loc+3]); if (i4<0) i4=(int)(256+i4);
    i4 = (int)(16777216*i4 + 65536*i3 + 256*i2 + i1);
    return i4;
  }

}
