//: MuSR_RunHeader.java

package musr.spect;


import java.util.*;

public class MuSR_RunHeader implements java.io.Serializable {

  public int fmtID;             // should be 0x02010000 [in hex]
  public int exptNumber;        // EEC Experiment Number
  public int runNumber;         // run number
  public int timeBegin;         // times of beginning and end of run
  public int timeEnd;           // in seconds since [sometime in 1970?]
  public int elapsedSec;        // = timeEnd - timeBegin [bad!]
  public int nSclrs;            // number of Scalers
  public int nHists;            // number of Histograms
  public int nIndVar;           // number of Independent Variables
  public String title;          // run title
  public String lab;            // TRIUMF
  public String area;           // like M9, M13, M15 or M20
  public String method;         // should be TD-?SR or TI-?SR
  public String apparatus;      // usually the RIG name
  public String insert;         // usually the MODE name
  public String sample;         // short name of sample
  public String orient;         // orientation of sample, field, mu pol.
  public String das;            // data acquisition system [MODAS]
  public String experimenter;   // who took this run of data
  public String temperature;    // sample temperature [usually K], TD only
  public String field;          // magnetic field [usually in G], TD only
  public String subtitle;       // only for TI runs
  public String comment1;       // only for TI runs
  public String comment2;       // only for TI runs
  public String comment3;       // only for TI runs

/* run header print method */
  public void print () {
    Format.print(System.out, "\nfmtID=0x%08x", fmtID);
    if ( runNumber > 0) System.out.print ( "\t runNumber "+runNumber );
    if ( title != null) System.out.print ( "\ntitle \""+title+"\"" );
    if ( nHists > 0) System.out.print ( "\t nHists: "+nHists );
    if ( subtitle != null) System.out.print ( "\n "+subtitle );
    if ( comment1 != null) System.out.print ( "\n "+comment1 );
    if ( comment2 != null) System.out.print ( "\n "+comment2 );
    if ( comment3 != null) System.out.print ( "\n "+comment3 );
    if ( lab != null) System.out.print ( "\nlab \""+lab+"\"" );
    if ( exptNumber > 0) System.out.print ( "\t exptNumber "+exptNumber );
    if ( experimenter != null) System.out.print ( "\t experimenter \""+experimenter+"\"" );
    if ( sample != null) System.out.print ( "\nsample \""+sample+"\"" );
    if ( orient != null) System.out.print ( ", orient \""+orient+"\"" );
    if ( temperature != null) System.out.print ( ", temperature \""+temperature+"\"" );
    if ( field != null) System.out.print ( ", field \""+field+"\"" );
    if ( timeEnd > 0) System.out.print ( "\n(timeEnd:"+timeEnd+") - " );
    if ( timeBegin > 0) System.out.print ( "(timeBegin:" +timeBegin+") = " );
    if ( elapsedSec > 0) System.out.print ( "(elapsedSec:"+elapsedSec+")" );
    if ( method != null) System.out.print ( "\nmethod \""+method+"\"" );
    if ( das != null) System.out.print ( "\t das \""+das+"\"" );
    if ( area != null) System.out.print ( "\narea \""+area+"\"" );
    if ( apparatus != null) System.out.print ( "\t apparatus \""+apparatus+"\"" );
    if ( insert != null) System.out.print ( "\t insert \""+insert+"\"" );
    if ( nSclrs > 0) System.out.print ( "\t nSclrs: "+nSclrs+"\n");
    if ( nIndVar > 0) System.out.print ( "\t nIndVar: "+nIndVar+"\n");
  }

/* run header prettyPrint method */
  public void prettyPrint () {
    Date StartTime = new Date( (long) 1000*timeBegin );
    int Year = Calendar.YEAR;
    if (Year < 71) { Year = 2000 + Year; }
    else { Year = 1900 + Year; } // Our very own Y2K bug!
    Date EndTime = new Date( (long) 1000*timeEnd );
    System.out.print ( "\n\n "+lab );
    System.out.print ( " Experiment # "+exptNumber );
    System.out.print ( "\t["+experimenter+"]" );
    System.out.print ( "\ton "+area );
    System.out.print ( "\tin "+Year+":" );
    //    Format.print(System.out, "\nfmtID=0x%08x", fmtID);
    System.out.print ( "\n\n"+das );
    System.out.print ( "\t"+method );
    System.out.print ( "\t Run # "+runNumber );
    System.out.print ( "\t ["+nHists+" Histograms" );
    if (nSclrs > 0 ) System.out.print ( ", "+nSclrs+" Scalers");
    if ( nIndVar > 0 ) System.out.print ( ", "+nIndVar+" Indep. Var.");
    System.out.print ( "]" );
    System.out.print ( "\n\n \""+title+"\"\n" );
    if ( subtitle != null) System.out.print ( "\n "+subtitle );
    if ( comment1 != null) System.out.print ( "\n "+comment1 );
    if ( comment2 != null) System.out.print ( "\n "+comment2 );
    if ( comment3 != null) System.out.print ( "\n "+comment3 );
    System.out.print ( "\n" );
    if ( sample != null ) System.out.print ( "Sample: \""+sample+"\"  " );
    if ( orient != null ) System.out.print ( "Orientation: \""+orient+"\"  " );
    if ( temperature != null)
      System.out.print ( "Temperature = "+temperature+"  " );
    if ( field != null) System.out.print ( "Field = "+field );
    System.out.print ( "\nStarted "+StartTime );
    System.out.print ( ".\tEnded "+EndTime );
    System.out.print ( "\nElapsed: "+elapsedSec+" s.  " );
    System.out.print ( "\tApparatus = \""+apparatus+"\"" );
    System.out.print ( "\tInsert = \""+insert+"\"" );
    System.out.print ( "\n" );
  }

/* run header flexdb (special format printout) method */
  public void flexdb () {
    Date StartTime = new Date( (long) 1000*timeBegin );
    int Year = Calendar.YEAR;
    if (Year < 71) { Year = 2000 + Year; }
    else { Year = 1900 + Year; } // Our very own Y2K bug!
    Date EndTime = new Date( (long) 1000*timeEnd );
    System.out.print("run="+runNumber+"\n");
    System.out.print("title="+title+"\n");
    System.out.print("expt="+exptNumber+"\n");
    System.out.print("year="+Year+"\n");
    System.out.print("timeBegin="+StartTime+"\n");
    System.out.print("timeEnd="+EndTime+"\n");
    System.out.print("elapsed="+elapsedSec+"\n");
    System.out.print("lab="+lab+"\n");
    System.out.print("area="+area+"\n");
    System.out.print("method="+method+"\n");
    System.out.print("apparatus="+apparatus+"\n");
    System.out.print("insert="+insert+"\n");
    System.out.print("sample="+sample+"\n");
    System.out.print("orient="+orient+"\n");
    System.out.print("das="+das+"\n");
    System.out.print("experimenter="+experimenter+"\n");
    System.out.print("temperature="+temperature+"\n");
    System.out.print("field="+field+"\n");
    if (subtitle != null) {
      System.out.print("subtitle="+subtitle+"\n");
    }
    if (comment1 != null) {
      System.out.print("comment1="+comment1+"\n");
    }
    if (comment2 != null) {
      System.out.print("comment2="+comment2+"\n");
    }
    if (comment3 != null) {
      System.out.print("comment3="+comment3+"\n");
    }
    System.out.print ("nHists="+nHists+"\n");
    System.out.print ("nIndVar="+nIndVar+"\n");
  }

/* run header csv (special format printout) method */
  public void csv () {
    System.out.print("\"[RUN]\", ");
    Date StartTime = new Date( (long) 1000*timeBegin );
    int Year = Calendar.YEAR;
    if (Year < 71) { Year = 2000 + Year; }
    else { Year = 1900 + Year; } // Our very own Y2K bug!
    Date EndTime = new Date( (long) 1000*timeEnd );
    System.out.print("\""+runNumber+"\",");
    System.out.print("\""+title+"\",");
    System.out.print("\""+exptNumber+"\",");
    System.out.print("\""+Year+"\",");
    System.out.print("\""+StartTime+"\",");
    System.out.print("\""+EndTime+"\",");
    System.out.print("\""+elapsedSec+"\",");
    System.out.print("\""+lab+"\",");
    System.out.print("\""+area+"\",");
    System.out.print("\""+method+"\",");
    System.out.print("\""+apparatus+"\",");
    System.out.print("\""+insert+"\",");
    System.out.print("\""+sample+"\",");
    System.out.print("\""+orient+"\",");
    System.out.print("\""+das+"\",");
    System.out.print("\""+experimenter+"\",");
    System.out.print("\""+temperature+"\",");
    System.out.print("\""+field+"\",");
    System.out.print("\""+subtitle+"\",");
    System.out.print("\""+comment1+"\",");
    System.out.print("\""+comment2+"\",");
    System.out.print("\""+comment3+"\",");
    System.out.print ("\""+nHists+"\",");
    System.out.print ("\""+nIndVar+"\"\n");
  }

/* run header sql (special format printout) method */
  public void sql () {
    Date StartTime = new Date( (long) 1000*timeBegin );
    int Year = Calendar.YEAR;
    if (Year < 71) { Year = 2000 + Year; }
    else { Year = 1900 + Year; } // Our very own Y2K bug!
    Date EndTime = new Date( (long) 1000*timeEnd );
    System.out.print("DELETE FROM runs WHERE");
    System.out.print(" lab=\""+lab+"\"");
    System.out.print(" AND area=\""+area+"\"");
    System.out.print(" AND year="+Year);
    System.out.print(" AND expt=\""+exptNumber+"\"");
    System.out.print(" AND run="+runNumber+";\n");
//    System.out.print(" AND title=\""+title+"\";\n");
    System.out.print("INSERT INTO runs SET ");
    System.out.print("lab=\""+lab+"\"");
    System.out.print(",area=\""+area+"\"");
    System.out.print(",year="+Year);
    System.out.print(",expt=\""+exptNumber+"\"");
    System.out.print(",run="+runNumber);
    System.out.print(",title=\""+title+"\"");
    System.out.print(",method=\""+method+"\"");
    if ( nHists > 0 ) System.out.print(",nH="+nHists);
    if ( elapsedSec > 0 ) System.out.print(",elaps="+elapsedSec);
    if ( nIndVar > 0 ) System.out.print(",nIV="+nIndVar);
    if ( experimenter != null ) System.out.print(",oper=\""+experimenter+"\"");
    if ( apparatus != null ) System.out.print(",rig=\""+apparatus+"\"");
    if ( insert != null ) System.out.print(",mode=\""+insert+"\"");
    if ( sample != null ) System.out.print(",sample=\""+sample+"\"");
    if ( orient != null ) System.out.print(",orient=\""+orient+"\"");
    if ( temperature != null ) System.out.print(",temp=\""+temperature+"\"");
    if ( field != null ) System.out.print(",field=\""+field+"\"");
//    if ( method == "TI-SR" ) {
/*    if (fmtID == 0x02020000) {
      if ( subtitle != "no subtitle yet" && subtitle != null )
	  System.out.print(",subttl=\""+subtitle+"\"");
      if ( comment1 != null ) System.out.print(",cmnt1=\""+comment1+"\"");
      if ( comment2 != "no comment" && comment2 != null )
	  System.out.print(",cmnt2=\""+comment2+"\"");
      if ( comment3 != "no comment" && comment3 != null )
	  System.out.print(",cmnt3=\""+comment3+"\"");
    }
*/
    System.out.print (";\n");
/*
    System.out.print(",?=\""+StartTime+"\"");
    System.out.print(",?=\""+EndTime+"\"");
    System.out.print(",?=\""+das+"\"");
*/
  }

/* run header updsql (special format printout) method */
  public void updsql () {
    Date StartTime = new Date( (long) 1000*timeBegin );
    int Year = Calendar.YEAR;
    if (Year < 71) { Year = 2000 + Year; }
    else { Year = 1900 + Year; } // Our very own Y2K bug!
    Date EndTime = new Date( (long) 1000*timeEnd );
    //    if ( method == "TI-?SR" ) {
    if (fmtID == 0x02020000) {
	System.out.print("DELETE FROM runs WHERE");
	System.out.print(" lab=\""+lab+"\"");
	System.out.print(" AND area=\""+area+"\"");
	System.out.print(" AND year="+Year);
	System.out.print(" AND expt=\""+exptNumber+"\"");
	System.out.print(" AND run="+runNumber+";\n");
	System.out.print("INSERT INTO runs SET ");
	System.out.print("lab=\""+lab+"\"");
	System.out.print(",area=\""+area+"\"");
	System.out.print(",year="+Year);
	System.out.print(",expt=\""+exptNumber+"\"");
	System.out.print(",run="+runNumber);
	System.out.print(",title=\""+title+"\"");
	System.out.print(",method=\""+method+"\"");
	if ( nHists > 0 ) System.out.print(",nH="+nHists);
	if ( elapsedSec > 0 ) System.out.print(",elaps="+elapsedSec);
	if ( nIndVar > 0 ) System.out.print(",nIV="+nIndVar);
	if ( experimenter != null ) System.out.print(",oper=\""+experimenter+"\"");
	if ( apparatus != null ) System.out.print(",rig=\""+apparatus+"\"");
	if ( insert != null ) System.out.print(",mode=\""+insert+"\"");
	if ( sample != null ) System.out.print(",sample=\""+sample+"\"");
	if ( orient != null ) System.out.print(",orient=\""+orient+"\"");
	if ( temperature != null ) System.out.print(",temp=\""+temperature+"\"");
	if ( field != null ) System.out.print(",field=\""+field+"\"");
/*	if ( subtitle != "no subtitle yet" && subtitle != null )
	    System.out.print(",subttl=\""+subtitle+"\"");
	if ( comment1 != null ) System.out.print(",cmnt1=\""+comment1+"\"");
	if ( comment2 != "no comment" && comment2 != null )
	    System.out.print(",cmnt2=\""+comment2+"\"");
	if ( comment3 != "no comment" && comment3 != null )
	    System.out.print(",cmnt3=\""+comment3+"\"");
*/
	System.out.print (";\n");
    }
    else if (fmtID == 0x02010000) {
	System.out.print("UPDATE runs SET ");
	System.out.print("title=\""+title+"\"");
	System.out.print(",method=\""+method+"\"");
	if ( nHists > 0 ) System.out.print(",nH="+nHists);
	if ( elapsedSec > 0 ) System.out.print(",elaps="+elapsedSec);
	if ( nIndVar > 0 ) System.out.print(",nIV="+nIndVar);
	if ( experimenter != null ) System.out.print(",oper=\""+experimenter+"\"");
	if ( apparatus != null ) System.out.print(",rig=\""+apparatus+"\"");
	if ( insert != null ) System.out.print(",mode=\""+insert+"\"");
	if ( sample != null ) System.out.print(",sample=\""+sample+"\"");
	if ( orient != null ) System.out.print(",orient=\""+orient+"\"");
	if ( temperature != null ) System.out.print(",temp=\""+temperature+"\"");
	if ( field != null ) System.out.print(",field=\""+field+"\"");
	System.out.print(" WHERE");
	System.out.print(" lab=\""+lab+"\"");
	System.out.print(" AND area=\""+area+"\"");
	System.out.print(" AND year="+Year);
	System.out.print(" AND expt=\""+exptNumber+"\"");
	System.out.print(" AND run="+runNumber);
	System.out.print (";\n");
    }
/*
    System.out.print(",?=\""+StartTime+"\"");
    System.out.print(",?=\""+EndTime+"\"");
    System.out.print(",?=\""+das+"\"");
*/
  }

}
